/*=j2s=
#Java2Script Configuration
#Sun Jan 08 15:50:30 CST 2006
j2s.resources.list=
=*/
/*******************************************************************************
 * Copyright (c) 2005 ognize.com and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Web:
 *     http://j2s.sf.net/
 *     http://sourceforge.net/projects/j2s/
 * Contributors:
 *     ognize.com - initial API and implementation
 *******************************************************************************/
/*******
 * Copyright ognize.com (2005).
 *
 * Test cases for Script Oriented Web Architecture
 *
 * @author josson smith
 * @create Oct 27, 2005
 *******/
 
/**
 * Class Clazz. All the methods are static in this class.
 */
/* static */
function Clazz () {
};

function NullObject () {
};

/**
 * Return the class name of the given class or object.
 *
 * @param clazzHost given class or object
 * @return class name
 */
/* public */
Clazz.getClassName = function (clazzHost) {
	if (clazzHost == null) {
		/* 
		 * null is always treated as Object.
		 * But what about "undefined"?
		 */
		return "NullObject";
	}
	if (typeof clazzHost == "function") {
		var clazz = clazzHost;
		if (clazz.__CLASS_NAME__ != null) {
			/* user defined class name */
			return clazz.__CLASS_NAME__;
		}
		var clazzStr = clazz.toString ();
		var idx1 = clazzStr.indexOf ("function") + 8;
		var idx2 = clazzStr.indexOf ("(", idx1);
		var clazzName = clazzStr.substring (idx1, idx2)
				.replace (/^\s+/, "").replace (/\s+$/, ""); // .trim ()
		if (clazzName == "anonymous") {
			return "Function";
		} else {
			return clazzName;
		}
	} else {
		var obj = clazzHost;
		if (obj instanceof Clazz.CastedNull) {
			return obj.clazzName;
		} else {
			var objType = typeof obj;
			if (objType == "string") {
				/* 
				 * Always treat the constant string as String object.
				 * This will be compatiable with Java String instance.
				 */
				return "String";
			} else if (objType == "object") {
				if (obj.__CLASS_NAME__ != null) {
					/* user defined class name */
					return obj.__CLASS_NAME__;
				} else {
					return Clazz.getClassName (obj.constructor);
				}
			}
			return Clazz.getClassName (obj.constructor);
		}
	}
};
/**
 * Return the class of the given class or object.
 *
 * @param clazzHost given class or object
 * @return class name
 */
/* public */
Clazz.getClass = function (clazzHost) {
	if (clazzHost == null) {
		/* 
		 * null is always treated as Object.
		 * But what about "undefined"?
		 */
		return Object;
	}
	if (typeof clazzHost == "function") {
		return clazzHost;
	} else {
		var clazzName = null;
		var obj = clazzHost;
		if (obj instanceof Clazz.CastedNull) {
			clazzName = obj.clazzName;
		} else {
			if (typeof obj == "object") {
				/* user defined class name */
				if (obj.__CLASS_NAME__ != null) {
					clazzName = obj.__CLASS_NAME__;
				} else {
					return obj.constructor;
				}
			}
		}
		if (clazzName != null) {
			var hostedClazz = null;
			eval ("hostedClazz = " + clazzName + ";");
			return hostedClazz;
		} else {
			return obj.constructor;
		}
	}
};

/*
 * Be used to copy members of class
 */
/* protected */
Clazz.extendsProperties = function (hostThis, hostSuper) {
	for (var o in hostSuper) {
		if (o != "prototype" && o != "superClazz"
				&& o != "__CLASS_NAME__" && o != "implementz") {
			hostThis[o] = hostSuper[o];
		}
	}
};

/*
 * Be used to copy members of interface
 */
/* protected */
Clazz.implementsProperties = function (hostThis, hostSuper) {
	for (var o in hostSuper) {
		if (o != "prototype" && o != "superClazz"
				&& o != "__CLASS_NAME__" && o != "implementz") {
			hostThis[o] = hostSuper[o];
			hostThis.prototype[o] = hostSuper[o];
		}
	}
	for (var o in hostSuper.prototype) {
		if (o != "__CLASS_NAME__") {
			hostThis.prototype[o] = hostSuper.prototype[o];
		}
	}
};

Clazz.args4InheritClass = function () {
};
Clazz.inheritArgs = new Clazz.args4InheritClass ();

/**
 * Inherit class with "extends" keyword and also copy those static members. 
 * Example, as in Java, if NAME is a static member of ClassA, and ClassB 
 * extends ClassA then ClassB.NAME can be accessed in some ways.
 *
 * @param clazzThis child class to be extended
 * @param clazzSuper super class which is inherited from
 * @param objSuper super class instance
 */
/* public */
Clazz.inheritClass = function (clazzThis, clazzSuper, objSuper) {
	var thisClassName = Clazz.getClassName (clazzThis);
	Clazz.extendsProperties (clazzThis, clazzSuper);
	if (objSuper != null) {
		clazzThis.prototype = objSuper;
	} else {
		clazzThis.prototype = new clazzSuper (Clazz.inheritArgs);
	}
	clazzThis.superClazz = clazzSuper;
	clazzThis.__CLASS_NAME__ = thisClassName;
	clazzThis.prototype.__CLASS_NAME__ = thisClassName;
};

/**
 * Implementation of Java's keyword "implements".
 * As in JavaScript there are on "implements" keyword implemented, a property
 * of "implementz" is added to the class to record the interfaces the class
 * is implemented.
 * 
 * @param clazzThis the class to implement
 * @param interfacez Array of interfaces
 */
/* public */
Clazz.implementOf = function (clazzThis, interfacez) {
	if (arguments.length >= 2) {
		if (clazzThis.implementz == null) {
			clazzThis.implementz = new Array ();
		}
		var impls = clazzThis.implementz;
		if (arguments.length == 2) {
			if (typeof interfacez == "function") {
				impls[impls.length] = interfacez;
				Clazz.implementsProperties (clazzThis, interfacez);
			} else if (interfacez instanceof Array) {
				impls.concat (interfacez);
				for (var i = 0; i < interfacez.length; i++) {
					Clazz.implementsProperties (clazzThis, interfacez[i]);
				}
			}
		} else {
			for (var i = 1; i < arguments.length; i++) {
				impls[impls.length] = arguments[i];
				Clazz.implementsProperties (clazzThis, arguments[i]);
			}
		}
	}
};

/**
 * TODO: More should be done for interface's inheritance
 */
/* public */
Clazz.extendInterface = Clazz.implementOf;

/* protected */
Clazz.equalsOrExtendsLevel = function (clazzThis, clazzAncestor) {
	if (clazzThis == clazzAncestor) {
		return 0;
	}
	if (clazzThis.implementz != null) {
		var impls = clazzThis.implementz;
		for (var i = 0; i < impls.length; i++) {
			var level = Clazz.equalsOrExtendsLevel (impls[i], clazzAncestor);
			if (level >= 0) {
				return level + 1;
			}
		}
	}
	return -1;
};

/* protected */
Clazz.getClassNameEvalStr = function (clazzVarName, clazzName) {
	var innerTypes = new Array (
		"number", "string", "function", "object", "array", "boolean"
	);
	for (var i = 0; i < innerTypes.length; i++) {
		if (innerTypes[i] == clazzName) {
			return clazzVarName + " = " 
					+ clazzName.substring (0, 1).toUpperCase ()
					+ clazzName.substring (1) + ";";
		}
	}
	return clazzVarName + " = " + clazzName + ";";
};
/* protected */
Clazz.getInheritedLevel = function (clazzTarget, clazzBase) {
	if (clazzTarget == clazzBase) {
		return 0;
	}
	if ((typeof clazzTarget == "string" && "void" == clazzTarget) 
			|| (typeof clazzBase == "string" && "void" == clazzBase)) {
		return -1;
	}
	if ((typeof clazzTarget == "string" && "unknown" == clazzTarget) 
			|| (typeof clazzBase == "string" && "unknown" == clazzBase)) {
		return -1;
	}
	if ((typeof clazzTarget == "string" && "NullObject" == clazzTarget) 
			|| NullObject == clazzTarget) {
		if (clazzBase != Number && clazzBase != Boolean
				&& clazzBase != NullObject) {
			return 0;
		}
	}
	if (typeof clazzTarget == "string") {
		eval (Clazz.getClassNameEvalStr ("clazzTarget", clazzTarget));
	}
	if (typeof clazzBase == "string") {
		eval (Clazz.getClassNameEvalStr ("clazzBase", clazzBase));
	}
	var level = 0;
	var zzalc = clazzTarget;
	while (zzalc != clazzBase && level < 10) {
		/* maybe clazzBase is interface */
		if (zzalc.implementz != null) {
			var impls = zzalc.implementz;
			for (var i = 0; i < impls.length; i++) {
				var implsLevel = Clazz.equalsOrExtendsLevel (impls[i], clazzBase);
				if (implsLevel >= 0) {
					return level + implsLevel + 1;
				}
			}
		}
		
		zzalc = zzalc.superClazz;
		if (zzalc == null) {
			if (clazzBase == Object) {
				return level + 1;
			} else {
				return -1;
			}
		}
		level++;
	}
	return level;
};

/**
 * Implements Java's keyword "instanceof" in JavaScript's way.
 * As in JavaScript part of the object inheritance is implemented
 * in only-JavaScript way.
 *
 * @param obj the object to be tested
 * @param clazz the class to be checked
 * @return whether the object is an instance of the class
 */
/* public */
Clazz.instanceOf = function (obj, clazz) {
	if (obj == null) {
		return clazz == undefined; // should usually false
	}
	if (clazz == null) {
		return false;
	}
	if (obj instanceof clazz) {
		return true;
	} else {
		/*
		 * To check all the inherited interfaces.
		 */
		var clazzName = Clazz.getClassName (obj);
		return Clazz.getInheritedLevel (clazzName, clazz) >= 0;
	}
};

/**
 * Call super method of the class. 
 * The same effect as Java's expression:
 * <code> super.* () </code>
 * 
 * @param objThis host object
 * @param clazzThis host object's class. Can not get the class me by host 
 * object. Because the clazzThis is used as one signal for the JavaScript
 * to determine which class is the super.* method calling.
 * @param funName method name to be called
 * @param funParams Array of method parameters
 */
/* public */
Clazz.superCall = function (objThis, clazzThis, funName, funParams) {
	var params = Clazz.getParamsType (funParams);
	if (objThis[funName] == null) {
		if ("construct" == funName) {
			/* No super constructor */
			return ;
		}
		throw new MethodNotFoundException (objThis, clazzThis, 
				"super." + funName, params.typeString);
	}
	if (objThis[funName].claxxOwner != null) {
		if (objThis[funName].claxxOwner == clazzThis) {
			if ("construct" == funName) {
				/* No super constructor */
				return ;
			}
			throw new MethodNotFoundException (objThis, clazzThis, 
					"super." + funName, params.typeString);
		}
		/*
		 * This is a single method, call directly!
		 */
		var callException;
		if (funParams == null) {
			try {
				returnObj = objThis[funName]
						.apply (objThis, []);
			} catch (e) {
				callException = e;
			}
		} else {
			try {
				returnObj = objThis[funName]
						.apply (objThis, funParams);
			} catch (e) {
				callException = e;
			}
		}
		if (callException != null) {
			throw callException;
		}
		return returnObj;
	}
	Clazz.prepareForSuperCall (objThis, clazzThis);
	var length = objThis[funName].stacks.length;
	var returnObj;
	var callException;
	var isReturned = false;
	for (var i = 0; i < length; i++) {
		if (objThis[funName].stacks[i] == clazzThis) {
			if (i > 0) {
				var clazzParent = objThis[funName].stacks[i - 1];
				if (funParams == null) {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, []);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				} else {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, funParams);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				}
			} else {
				var clazzParent = objThis[funName].stacks[0];
				var unknownFun = clazzParent.prototype[funName]["\\unknown"];
				if (unknownFun != null) {
					if (funParams == null) {
						try {
							returnObj = unknownFun.apply (objThis, []);
						} catch (e) {
							callException = e;
						}
						isReturned = true;
						break;
					} else {
						try {
							returnObj = unknownFun.apply (objThis, funParams);
						} catch (e) {
							callException = e;
						}
						isReturned = true;
						break;
					}
				}
			}
			break;
		}
	}
	var lastLevel;
	if (!isReturned) {
		for (var i = length - 1; i >= 0; i--) {
			var level = Clazz.getInheritedLevel (clazzThis, 
					objThis[funName].stacks[i]);
			lastLevel = level;
			if (level > 0) { /* once ">" was ">=" */
				var clazzParent = objThis[funName].stacks[i];
				if (funParams == null) {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, []);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				} else {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, funParams);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				}
			}
		}
	}
	Clazz.restoreFromSuperCall (objThis, clazzThis);
	if (isReturned) {
		if (callException != null) {
			throw callException;
		}
		return returnObj;
	}
	if (/*lastLevel == 0 && */funName == "construct") {
		/* No super constructor! */
		return ;
	}
	throw MethodNotFoundException (objThis, clazzThis, 
			funName, params.typeString);
};

/**
 * Call super constructor of the class. 
 * The same effect as Java's expression: 
 * <code> super () </code>
 */
/* public */
Clazz.superConstructor = function (objThis, clazzThis, funParams) {
	Clazz.superCall (objThis, clazzThis, "construct", funParams);
};

/**
 * Class for null with a given class as to be casted.
 * This class will be used as an implementation of Java's casting way.
 * For example,
 * <code> this.call ((String) null); </code>
 */
/* protcted */
Clazz.CastedNull = function (asClazz) {
	if (asClazz != null) {
		if (asClazz instanceof String) {
			this.clazzName = asClazz;
		} else if (asClazz instanceof Function) {
			this.clazzName = Clazz.getClassName (asClazz);
		} else {
			this.clazzName = "" + asClazz;
		}
	} else {
		this.clazzName = "Object";
	}
	this.toString = function () {
		return null;
	};
	this.valueOf = function () {
		return null;
	};
};

/**
 * API for Java's casting null.
 * @see Clazz.CastedNull
 *
 * @param asClazz given class
 * @return an instance of class Clazz.CastedNull
 */
/* public */
Clazz.castNullAs = function (asClazz) {
	return new Clazz.CastedNull (asClazz);
};

/** 
 * MethodException will be used as a signal to notify that the method is
 * not found in the current clazz hierarchy.
 */
/* private */
Clazz.MethodException = function () {
	/*
	this.message = "The static Clazz instance can not found the method!";
	this.toString = function () {
		return this.message;
	};
	*/
};

/* private */
Clazz.getParamsType = function (funParams) {
	var params = new Array ();
	var containsCastedNullParams = false;
	if (funParams != null) {
		for (var i = 0; i < funParams.length; i++) {
			params[i] = Clazz.getClassName (funParams[i]);
			if (funParams[i] instanceof Clazz.CastedNull) {
				containsCastedNullParams = true;
			}
		}
	}
	if (params.length == 0) {
		params[0] = "void";
	}
	params.typeString = "\\" + params.join ('\\');
	params.containsCastedNullParams = containsCastedNullParams;
	return params;
};
/**
 * Search the given class prototype, find the method with the same
 * method name and the same parameter signatures with the given 
 * parameters, and then run the method with the given parameters.
 *
 * @param objThis the current host object
 * @param claxxRef the current host object's class
 * @param fxName the method name
 * @param funParams the given arguments
 * @return the result of the specified method of the host object,
 * the return maybe void.
 * @throws MethodNotFoundException if no matched method is found
 */
/* protected */
Clazz.searchAndRunMethod = function (objThis, claxxRef, fxName, funParams) {
	var params = Clazz.getParamsType (funParams);
	/*
	 * Search the inheritance stacks to get the given class' function
	 */
	if (objThis[fxName].stacks == null) {
		log (fxName);
		log (claxxRef.getName ());
		log (objThis);
		log (objThis[fxName]);
		log ("error");
	}
	var length = objThis[fxName].stacks.length;
	for (var i = 0; i < length; i++) {
		if (objThis[fxName].stacks[i] == claxxRef) {
			var clazzFun = objThis[fxName].stacks[i].prototype[fxName];
			if (clazzFun == null) {
				/*
				 * Should try to call its super methods before throwing out
				 * exception.
				 */
				throw new MethodNotFoundException (objThis, claxxRef, 
						fxName, params.typeString);
			}
			try {
				return Clazz.tryToSearchAndRun (objThis, clazzFun, params, 
						funParams);
			} catch (e) {
				if (!(e instanceof Clazz.MethodException)) {
					throw e;
				}
			}
			/*
			 * Try to call its super methods before throwing out exception.
			 */
			break;
		}
	}
	for (var i = length - 1; i >= 0; i--) {
		if (Clazz.getInheritedLevel (claxxRef, 
				objThis[fxName].stacks[i]) >= 0) {
			var clazzFun = objThis[fxName].stacks[i].prototype[fxName];
			if (clazzFun == null) {
				/*
				 * Should try to call its super methods before throwing out
				 * exception.
				 */
				throw new MethodNotFoundException (objThis, claxxRef, 
						fxName, params.typeString);
			}
			try {
				return Clazz.tryToSearchAndRun (objThis, clazzFun, params, 
						funParams);
			} catch (e) {
				if (!(e instanceof Clazz.MethodException)) {
					throw e;
				}
			}
			/*
			 * Try to call its super methods before throwing out exception.
			 */
			continue;
		}
	}
	if ("construct" == fxName) {
		return ;
	}
	throw new MethodNotFoundException (objThis, claxxRef, 
			fxName, params.typeString);
};

/* private */
Clazz.tryToSearchAndRun = function (objThis, clazzFun, params, funParams) {
	var methodSignatures = new Array ();
	for (var fn in clazzFun) {
		if (fn.indexOf ('\\') == 0) {
			methodSignatures[methodSignatures.length] = fn.substring (1);
		}
	}
	var method = Clazz.searchMethod (methodSignatures, params);
	if (method != null) {
		var fn = "\\" + method;
		if (clazzFun[fn] != null) {
			if (params.containsCastedNullParams) {
				var methodParams = new Array ();
				for (var k = 0; k < funParams.length; k++) {
					if (funParams[k] instanceof Clazz.CastedNull) {
						methodParams[k] = null;
					} else {
						methodParams[k] = funParams[k];
					}
				}
				return clazzFun[fn].apply (objThis, methodParams);
			} else {
				return clazzFun[fn].apply (objThis, funParams);
			}
		}
	}
	throw new Clazz.MethodException ();
};
/**
 * Search the existed polynomial methods to get the matched method with
 * the given parameter types.
 *
 * @param existedMethods Array of string which contains method parameters
 * @param paramTypes Array of string that is parameter type.
 * @return string of method parameters seperated by "\\"
 */
/* private */
Clazz.searchMethod = function (existedMethods, paramTypes) {
	var roundOne = new Array ();
	for (var i = 0; i < existedMethods.length; i++) {
		var split = existedMethods[i].split (/\\/);
		if (split.length == paramTypes.length) {
			roundOne[roundOne.length] = split;
		}
	}
	var resultOne = roundOne;
	var roundTwo = new Array ();
	for (var i = 0; i < resultOne.length; i++) {
		var fittedLevel = new Array ();
		var isFitted = true;
		for (var j = 0; j < resultOne[i].length; j++) {
			//fittedLevel[j] = Clazz.getInheritedLevel (paramTypes[j], 
			//		resultOne[i][j]);
			/*
			 * The following scoped codes are to implement the upper
			 * commented codes.
			 */
			var s1 = null;
			if ("void" == paramTypes[j] || "number" == paramTypes[j]
					|| "string" == paramTypes[j] 
					|| "string" == paramTypes[j] 
					|| "function" == paramTypes[j] 
					|| "object" == paramTypes[j]
					|| "unknown" == paramTypes[j]) {
				s1 = "\"" + paramTypes[j] + "\"";
			} else {
				s1 = paramTypes[j];
			}
			var s2 = null;
			if ("void" == resultOne[i][j] || "number" == resultOne[i][j]
					|| "string" == resultOne[i][j]
					|| "string" == resultOne[i][j]
					|| "function" == resultOne[i][j]
					|| "object" == resultOne[i][j]
					|| "unknown" == resultOne[i][j]) {
				s2 = "\"" + resultOne[i][j] + "\"";
			} else {
				s2 = resultOne[i][j];
			}
			fittedLevel[j] = eval ("Clazz.getInheritedLevel (" + s1 + ", " 
					+ s2 + ");");
			if (fittedLevel[j] < 0) {
				isFitted = false;
				break;
			}
		}
		if (isFitted) {
			fittedLevel[paramTypes.length] = i;
			roundTwo[roundTwo.length] = fittedLevel;
		}
	}
	if (roundTwo.length == 0) {
		return null;
	}
	var resultTwo = roundTwo;
	var min = resultTwo[0];
	for (var i = 1; i < resultTwo.length; i++) {
		var isVectorLesser = true;
		for (var j = 0; j < paramTypes.length; j++) {
			if (min[j] < resultTwo[i][j]) {
				isVectorLesser = false;;
				break;
			}
		}
		if (isVectorLesser) {
			min = resultTwo[i];
		}
	}
	var index = min[paramTypes.length];
	var params = resultOne[index];
	var methodParams = "";
	for (var i = 0; i < params.length; i++) {
		methodParams += params[i];
		if (i != params.length - 1) {
			methodParams += "\\";
		}
	}
	return methodParams;
};

/**
 * Generate delegating function for the given method name.
 *
 * @param claxxRef the specified class for the method
 * @funName method name of the specified method
 * @return the method delegate which will try to search the method
 * from the given class by the parameters
 */
/* private */
Clazz.generateDelegatingMethod = function (claxxRef, funName) {
	/*
	 * Delegating method.
	 * Each time the following expression will generate a new 
	 * function object.
	 */
	/*
	var delegating = function () {
			var classRefrence = arguments.callee.claxxRefrence;
			var methodName = arguments.callee.methodName;
			return Clazz.searchAndRunMethod (this, classRefrence, 
					methodName, arguments);
	};
	*/
	var delegating = function () {
			var r = arguments;
			return SARM (this, r.callee.claxxRefrence, r.callee.methodName, r);
	};
	delegating.methodName = funName;
	delegating.claxxRefrence = claxxRef;
	return delegating;
};

SARM = Clazz.searchAndRunMethod;

/*
 * Define method for the class with the given method name and method
 * body and method parameter signature.
 *
 * @param clazzThis host class in which the method to be defined
 * @param funName method name
 * @param funBody function object, e.g function () { ... }
 * @funParams paramether signature, e.g ["string", "number"]
 */
/* public */
Clazz.defineMethod = function (clazzThis, funName, funBody, funParams) {
	var fpName = "";
	if (funParams != null) {
		/* 
		 * If funParams is Array, funParams.toString() will
		 * also return "*,*,..." string.
		 */
		var paramStr = funParams.toString ()
				.replace (/^\s+/, "").replace (/\s+$/, ""); //.trim ()
		if (paramStr.length != 0) {
			var params = paramStr.split (/\s*,\s*/);
			fpName += "\\" + params.join ('\\');
		} else {
			fpName += "\\void";
		}
	} else {
		fpName += "\\void";
	}
	/*
	 * For method the first time is defined, just keep it rather than
	 * wrapping into deep hierarchies!
	 */
	if (clazzThis.prototype[funName] == null) {
		funBody.funParams = fpName;
		funBody.claxxOwner = clazzThis;
		clazzThis.prototype[funName] = funBody;
		return ;
	} else if (clazzThis.prototype[funName].claxxOwner == clazzThis
			&& clazzThis.prototype[funName].funParams == fpName) {
		return ;
	}
	var oldFun = null;
	var oldStacks = new Array ();
	if (clazzThis.prototype[funName] != null) {
		if (clazzThis.prototype[funName].stacks == null) {
			/* method is not defined by Clazz.defineMethod () */
			oldFun = clazzThis.prototype[funName];
		} else {
			oldStacks = clazzThis.prototype[funName].stacks;
		}
	}
	
	if (oldFun != null && oldFun.claxxOwner != null) {
		/*
		 * oldStacks should be "new Array ()";
		 */
		oldStacks[0] = oldFun.claxxOwner;
		/*
		if (oldFun.claxxOwner != clazzThis) {
			if ("releaseChild" == funName) {
				error (" in here ");
			}
			oldFun.claxxOwner.prototype[funName].stacks = oldStacks;
			oldFun.claxxOwner.prototype[funName] = Clazz
					.generateDelegatingMethod (oldFun.claxxOwner, funName);
			oldFun.claxxOwner.prototype[funName][oldFun.funParams] = oldFun;
			oldFun.claxxOwner = null;
			oldFun.funParams = null;
			oldFun = null;
		}
		//*/
	}
	/* method has not been defined yet */
	/* method is not defined by Clazz.defineMethod () */
	/* method is defined in super class */
	if (clazzThis.prototype[funName] == null 
			|| clazzThis.prototype[funName].stacks == null 
			|| clazzThis.prototype[funName].claxxRefrence != clazzThis) {
		/*
		 * Generate a new delegating method for the class
		 */
		clazzThis.prototype[funName] = Clazz
				.generateDelegatingMethod (clazzThis, funName);
		/*
		 * Keep the class inheritance stacks
		 */
		var arr = new Array ();
		for (var i = 0; i < oldStacks.length; i++) {
			arr[i] = oldStacks[i];
		}
		clazzThis.prototype[funName].stacks = arr;
	}
	var ss = clazzThis.prototype[funName].stacks;

	if (ss.length == 0/* || ss[ss.length - 1] != clazzThis*/) {
		ss[ss.length] = clazzThis;
	} else {
		var existed = false;
		for (var i = ss.length - 1; i >= 0; i--) {
			if (ss[i] == clazzThis) {
				existed = true;
				break;
			}
		}
		if (!existed) {
			ss[ss.length] = clazzThis;
		}
	}

	if (oldFun != null) {
		if (oldFun.claxxOwner == clazzThis) {
			clazzThis.prototype[funName][oldFun.funParams] = oldFun;
			oldFun.claxxOwner = null;
			oldFun.funParams = null;
		} else {
			/*
			 * The function is not defined Clazz.defineMethod ().
			 * Try to fixup the method ...
			 * As a matter of lost method information, I just suppose
			 * the method to be fixed is with void parameter!
			 */
			clazzThis.prototype[funName]["\\unknown"] = oldFun;
		}
	}
	clazzThis.prototype[funName][fpName] = funBody;
};

/**
 * Make constructor for the class with the given function body and parameters
 * signature.
 * 
 * @param clazzThis host class
 * @param funBody constructor body
 * @param funParams constructor parameters signature
 */
/* public */
Clazz.makeConstructor = function (clazzThis, funBody, funParams) {
	var funName = "construct";
	Clazz.defineMethod (clazzThis, funName, funBody, funParams);
};

/**
 * Prepare callback for instance of anonymous Class.
 * For example for the callback:
 *     this.callbacks.MyEditor.sayHello();
 *
 * @param objThis the host object for callback
 * @param classThisObj the "this" object to be hooked
 * 
 * Attention: parameters should not be null!
 */
/* protected */
Clazz.prepareCallback = function (objThis, classThisObj) {
	if (objThis != null && classThisObj != null) {
		var obs = objThis["callbacks"];
		if (obs == null) {
			obs = new Array ();
			objThis["callbacks"] = obs;
		}
		var className = Clazz.getClassName (classThisObj);
		//if (obs[className] == null) { /* == null make no sense! */
			obs[className] = classThisObj;
			var clazz = Clazz.getClass (classThisObj);
			while (clazz.superClazz != null) {
				clazz = clazz.superClazz;
				obs[Clazz.getClassName (clazz)] = classThisObj;
			}
		//}
		var cbs = classThisObj["callbacks"];
		if (cbs != null && cbs instanceof Array) {
			for (var s in cbs) {
				if (s != "length") {
					obs[s] = cbs[s];
				}
			}
		}
	}
};

/**
 * Construct instance of the given inner class.
 *
 * @param classInner given inner class, alway with name like "*$*"
 * @objThis this instance which can be used to call back.
 * @return the constructed object
 */
/* public */
Clazz.innerTypeInstance = function (clazzInner, objThis) {
	var obj = null;
	if (arguments.length == 2) {
		obj = new clazzInner (objThis);
	} else if (arguments.length == 3) {
		obj = new clazzInner (objThis, arguments[2]);
	} else if (arguments.length == 4) {
		obj = new clazzInner (objThis, arguments[2], arguments[3]);
	} else if (arguments.length == 5) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4]);
	} else if (arguments.length == 6) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5]);
	} else if (arguments.length == 7) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6]);
	} else if (arguments.length == 8) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6], arguments[7]);
	} else if (arguments.length == 9) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6], arguments[7], arguments[8]);
	} else if (arguments.length == 10) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6], arguments[7], arguments[8],
				arguments[9]);
	} else {
		/*
		 * Should construct instance manually.
		 */
		obj = new clazzInner ();
		if (obj.construct == null) {
			throw new String ("No support anonymous class constructor with " 
					+ "more than 8 parameters.");
		}
		var args = new Array ();
		for (var i = 2; i < arguments.length; i++) {
			args[i - 2] = arguments[i];
		}
		obj.construct.apply (obj, args);
	}
	Clazz.prepareCallback (obj, objThis);
	return obj;
};

/* protected */
Clazz.allPackage = new Object ();

/* public */
Clazz.declarePackage = function (pkgName) {
	if (pkgName != null && pkgName.length != 0) {
		var pkgFrags = pkgName.split (/\./);
		var pkg = null;
		for (var i = 0; i < pkgFrags.length; i++) {
			if (pkg == null) {
				if (Clazz.allPackage[pkgFrags[0]] == null) {
					Clazz.allPackage[pkgFrags[0]] = new Object ();
					eval (pkgFrags[0] + " = Clazz.allPackage[pkgFrags[0]];" );
				}
				pkg = Clazz.allPackage[pkgFrags[0]];
			} else {
				if (pkg[pkgFrags[i]] == null) {
					pkg[pkgFrags[i]] = new Object ();
				}
				pkg = pkg[pkgFrags[i]]
			}
		}
		return pkg;
	}
};

/* public */
Clazz.isClassDefined = Clazz.isDefinedClass = function (clazzName) {
	if (clazzName != null && clazzName.length != 0) {
		var pkgFrags = clazzName.split (/\./);
		var pkg = null;
		for (var i = 0; i < pkgFrags.length; i++) {
			if (pkg == null) {
				if (Clazz.allPackage[pkgFrags[0]] == null) {
					//error (clazzName + " / " + false);
					return false;
				}
				pkg = Clazz.allPackage[pkgFrags[0]];
			} else {
				if (pkg[pkgFrags[i]] == null) {
					//error (clazzName + " / " + false);
					return false;
				}
				pkg = pkg[pkgFrags[i]]
			}
		}
		//error (clazzName + " / " + (pkg != null));
		return pkg != null;
	} else {
		/* consider null or empty name as non-defined class */
		return false;
	}
};

/* protected */
Clazz.instantialize = function (objThis, args) {
	if (args != null && args.length == 1 && args[0] != null 
			&& args[0] instanceof Clazz.args4InheritClass) {
		return ;
	}
	if (objThis.construct != null) {
		objThis.construct.apply (objThis, args);
	}
};

/* protected */
Clazz.stackSuperField = function (objThis, clazzThis, field) {
	if (objThis.superFields == null) {
		objThis.superFields = new Array ();
		if (objThis instanceof Function) {
			/* static member! */
			objThis.prototype.superFields = objThis.superFields;
		}
	}
	var clazzName = Clazz.getClassName (clazzThis);
	var fieldName = clazzName + ".super." + field;
	if (objThis.superFields[fieldName] == null) {
		objThis.superFields[fieldName] = objThis[field];
	}
};

/*
 * super.width will be accessed as Clazz.superAccess (this, SuperClass, "width")?
 *
 * ATTENTION: readonly!
 */
/* public */
Clazz.superAccess = function (objThis, clazzThis, field) {
	if (objThis.superFields == null) {
		return objThis[field];
	}
	var clazzName = Clazz.getClassName (clazzThis);
	var fieldName = clazzName + ".super." + field;
	if (typeof objThis.superFields[fieldName] == "undefined") {
		return objThis[field];
	}
	return objThis.superFields[fieldName];
};

/*
 * See more in Clazz#superCall
 */
/* protected */
Clazz.prepareForSuperCall = function (objThis, clazzThis) {
	if (objThis.superFields == null) {
		return ;
	}
	var stackedFields = new Array ();
	for (var f in objThis.superFields) {
		var idx = f.lastIndexOf (".super.");
		if (idx != -1) {
			var field = f.substring (idx + 7); // 7 = ".super.".length
			stackedFields[stackedFields.length] = field;
		}
	}
	if (stackedFields.length == 0) {
		return ;
	}
	var clazzName = Clazz.getClassName (clazzThis);
	for (var i = 0; i < stackedFields.length; i++) {
		var field = stackedFields[i];
		var fieldName = clazzName + ".this." + field;
		objThis.superFields[fieldName] = objThis[field];
		
		fieldName = clazzName + ".super." + field;
		objThis[field] = objThis.superFields[fieldName];
	}
};

/*
 * See more in Clazz#superCall
 */
/* protected */
Clazz.restoreFromSuperCall = function (objThis, clazzThis) {
	if (objThis.superFields == null) {
		return ;
	}
	var stackedFields = new Array ();
	for (var f in objThis.superFields) {
		var idx = f.lastIndexOf (".this.");
		if (idx != -1) {
			var field = f.substring (idx + 6); // 6 = ".this.".length
			stackedFields[stackedFields.length] = field;
		}
	}
	if (stackedFields.length == 0) {
		return ;
	}
	var clazzName = Clazz.getClassName (clazzThis);
	for (var i = 0; i < stackedFields.length; i++) {
		var field = stackedFields[i];
		var fieldName = clazzName + ".super." + field;
		/* restore the value in super scope */
		objThis.superFields[fieldName] = objThis[field];
		
		fieldName = clazzName + ".this." + field;
		objThis[field] = objThis.superFields[fieldName];
		delete objThis.superFields[fieldName];
	}
};

Clazz.newArray  = function () {
	var args = arguments;
	if (arguments.length == 1) {
		if (arguments[0] instanceof Array) {
			args = arguments[0];
		}
	}
	if (args.length <= 1) {
		return new Array ();
	} else if (args.length == 2) {
		var dim = args[0];
		var val = args[1];
		var arr = new Array (dim);
		for (var i = 0; i < dim; i++) {
			arr[i] = val;
		}
		return arr;
	} else {
		var dim = args[0];
		var len = args.length - 1;
		var xargs = new Array (len);
		for (var i = 0; i < len; i++) {
			xargs[i] = args[i + 1];
		}
		var arr = new Array (dim);
		for (var i = 0; i < dim; i++) {
			arr[i] = Clazz.newArray (xargs);
		}
		return arr;
	}
};

Clazz.makeFunction = function (jsr) {
	return function () {
		jsr.run ();
		if (jsr.returnSet == 1) {
			return jsr.returnNumber;
		} else if (jsr.returnSet == 2) {
			return jsr.returnBoolean;
		} else if (jsr.returnSet == 3) {
			return jsr.returnObject;
		}
	};
};

Function.prototype.equals = function (aFun) {
	return this.prototype.constructor == aFun;
};

Function.prototype.getName = function () {
	return Clazz.getClassName (this);
};

Clazz.declarePackage ("java.lang");
java.lang.Object = Object;

Object.prototype.equals = function (obj) {
	return this == obj;
};

Object.prototype.hashCode = function () {
	return this.toString ().hashCode ();
};

Object.prototype.getClass = function () {
	return Clazz.getClass (this);
};

Object.prototype.clone = function () {
	return this;
};

Object.prototype.finalize = function () {
};

Object.prototype.notify = function () {
};

Object.prototype.notifyAll = function () {
};

Object.prototype.wait = function () {
};
