/*=j2s=
#Java2Script Configuration
#Sun Jan 16 00:50:30 CST 2006
j2s.resources.list=console.css,java/lang/Class.js,java/lang/Object.js,java/lang/String.js,java/lang/Runnable.js,java/lang/Cloneable.js,java/io/Serializable.js,java/lang/Comparable.js,java/lang/CharSequence.js,java/lang/StringBuffer.js,java/util/Comparator.js,java/util/Iterator.js,java/util/ListIterator.js,java/util/Enumeration.js,java/util/Collection.js,java/util/Set.js,java/util/Map.js,java/util/List.js,java/util/RandomAccess.js,java/util/AbstractCollection.js,java/util/AbstractSet.js,java/util/AbstractMap.js,java/util/AbstractList.js,java/util/ArrayList.js,java/util/HashMap.js,java/util/HashSet.js,java/util/Dictionary.js,java/util/Hashtable.js,java/util/Properties.js,java/util/Vector.js,java/util/Stack.js
=*/
/*******************************************************************************
 * Copyright (c) 2005 ognize.com and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Web:
 *     https://sourceforge.net/projects/j2s/
 * Contributors:
 *     ognize.com - initial API and implementation
 *******************************************************************************/
/*******
 * Copyright ognize.com (2005).
 *
 * Test cases for Script Oriented Web Architecture
 *
 * @author josson smith
 * @create Nov 5, 2005
 *******/
 
/**
 * Class Clazz. All the methods are static in this class.
 */
/* static */
function Clazz () {
};

function NullObject () {
};

/**
 * Return the class name of the given class or object.
 *
 * @param clazzHost given class or object
 * @return class name
 */
/* public */
Clazz.getClassName = function (clazzHost) {
	if (clazzHost == null) {
		/* 
		 * null is always treated as Object.
		 * But what about "undefined"?
		 */
		return "NullObject";
	}
	if (typeof clazzHost == "function") {
		var clazz = clazzHost;
		if (clazz.__CLASS_NAME__ != null) {
			/* user defined class name */
			return clazz.__CLASS_NAME__;
		}
		var clazzStr = clazz.toString ();
		var idx1 = clazzStr.indexOf ("function") + 8;
		var idx2 = clazzStr.indexOf ("(", idx1);
		var clazzName = clazzStr.substring (idx1, idx2)
				.replace (/^\s+/, "").replace (/\s+$/, ""); // .trim ()
		if (clazzName == "anonymous") {
			return "Function";
		} else {
			return clazzName;
		}
	} else {
		var obj = clazzHost;
		if (obj instanceof Clazz.CastedNull) {
			return obj.clazzName;
		} else {
			var objType = typeof obj;
			if (objType == "string") {
				/* 
				 * Always treat the constant string as String object.
				 * This will be compatiable with Java String instance.
				 */
				return "String";
			} else if (objType == "object") {
				if (obj.__CLASS_NAME__ != null) {
					/* user defined class name */
					return obj.__CLASS_NAME__;
				} else {
					return Clazz.getClassName (obj.constructor);
				}
			}
			return Clazz.getClassName (obj.constructor);
		}
	}
};
/**
 * Return the class of the given class or object.
 *
 * @param clazzHost given class or object
 * @return class name
 */
/* public */
Clazz.getClass = function (clazzHost) {
	if (clazzHost == null) {
		/* 
		 * null is always treated as Object.
		 * But what about "undefined"?
		 */
		return Object;
	}
	if (typeof clazzHost == "function") {
		return clazzHost;
	} else {
		var clazzName = null;
		var obj = clazzHost;
		if (obj instanceof Clazz.CastedNull) {
			clazzName = obj.clazzName;
		} else {
			if (typeof obj == "object") {
				/* user defined class name */
				if (obj.__CLASS_NAME__ != null) {
					clazzName = obj.__CLASS_NAME__;
				} else {
					return obj.constructor;
				}
			}
		}
		if (clazzName != null) {
			var hostedClazz = null;
			eval ("hostedClazz = " + clazzName + ";");
			return hostedClazz;
		} else {
			return obj.constructor;
		}
	}
};

/*
 * Be used to copy members of class
 */
/* protected */
Clazz.extendsProperties = function (hostThis, hostSuper) {
	for (var o in hostSuper) {
		if (o != "prototype" && o != "superClazz"
				&& o != "__CLASS_NAME__" && o != "implementz") {
			hostThis[o] = hostSuper[o];
		}
	}
};

/*
 * Be used to copy members of interface
 */
/* protected */
Clazz.implementsProperties = function (hostThis, hostSuper) {
	for (var o in hostSuper) {
		if (o != "prototype" && o != "superClazz"
				&& o != "__CLASS_NAME__" && o != "implementz") {
			hostThis[o] = hostSuper[o];
			hostThis.prototype[o] = hostSuper[o];
		}
	}
	for (var o in hostSuper.prototype) {
		if (o != "__CLASS_NAME__") {
			hostThis.prototype[o] = hostSuper.prototype[o];
		}
	}
};

Clazz.args4InheritClass = function () {
};
Clazz.inheritArgs = new Clazz.args4InheritClass ();

/**
 * Inherit class with "extends" keyword and also copy those static members. 
 * Example, as in Java, if NAME is a static member of ClassA, and ClassB 
 * extends ClassA then ClassB.NAME can be accessed in some ways.
 *
 * @param clazzThis child class to be extended
 * @param clazzSuper super class which is inherited from
 * @param objSuper super class instance
 */
/* public */
Clazz.inheritClass = function (clazzThis, clazzSuper, objSuper) {
	var thisClassName = Clazz.getClassName (clazzThis);
	Clazz.extendsProperties (clazzThis, clazzSuper);
	if (objSuper != null) {
		clazzThis.prototype = objSuper;
	} else {
		clazzThis.prototype = new clazzSuper (Clazz.inheritArgs);
	}
	clazzThis.superClazz = clazzSuper;
	clazzThis.__CLASS_NAME__ = thisClassName;
	clazzThis.prototype.__CLASS_NAME__ = thisClassName;
};

/**
 * Implementation of Java's keyword "implements".
 * As in JavaScript there are on "implements" keyword implemented, a property
 * of "implementz" is added to the class to record the interfaces the class
 * is implemented.
 * 
 * @param clazzThis the class to implement
 * @param interfacez Array of interfaces
 */
/* public */
Clazz.implementOf = function (clazzThis, interfacez) {
	if (arguments.length >= 2) {
		if (clazzThis.implementz == null) {
			clazzThis.implementz = new Array ();
		}
		var impls = clazzThis.implementz;
		if (arguments.length == 2) {
			if (typeof interfacez == "function") {
				impls[impls.length] = interfacez;
				Clazz.implementsProperties (clazzThis, interfacez);
			} else if (interfacez instanceof Array) {
				impls.concat (interfacez);
				for (var i = 0; i < interfacez.length; i++) {
					Clazz.implementsProperties (clazzThis, interfacez[i]);
				}
			}
		} else {
			for (var i = 1; i < arguments.length; i++) {
				impls[impls.length] = arguments[i];
				Clazz.implementsProperties (clazzThis, arguments[i]);
			}
		}
	}
};

/**
 * TODO: More should be done for interface's inheritance
 */
/* public */
Clazz.extendInterface = Clazz.implementOf;

/* protected */
Clazz.equalsOrExtendsLevel = function (clazzThis, clazzAncestor) {
	if (clazzThis == clazzAncestor) {
		return 0;
	}
	if (clazzThis.implementz != null) {
		var impls = clazzThis.implementz;
		for (var i = 0; i < impls.length; i++) {
			var level = Clazz.equalsOrExtendsLevel (impls[i], clazzAncestor);
			if (level >= 0) {
				return level + 1;
			}
		}
	}
	return -1;
};

/* protected */
Clazz.getClassNameEvalStr = function (clazzVarName, clazzName) {
	var innerTypes = new Array (
		"number", "string", "function", "object", "array", "boolean"
	);
	for (var i = 0; i < innerTypes.length; i++) {
		if (innerTypes[i] == clazzName) {
			return clazzVarName + " = " 
					+ clazzName.substring (0, 1).toUpperCase ()
					+ clazzName.substring (1) + ";";
		}
	}
	return clazzVarName + " = " + clazzName + ";";
};
/* protected */
Clazz.getInheritedLevel = function (clazzTarget, clazzBase) {
	if (clazzTarget == clazzBase) {
		return 0;
	}
	if ((typeof clazzTarget == "string" && "void" == clazzTarget) 
			|| (typeof clazzBase == "string" && "void" == clazzBase)) {
		return -1;
	}
	if ((typeof clazzTarget == "string" && "unknown" == clazzTarget) 
			|| (typeof clazzBase == "string" && "unknown" == clazzBase)) {
		return -1;
	}
	if ((typeof clazzTarget == "string" && "NullObject" == clazzTarget) 
			|| NullObject == clazzTarget) {
		if (clazzBase != Number && clazzBase != Boolean
				&& clazzBase != NullObject) {
			return 0;
		}
	}
	if (typeof clazzTarget == "string") {
		eval (Clazz.getClassNameEvalStr ("clazzTarget", clazzTarget));
	}
	if (typeof clazzBase == "string") {
		eval (Clazz.getClassNameEvalStr ("clazzBase", clazzBase));
	}
	var level = 0;
	var zzalc = clazzTarget;
	while (zzalc != clazzBase && level < 10) {
		/* maybe clazzBase is interface */
		if (zzalc.implementz != null) {
			var impls = zzalc.implementz;
			for (var i = 0; i < impls.length; i++) {
				var implsLevel = Clazz.equalsOrExtendsLevel (impls[i], clazzBase);
				if (implsLevel >= 0) {
					return level + implsLevel + 1;
				}
			}
		}
		
		zzalc = zzalc.superClazz;
		if (zzalc == null) {
			if (clazzBase == Object) {
				return level + 1;
			} else {
				return -1;
			}
		}
		level++;
	}
	return level;
};

/**
 * Implements Java's keyword "instanceof" in JavaScript's way.
 * As in JavaScript part of the object inheritance is implemented
 * in only-JavaScript way.
 *
 * @param obj the object to be tested
 * @param clazz the class to be checked
 * @return whether the object is an instance of the class
 */
/* public */
Clazz.instanceOf = function (obj, clazz) {
	if (obj == null) {
		return clazz == undefined; // should usually false
	}
	if (clazz == null) {
		return false;
	}
	if (obj instanceof clazz) {
		return true;
	} else {
		/*
		 * To check all the inherited interfaces.
		 */
		var clazzName = Clazz.getClassName (obj);
		return Clazz.getInheritedLevel (clazzName, clazz) >= 0;
	}
};

/**
 * Call super method of the class. 
 * The same effect as Java's expression:
 * <code> super.* () </code>
 * 
 * @param objThis host object
 * @param clazzThis host object's class. Can not get the class me by host 
 * object. Because the clazzThis is used as one signal for the JavaScript
 * to determine which class is the super.* method calling.
 * @param funName method name to be called
 * @param funParams Array of method parameters
 */
/* public */
Clazz.superCall = function (objThis, clazzThis, funName, funParams) {
	var params = Clazz.getParamsType (funParams);
	if (objThis[funName] == null) {
		if ("construct" == funName) {
			/* No super constructor */
			return ;
		}
		throw new MethodNotFoundException (objThis, clazzThis, 
				"super." + funName, params.typeString);
	}
	if (objThis[funName].claxxOwner != null) {
		if (objThis[funName].claxxOwner == clazzThis) {
			if ("construct" == funName) {
				/* No super constructor */
				return ;
			}
			throw new MethodNotFoundException (objThis, clazzThis, 
					"super." + funName, params.typeString);
		}
		/*
		 * This is a single method, call directly!
		 */
		var callException;
		if (funParams == null) {
			try {
				returnObj = objThis[funName]
						.apply (objThis, []);
			} catch (e) {
				callException = e;
			}
		} else {
			try {
				returnObj = objThis[funName]
						.apply (objThis, funParams);
			} catch (e) {
				callException = e;
			}
		}
		if (callException != null) {
			throw callException;
		}
		return returnObj;
	}
	Clazz.prepareForSuperCall (objThis, clazzThis);
	var length = objThis[funName].stacks.length;
	var returnObj;
	var callException;
	var isReturned = false;
	for (var i = 0; i < length; i++) {
		if (objThis[funName].stacks[i] == clazzThis) {
			if (i > 0) {
				var clazzParent = objThis[funName].stacks[i - 1];
				if (funParams == null) {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, []);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				} else {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, funParams);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				}
			} else {
				var clazzParent = objThis[funName].stacks[0];
				var unknownFun = clazzParent.prototype[funName]["\\unknown"];
				if (unknownFun != null) {
					if (funParams == null) {
						try {
							returnObj = unknownFun.apply (objThis, []);
						} catch (e) {
							callException = e;
						}
						isReturned = true;
						break;
					} else {
						try {
							returnObj = unknownFun.apply (objThis, funParams);
						} catch (e) {
							callException = e;
						}
						isReturned = true;
						break;
					}
				}
			}
			break;
		}
	}
	var lastLevel;
	if (!isReturned) {
		for (var i = length - 1; i >= 0; i--) {
			var level = Clazz.getInheritedLevel (clazzThis, 
					objThis[funName].stacks[i]);
			lastLevel = level;
			if (level > 0) { /* once ">" was ">=" */
				var clazzParent = objThis[funName].stacks[i];
				if (funParams == null) {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, []);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				} else {
					try {
						returnObj = clazzParent.prototype[funName]
								.apply (objThis, funParams);
					} catch (e) {
						callException = e;
					}
					isReturned = true;
					break;
				}
			}
		}
	}
	Clazz.restoreFromSuperCall (objThis, clazzThis);
	if (isReturned) {
		if (callException != null) {
			throw callException;
		}
		return returnObj;
	}
	if (/*lastLevel == 0 && */funName == "construct") {
		/* No super constructor! */
		return ;
	}
	throw MethodNotFoundException (objThis, clazzThis, 
			funName, params.typeString);
};

/**
 * Call super constructor of the class. 
 * The same effect as Java's expression: 
 * <code> super () </code>
 */
/* public */
Clazz.superConstructor = function (objThis, clazzThis, funParams) {
	Clazz.superCall (objThis, clazzThis, "construct", funParams);
};

/**
 * Class for null with a given class as to be casted.
 * This class will be used as an implementation of Java's casting way.
 * For example,
 * <code> this.call ((String) null); </code>
 */
/* protcted */
Clazz.CastedNull = function (asClazz) {
	if (asClazz != null) {
		if (asClazz instanceof String) {
			this.clazzName = asClazz;
		} else if (asClazz instanceof Function) {
			this.clazzName = Clazz.getClassName (asClazz);
		} else {
			this.clazzName = "" + asClazz;
		}
	} else {
		this.clazzName = "Object";
	}
	this.toString = function () {
		return null;
	};
	this.valueOf = function () {
		return null;
	};
};

/**
 * API for Java's casting null.
 * @see Clazz.CastedNull
 *
 * @param asClazz given class
 * @return an instance of class Clazz.CastedNull
 */
/* public */
Clazz.castNullAs = function (asClazz) {
	return new Clazz.CastedNull (asClazz);
};

/** 
 * MethodException will be used as a signal to notify that the method is
 * not found in the current clazz hierarchy.
 */
/* private */
Clazz.MethodException = function () {
	/*
	this.message = "The static Clazz instance can not found the method!";
	this.toString = function () {
		return this.message;
	};
	*/
};

/* private */
Clazz.getParamsType = function (funParams) {
	var params = new Array ();
	var containsCastedNullParams = false;
	if (funParams != null) {
		for (var i = 0; i < funParams.length; i++) {
			params[i] = Clazz.getClassName (funParams[i]);
			if (funParams[i] instanceof Clazz.CastedNull) {
				containsCastedNullParams = true;
			}
		}
	}
	if (params.length == 0) {
		params[0] = "void";
	}
	params.typeString = "\\" + params.join ('\\');
	params.containsCastedNullParams = containsCastedNullParams;
	return params;
};
/**
 * Search the given class prototype, find the method with the same
 * method name and the same parameter signatures with the given 
 * parameters, and then run the method with the given parameters.
 *
 * @param objThis the current host object
 * @param claxxRef the current host object's class
 * @param fxName the method name
 * @param funParams the given arguments
 * @return the result of the specified method of the host object,
 * the return maybe void.
 * @throws MethodNotFoundException if no matched method is found
 */
/* protected */
Clazz.searchAndRunMethod = function (objThis, claxxRef, fxName, funParams) {
	var params = Clazz.getParamsType (funParams);
	/*
	 * Search the inheritance stacks to get the given class' function
	 */
	if (objThis[fxName].stacks == null) {
		log (fxName);
		log (claxxRef.getName ());
		log (objThis);
		log (objThis[fxName]);
		log ("error");
	}
	var length = objThis[fxName].stacks.length;
	for (var i = 0; i < length; i++) {
		if (objThis[fxName].stacks[i] == claxxRef) {
			var clazzFun = objThis[fxName].stacks[i].prototype[fxName];
			if (clazzFun == null) {
				/*
				 * Should try to call its super methods before throwing out
				 * exception.
				 */
				throw new MethodNotFoundException (objThis, claxxRef, 
						fxName, params.typeString);
			}
			try {
				return Clazz.tryToSearchAndRun (objThis, clazzFun, params, 
						funParams);
			} catch (e) {
				if (!(e instanceof Clazz.MethodException)) {
					throw e;
				}
			}
			/*
			 * Try to call its super methods before throwing out exception.
			 */
			break;
		}
	}
	for (var i = length - 1; i >= 0; i--) {
		if (Clazz.getInheritedLevel (claxxRef, 
				objThis[fxName].stacks[i]) >= 0) {
			var clazzFun = objThis[fxName].stacks[i].prototype[fxName];
			if (clazzFun == null) {
				/*
				 * Should try to call its super methods before throwing out
				 * exception.
				 */
				throw new MethodNotFoundException (objThis, claxxRef, 
						fxName, params.typeString);
			}
			try {
				return Clazz.tryToSearchAndRun (objThis, clazzFun, params, 
						funParams);
			} catch (e) {
				if (!(e instanceof Clazz.MethodException)) {
					throw e;
				}
			}
			/*
			 * Try to call its super methods before throwing out exception.
			 */
			continue;
		}
	}
	if ("construct" == fxName) {
		return ;
	}
	throw new MethodNotFoundException (objThis, claxxRef, 
			fxName, params.typeString);
};

/* private */
Clazz.tryToSearchAndRun = function (objThis, clazzFun, params, funParams) {
	var methodSignatures = new Array ();
	var xfparams = null;
	for (var fn in clazzFun) {
		if (fn.indexOf ('\\') == 0) {
			methodSignatures[methodSignatures.length] = fn.substring (1);
		}
		/*
		 * When there are only one method in the class, use the funParams
		 * to identify the parameter type.
		 *
		 * Fixed a bug of ArrayList.remove (Object)
		 */
		if (fn == "funParams") {
			xfparams = clazzFun.funParams;
		}
	}
	var generic = false;
	if (methodSignatures.length == 0 && xfparams != null) {
		methodSignatures[0] = xfparams.substring (1);
		generic = true;
	}
	var method = Clazz.searchMethod (methodSignatures, params);
	if (method != null) {
		var f = null;
		if (generic) { /* Use the generic method */
			f = clazzFun;
		} else {
			f = clazzFun["\\" + method];
		}
		if (f != null) {
			if (params.containsCastedNullParams) {
				var methodParams = new Array ();
				for (var k = 0; k < funParams.length; k++) {
					if (funParams[k] instanceof Clazz.CastedNull) {
						methodParams[k] = null;
					} else {
						methodParams[k] = funParams[k];
					}
				}
				return f.apply (objThis, methodParams);
			} else {
				return f.apply (objThis, funParams);
			}
		}
	}
	throw new Clazz.MethodException ();
};
/**
 * Search the existed polynomial methods to get the matched method with
 * the given parameter types.
 *
 * @param existedMethods Array of string which contains method parameters
 * @param paramTypes Array of string that is parameter type.
 * @return string of method parameters seperated by "\\"
 */
/* private */
Clazz.searchMethod = function (existedMethods, paramTypes) {
	var roundOne = new Array ();
	for (var i = 0; i < existedMethods.length; i++) {
		var split = existedMethods[i].split (/\\/);
		if (split.length == paramTypes.length) {
			roundOne[roundOne.length] = split;
		}
	}
	var resultOne = roundOne;
	var roundTwo = new Array ();
	for (var i = 0; i < resultOne.length; i++) {
		var fittedLevel = new Array ();
		var isFitted = true;
		for (var j = 0; j < resultOne[i].length; j++) {
			//fittedLevel[j] = Clazz.getInheritedLevel (paramTypes[j], 
			//		resultOne[i][j]);
			/*
			 * The following scoped codes are to implement the upper
			 * commented codes.
			 */
			var s1 = null;
			if ("void" == paramTypes[j] || "number" == paramTypes[j]
					|| "string" == paramTypes[j] 
					|| "string" == paramTypes[j] 
					|| "function" == paramTypes[j] 
					|| "object" == paramTypes[j]
					|| "unknown" == paramTypes[j]) {
				s1 = "\"" + paramTypes[j] + "\"";
			} else {
				s1 = paramTypes[j];
			}
			var s2 = null;
			if ("void" == resultOne[i][j] || "number" == resultOne[i][j]
					|| "string" == resultOne[i][j]
					|| "string" == resultOne[i][j]
					|| "function" == resultOne[i][j]
					|| "object" == resultOne[i][j]
					|| "unknown" == resultOne[i][j]) {
				s2 = "\"" + resultOne[i][j] + "\"";
			} else {
				s2 = resultOne[i][j];
			}
			fittedLevel[j] = eval ("Clazz.getInheritedLevel (" + s1 + ", " 
					+ s2 + ");");
			if (fittedLevel[j] < 0) {
				isFitted = false;
				break;
			}
		}
		if (isFitted) {
			fittedLevel[paramTypes.length] = i;
			roundTwo[roundTwo.length] = fittedLevel;
		}
	}
	if (roundTwo.length == 0) {
		return null;
	}
	var resultTwo = roundTwo;
	var min = resultTwo[0];
	for (var i = 1; i < resultTwo.length; i++) {
		var isVectorLesser = true;
		for (var j = 0; j < paramTypes.length; j++) {
			if (min[j] < resultTwo[i][j]) {
				isVectorLesser = false;;
				break;
			}
		}
		if (isVectorLesser) {
			min = resultTwo[i];
		}
	}
	var index = min[paramTypes.length];
	var params = resultOne[index];
	var methodParams = "";
	for (var i = 0; i < params.length; i++) {
		methodParams += params[i];
		if (i != params.length - 1) {
			methodParams += "\\";
		}
	}
	return methodParams;
};

/**
 * Generate delegating function for the given method name.
 *
 * @param claxxRef the specified class for the method
 * @funName method name of the specified method
 * @return the method delegate which will try to search the method
 * from the given class by the parameters
 */
/* private */
Clazz.generateDelegatingMethod = function (claxxRef, funName) {
	/*
	 * Delegating method.
	 * Each time the following expression will generate a new 
	 * function object.
	 */
	/*
	var delegating = function () {
			var classRefrence = arguments.callee.claxxRefrence;
			var methodName = arguments.callee.methodName;
			return Clazz.searchAndRunMethod (this, classRefrence, 
					methodName, arguments);
	};
	*/
	var delegating = function () {
			var r = arguments;
			return SARM (this, r.callee.claxxRefrence, r.callee.methodName, r);
	};
	delegating.methodName = funName;
	delegating.claxxRefrence = claxxRef;
	return delegating;
};

SARM = Clazz.searchAndRunMethod;

/*
 * Define method for the class with the given method name and method
 * body and method parameter signature.
 *
 * @param clazzThis host class in which the method to be defined
 * @param funName method name
 * @param funBody function object, e.g function () { ... }
 * @funParams paramether signature, e.g ["string", "number"]
 */
/* public */
Clazz.defineMethod = function (clazzThis, funName, funBody, funParams) {
	var fpName = "";
	if (funParams != null) {
		/* 
		 * If funParams is Array, funParams.toString() will
		 * also return "*,*,..." string.
		 */
		var paramStr = funParams.toString ()
				.replace (/^\s+/, "").replace (/\s+$/, ""); //.trim ()
		if (paramStr.length != 0) {
			var params = paramStr.split (/\s*,\s*/);
			fpName += "\\" + params.join ('\\');
		} else {
			fpName += "\\void";
		}
	} else {
		fpName += "\\void";
	}
	/*
	 * For method the first time is defined, just keep it rather than
	 * wrapping into deep hierarchies!
	 */
	if (clazzThis.prototype[funName] == null) {
		funBody.funParams = fpName;
		funBody.claxxOwner = clazzThis;
		clazzThis.prototype[funName] = funBody;
		return ;
	} else if (clazzThis.prototype[funName].claxxOwner == clazzThis
			&& clazzThis.prototype[funName].funParams == fpName) {
		return ;
	}
	var oldFun = null;
	var oldStacks = new Array ();
	if (clazzThis.prototype[funName] != null) {
		if (clazzThis.prototype[funName].stacks == null) {
			/* method is not defined by Clazz.defineMethod () */
			oldFun = clazzThis.prototype[funName];
		} else {
			oldStacks = clazzThis.prototype[funName].stacks;
		}
	}
	
	if (oldFun != null && oldFun.claxxOwner != null) {
		/*
		 * oldStacks should be "new Array ()";
		 */
		oldStacks[0] = oldFun.claxxOwner;
		/*
		if (oldFun.claxxOwner != clazzThis) {
			if ("releaseChild" == funName) {
				error (" in here ");
			}
			oldFun.claxxOwner.prototype[funName].stacks = oldStacks;
			oldFun.claxxOwner.prototype[funName] = Clazz
					.generateDelegatingMethod (oldFun.claxxOwner, funName);
			oldFun.claxxOwner.prototype[funName][oldFun.funParams] = oldFun;
			oldFun.claxxOwner = null;
			oldFun.funParams = null;
			oldFun = null;
		}
		//*/
	}
	/* method has not been defined yet */
	/* method is not defined by Clazz.defineMethod () */
	/* method is defined in super class */
	if (clazzThis.prototype[funName] == null 
			|| clazzThis.prototype[funName].stacks == null 
			|| clazzThis.prototype[funName].claxxRefrence != clazzThis) {
		/*
		 * Generate a new delegating method for the class
		 */
		clazzThis.prototype[funName] = Clazz
				.generateDelegatingMethod (clazzThis, funName);
		/*
		 * Keep the class inheritance stacks
		 */
		var arr = new Array ();
		for (var i = 0; i < oldStacks.length; i++) {
			arr[i] = oldStacks[i];
		}
		clazzThis.prototype[funName].stacks = arr;
	}
	var ss = clazzThis.prototype[funName].stacks;

	if (ss.length == 0/* || ss[ss.length - 1] != clazzThis*/) {
		ss[ss.length] = clazzThis;
	} else {
		var existed = false;
		for (var i = ss.length - 1; i >= 0; i--) {
			if (ss[i] == clazzThis) {
				existed = true;
				break;
			}
		}
		if (!existed) {
			ss[ss.length] = clazzThis;
		}
	}

	if (oldFun != null) {
		if (oldFun.claxxOwner == clazzThis) {
			clazzThis.prototype[funName][oldFun.funParams] = oldFun;
			oldFun.claxxOwner = null;
			oldFun.funParams = null;
		} else {
			/*
			 * The function is not defined Clazz.defineMethod ().
			 * Try to fixup the method ...
			 * As a matter of lost method information, I just suppose
			 * the method to be fixed is with void parameter!
			 */
			clazzThis.prototype[funName]["\\unknown"] = oldFun;
		}
	}
	clazzThis.prototype[funName][fpName] = funBody;
};

/**
 * Make constructor for the class with the given function body and parameters
 * signature.
 * 
 * @param clazzThis host class
 * @param funBody constructor body
 * @param funParams constructor parameters signature
 */
/* public */
Clazz.makeConstructor = function (clazzThis, funBody, funParams) {
	var funName = "construct";
	Clazz.defineMethod (clazzThis, funName, funBody, funParams);
};

/**
 * Prepare callback for instance of anonymous Class.
 * For example for the callback:
 *     this.callbacks.MyEditor.sayHello();
 *
 * @param objThis the host object for callback
 * @param classThisObj the "this" object to be hooked
 * 
 * Attention: parameters should not be null!
 */
/* protected */
Clazz.prepareCallback = function (objThis, classThisObj) {
	if (objThis != null && classThisObj != null) {
		var obs = objThis["callbacks"];
		if (obs == null) {
			obs = new Array ();
			objThis["callbacks"] = obs;
		}
		var className = Clazz.getClassName (classThisObj);
		//if (obs[className] == null) { /* == null make no sense! */
			obs[className] = classThisObj;
			var clazz = Clazz.getClass (classThisObj);
			while (clazz.superClazz != null) {
				clazz = clazz.superClazz;
				obs[Clazz.getClassName (clazz)] = classThisObj;
			}
		//}
		var cbs = classThisObj["callbacks"];
		if (cbs != null && cbs instanceof Array) {
			for (var s in cbs) {
				if (s != "length") {
					obs[s] = cbs[s];
				}
			}
		}
	}
};

/**
 * Construct instance of the given inner class.
 *
 * @param classInner given inner class, alway with name like "*$*"
 * @objThis this instance which can be used to call back.
 * @return the constructed object
 */
/* public */
Clazz.innerTypeInstance = function (clazzInner, objThis) {
	var obj = null;
	if (arguments.length == 2) {
		obj = new clazzInner (objThis);
	} else if (arguments.length == 3) {
		obj = new clazzInner (objThis, arguments[2]);
	} else if (arguments.length == 4) {
		obj = new clazzInner (objThis, arguments[2], arguments[3]);
	} else if (arguments.length == 5) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4]);
	} else if (arguments.length == 6) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5]);
	} else if (arguments.length == 7) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6]);
	} else if (arguments.length == 8) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6], arguments[7]);
	} else if (arguments.length == 9) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6], arguments[7], arguments[8]);
	} else if (arguments.length == 10) {
		obj = new clazzInner (objThis, arguments[2], arguments[3], arguments[4], 
				arguments[5], arguments[6], arguments[7], arguments[8],
				arguments[9]);
	} else {
		/*
		 * Should construct instance manually.
		 */
		obj = new clazzInner ();
		if (obj.construct == null) {
			throw new String ("No support anonymous class constructor with " 
					+ "more than 8 parameters.");
		}
		var args = new Array ();
		for (var i = 2; i < arguments.length; i++) {
			args[i - 2] = arguments[i];
		}
		obj.construct.apply (obj, args);
	}
	Clazz.prepareCallback (obj, objThis);
	return obj;
};

/* protected */
Clazz.allPackage = new Object ();

/* public */
Clazz.declarePackage = function (pkgName) {
	if (pkgName != null && pkgName.length != 0) {
		var pkgFrags = pkgName.split (/\./);
		var pkg = null;
		for (var i = 0; i < pkgFrags.length; i++) {
			if (pkg == null) {
				if (Clazz.allPackage[pkgFrags[0]] == null) {
					Clazz.allPackage[pkgFrags[0]] = new Object ();
					eval (pkgFrags[0] + " = Clazz.allPackage[pkgFrags[0]];" );
				}
				pkg = Clazz.allPackage[pkgFrags[0]];
			} else {
				if (pkg[pkgFrags[i]] == null) {
					pkg[pkgFrags[i]] = new Object ();
				}
				pkg = pkg[pkgFrags[i]]
			}
		}
		return pkg;
	}
};

/* public */
Clazz.isClassDefined = Clazz.isDefinedClass = function (clazzName) {
	if (clazzName != null && clazzName.length != 0) {
		var pkgFrags = clazzName.split (/\./);
		var pkg = null;
		for (var i = 0; i < pkgFrags.length; i++) {
			if (pkg == null) {
				if (Clazz.allPackage[pkgFrags[0]] == null) {
					//error (clazzName + " / " + false);
					return false;
				}
				pkg = Clazz.allPackage[pkgFrags[0]];
			} else {
				if (pkg[pkgFrags[i]] == null) {
					//error (clazzName + " / " + false);
					return false;
				}
				pkg = pkg[pkgFrags[i]]
			}
		}
		//error (clazzName + " / " + (pkg != null));
		return pkg != null;
	} else {
		/* consider null or empty name as non-defined class */
		return false;
	}
};

/* protected */
Clazz.instantialize = function (objThis, args) {
	if (args != null && args.length == 1 && args[0] != null 
			&& args[0] instanceof Clazz.args4InheritClass) {
		return ;
	}
	if (objThis.construct != null) {
		objThis.construct.apply (objThis, args);
	}
};

/* protected */
Clazz.stackSuperField = function (objThis, clazzThis, field) {
	if (objThis.superFields == null) {
		objThis.superFields = new Array ();
		if (objThis instanceof Function) {
			/* static member! */
			objThis.prototype.superFields = objThis.superFields;
		}
	}
	var clazzName = Clazz.getClassName (clazzThis);
	var fieldName = clazzName + ".super." + field;
	if (objThis.superFields[fieldName] == null) {
		objThis.superFields[fieldName] = objThis[field];
	}
};

/*
 * super.width will be accessed as Clazz.superAccess (this, SuperClass, "width")?
 *
 * ATTENTION: readonly!
 */
/* public */
Clazz.superAccess = function (objThis, clazzThis, field) {
	if (objThis.superFields == null) {
		return objThis[field];
	}
	var clazzName = Clazz.getClassName (clazzThis);
	var fieldName = clazzName + ".super." + field;
	if (typeof objThis.superFields[fieldName] == "undefined") {
		return objThis[field];
	}
	return objThis.superFields[fieldName];
};

/*
 * See more in Clazz#superCall
 */
/* protected */
Clazz.prepareForSuperCall = function (objThis, clazzThis) {
	if (objThis.superFields == null) {
		return ;
	}
	var stackedFields = new Array ();
	for (var f in objThis.superFields) {
		var idx = f.lastIndexOf (".super.");
		if (idx != -1) {
			var field = f.substring (idx + 7); // 7 = ".super.".length
			stackedFields[stackedFields.length] = field;
		}
	}
	if (stackedFields.length == 0) {
		return ;
	}
	var clazzName = Clazz.getClassName (clazzThis);
	for (var i = 0; i < stackedFields.length; i++) {
		var field = stackedFields[i];
		var fieldName = clazzName + ".this." + field;
		objThis.superFields[fieldName] = objThis[field];
		
		fieldName = clazzName + ".super." + field;
		objThis[field] = objThis.superFields[fieldName];
	}
};

/*
 * See more in Clazz#superCall
 */
/* protected */
Clazz.restoreFromSuperCall = function (objThis, clazzThis) {
	if (objThis.superFields == null) {
		return ;
	}
	var stackedFields = new Array ();
	for (var f in objThis.superFields) {
		var idx = f.lastIndexOf (".this.");
		if (idx != -1) {
			var field = f.substring (idx + 6); // 6 = ".this.".length
			stackedFields[stackedFields.length] = field;
		}
	}
	if (stackedFields.length == 0) {
		return ;
	}
	var clazzName = Clazz.getClassName (clazzThis);
	for (var i = 0; i < stackedFields.length; i++) {
		var field = stackedFields[i];
		var fieldName = clazzName + ".super." + field;
		/* restore the value in super scope */
		objThis.superFields[fieldName] = objThis[field];
		
		fieldName = clazzName + ".this." + field;
		objThis[field] = objThis.superFields[fieldName];
		delete objThis.superFields[fieldName];
	}
};

Clazz.newArray  = function () {
	var args = arguments;
	if (arguments.length == 1) {
		if (arguments[0] instanceof Array) {
			args = arguments[0];
		}
	}
	if (args.length <= 1) {
		return new Array ();
	} else if (args.length == 2) {
		var dim = args[0];
		if (typeof dim == "string") {
			dim = dim.charCodeAt (0); // char
		}
		var val = args[1];
		var arr = new Array (dim);
		for (var i = 0; i < dim; i++) {
			arr[i] = val;
		}
		return arr;
	} else {
		var dim = args[0];
		if (typeof dim == "string") {
			dim = dim.charCodeAt (0); // char
		}
		var len = args.length - 1;
		var xargs = new Array (len);
		for (var i = 0; i < len; i++) {
			xargs[i] = args[i + 1];
		}
		var arr = new Array (dim);
		for (var i = 0; i < dim; i++) {
			arr[i] = Clazz.newArray (xargs);
		}
		return arr;
	}
};

Clazz.makeFunction = function (jsr) {
	return function () {
		jsr.run ();
		if (jsr.returnSet == 1) {
			return jsr.returnNumber;
		} else if (jsr.returnSet == 2) {
			return jsr.returnBoolean;
		} else if (jsr.returnSet == 3) {
			return jsr.returnObject;
		}
	};
};

Function.prototype.equals = function (aFun) {
	return this.prototype.constructor == aFun;
};

Function.prototype.getName = function () {
	return Clazz.getClassName (this);
};

/*******
 * Copyright ognize.com (2005).
 *
 * API for Script Oriented Web Architecture
 *
 * @author josson smith
 * @create Oct 27, 2005
 *******/

function MethodNotFoundException (obj, clazz, method, params) {
	this.id = "MethodNotFoundException";
	var paramStr = "";
	var s = new Array ();
	if (params != null) {
		paramStr = params.substring (1).replace (/\\/g, ",");
	}
	var leadingStr = "";
	if (method != null && method != "construct") {
		leadingStr = "Method";
	} else {
		leadingStr = "Constructor";
	}
	this.message = leadingStr + " " + Clazz.getClassName (clazz) + "." 
					+ method + "(" + paramStr + ") is not found!";
	this.toString = function () {
		return this.id + ":" + this.message;
	}
}
/*******
 * Copyright ognize.com (2005).
 *
 * @author josson smith
 * @create Nov 5, 2005
 *******/

var buffer = "";
var bufferConsole = false;
var maxTotalLines =	1000;
var lines = 0;

function outputInConsole (s, isBreak, color) {
	if (isBreak) {
		lines++;
	}
	if (lines > maxTotalLines) {
		return ;
	}
 	var starts = new Date ();
 	if (s == null) {
 		s = "";
 	}
	s = ("" + s).replace (/&/g, "&amp;")
			.replace (/</g, "&lt;").replace (/>/g, "&gt;")
			.replace (/\r\n/g, "<br/>")
			.replace (/\r/g, "<br/>")
			.replace (/\n/g, "<br/>");
	if (color != null) {
		s = "<span style=\"color:" + color + ";\">" + s + "</span>";
	}
	if (isBreak) {
		s += "<br/>";
	}
	if (bufferConsole) {
		// Without buffer technologies, the mozilla will spend most
		// of the time in the following else codes.
		buffer += s;
	} else {
		var console = document.getElementById ("_console_");;
		if (console != null) {
			console.innerHTML += s;
			console.scrollTop += 100;
		} else {
			document.write (s);
		}
	}
 	var ends = new Date ();
	sumTime (starts, ends);
};

function flushConsoleBuffer () {
	if (!bufferConsole) {
		return ;
	}
	var console = document.getElementById ("_console_");;
	if (console != null) {
		console.innerHTML += buffer;
		console.scrollTop += 10000;
	} else {
		document.write (buffer);
	}
	buffer = "";
};

window.popupAlert = window.alert;

window.alert = function (s) {
	outputInConsole (s, true);
};

window.error = function (s) {
	outputInConsole (s, true, "red");
};

window.log = function (s) {
	outputInConsole (s, true, "yellow");
	if (s == "End") {
		flushConsoleBuffer ();
	}
};

window.assert = function () {
	var b = true;
	if (arguments.length == 1) {
		b = arguments[0];
	} else if (arguments.length == 2) {
		var x1 = arguments[0];
		var x2 = arguments[1];
		b = (x1 == x2);
	} else {
		var x1 = arguments[0];
		var x2 = arguments[1];
		var delta = arguments[2];
		b = Math.abs (x1 - x2) < Math.abs (delta);
	}
	if (b) {
		outputInConsole ("Passed", true, "green");
	} else {
		if (arguments.length >= 2) {
			outputInConsole ("Failed: expecting " + arguments[1] 
					+ ", but " + arguments[0] + " !", true, "red");
		} else {
			outputInConsole ("Failed", true, "red");
		}
	}
};

var totalTime = 0;
window.sumTime = function (t1, t2) {
	if (arguments.length == 1) {
		totalTime += t1;
	} else if (arguments.length == 2) {
		totalTime += t2.getTime () - t1.getTime ();
	}
};

var startTime = new Date ();
window.startLog = function () {
	startTime = new Date ();
};
startLog ();
window.endLog = function () {
	var endTime = new Date ();
	log (startTime);
	log (endTime);
	log ("Cost ms: " + (endTime.getTime () - startTime.getTime ()));
	log ("Sum time (in ms): " + totalTime);
	flushConsoleBuffer ();
};

System = new Object ();

System.arraycopy = System.arrayCopy = function (src, srcPos, dest, destPos, length) {
	for (var i = 0; i < length; i++) {
		dest[destPos + i] = src[srcPos + i];
	}
};

System.out = new Object ();

System.out.print = function (s) { 
	outputInConsole (s, false);
};

System.out.println = function (s) {
	outputInConsole (s, true);
};

System.err = new Object ();

System.err.print = function (s) { 
	outputInConsole (s, false, "red");
};

System.err.println = function (s) {
	outputInConsole (s, true, "red");
};

Console = new Object ();

Console.output = function (s, isBreak, color) {
	outputInConsole (s, isBreak, color);
};

Console.flush = function () {
	flushConsoleBuffer ();
};
Clazz.declarePackage ("java.lang");
java.lang.Object = Object;

Object.prototype.equals = function (obj) {
	return this == obj;
};

Object.prototype.hashCode = function () {
	return this.toString ().hashCode ();
};

Object.prototype.getClass = function () {
	return Clazz.getClass (this);
};

Object.prototype.clone = function () {
	return this;
};

Object.prototype.finalize = function () {
};

Object.prototype.notify = function () {
};

Object.prototype.notifyAll = function () {
};

Object.prototype.wait = function () {
};

Encoding = new Object();
Encoding.UTF8 = "utf-8";
Encoding.UTF16 = "utf-16";
Encoding.ASCII = "ascii";

/* protected */
Encoding.guessEncoding = function (str) {
	if (str.charCodeAt(0) == 0xEF && str.charCodeAt(1) == 0xBB && str.charCodeAt(2) == 0xBF) {
		return Encoding.UTF8;
	} else if (str.charCodeAt(0) == 0xFF && str.charCodeAt(1) == 0xFE) {
		return Encoding.UTF16;
	} else {
		return Encoding.ASCII;
	}
};
/* public */
Encoding.readUTF8 = function (str) {
	var encoding = this.guessEncoding(str);
	var startIdx = 0;
	if (encoding == Encoding.UTF8) {
		startIdx = 3;
	} else if (encoding == Encoding.UTF16) {
		startIdx = 2;
	}
	var arrs = new Array();
	for (var i = startIdx; i < str.length; i++) {
		var charCode = str.charCodeAt(i);
		if (charCode < 0x80) {
			arrs[arrs.length] = str.charAt(i);
		} else if (charCode > 0xc0 && charCode <= 0xe0) {
			var c1 = charCode & 0x1f;
			i++;
			var c2 = str.charCodeAt(i) & 0x3f;
			var c = (c1 << 6) + c2;
			arrs[arrs.length] = String.fromCharCode(c);
		} else if (charCode > 0xe0) {
			var c1 = charCode & 0x0f;
			i++;
			var c2 = str.charCodeAt(i) & 0x3f;
			i++;
			var c3 = str.charCodeAt(i) & 0x3f;
			var c = (c1 << 12) + (c2 << 6) + c3;
			arrs[arrs.length] = String.fromCharCode(c);
		}
	}
	return arrs.join ('');
};
/* public */
Encoding.convert2UTF8 = function (str) {
	var encoding = this.guessEncoding(str);
	var startIdx = 0;
	if (encoding == Encoding.UTF8) {
		return str;
	} else if (encoding == Encoding.UTF16) {
		startIdx = 2;
	}
	
	var offset = 0;
	var arrs = new Array(offset + str.length - startIdx);
	/*
	arrs[0] = String.fromCharCode(0xEF).charAt(0);
	arrs[1] = String.fromCharCode(0xBB).charAt(0);
	arrs[2] = String.fromCharCode(0xBF).charAt(0);
	*/
	for (var i = startIdx; i < str.length; i++) {
		var charCode = str.charCodeAt(i);
		if (charCode < 0x80) {
			arrs[offset + i - startIdx] = str.charAt(i);
		} else if (charCode < 0x07ff) { //(charCode > 0xc0 && charCode < 0xe0) {
			var c1 = 0xc0 + ((charCode & 0x07c0) >> 6);
			var c2 = 0x80 + (charCode & 0x003f);
			arrs[offset + i - startIdx] = String.fromCharCode(c1) + String.fromCharCode(c2);
		} else {
			var c1 = 0xe0 + ((charCode & 0xf000) >> 12);
			var c2 = 0x80 + ((charCode & 0x0fc0) >> 6);
			var c3 = 0x80 + (charCode & 0x003f);
			arrs[offset + i - startIdx] = String.fromCharCode(c1) + String.fromCharCode(c2) + String.fromCharCode(c3);
		}
	}
	return arrs.join ('');
};
Clazz.declarePackage ("java.lang");
java.lang.String = String;

String.prototype.trim = function () {
	var len = this.length;
	var st = 0;

	while ((st < len) && (this.charAt (st) <= ' ')) {
	    st++;
	}
	while ((st < len) && (this.charAt (len - 1) <= ' ')) {
	    len--;
	}
	return ((st > 0) || (len < len)) ? this.substring (st, len) : this;
};

/* private */
String.prototype.startsWith_string_number = function (prefix, toffset) {
	var to = toffset;
	var po = 0;
	var pc = prefix.length;
	// Note: toffset might be near -1>>>1.
	if ((toffset < 0) || (toffset > this.length - pc)) {
	    return false;
	}
	while (--pc >= 0) {
	    if (this.charAt (to++) != prefix.charAt (po++)) {
	        return false;
	    }
	}
	return true;
};

String.prototype.startsWith = function (prefix) { /* prefix, toffset */
	if (arguments.length == 1) {
		return this.startsWith_string_number (arguments[0], 0);
	} else if (arguments.length == 2) {
		return this.startsWith_string_number (arguments[0], arguments[1]);
	} else {
		return false;
	}
};

String.prototype.endsWith = function (suffix) {
	return this.startsWith (suffix, this.length - suffix.length);
};

String.prototype.equals = function (anObject) {
	return this == anObject;
};

String.prototype.equalsIgnoreCase = function (anotherString) {
	return this == anotherString 
			|| this.toLowerCase () == anotherString.toLowerCase ();
};

/* private */
String.prototype.hash = 0;

String.prototype.hashCode = function () {
	var h = this.hash;
	if (h == 0) {
		var off = 0;
		var len = this.length;
		for (var i = 0; i < len; i++) {
			h = 31*h + this.charCodeAt (off++);
			h &= 0xffffffff;
		}
		this.hash = h;
	}
    return h;
};

String.prototype.getBytes = function () {
	var utf8Str = Encoding.convert2UTF8 (this);
	var arrs = new Array (utf8Str.length);
	for (var i = 0; i < utf8Str.length; i++) {
		arrs[i] = utf8Str.charCodeAt (i);
	}
	return arrs;
};

String.prototype.compareTo = function (anotherString) {
	var len1 = this.length;
	var len2 = anotherString.length;
	var n = Math.min (len1, len2);
	var k = 0;
    while (k < n) {
		var c1 = this.charCodeAt (k);
		var c2 = anotherString.charCodeAt (k);
		if (c1 != c2) {
			return c1 - c2;
		}
		k++;
	}
	return len1 - len2;
};

String.prototype.toCharArray = function () {
	var result = new Array (this.length);
	for (var i = 0; i < this.length; i++) {
		result[i] = this.charAt (i);
	}
	return result;
};

String.valueOf = function (o) {
	return "" + o;
};

String.prototype.subSequence = function (beginIndex, endIndex) {
	return this.substring(beginIndex, endIndex);
};

String.prototype.compareToIgnoreCase = function (str) {
	if (this == str) {
		return 0;
	} else if (this > str) {
		return 1;
	} else {
		return -1;
	}
};

String.prototype.contentEquals = function (sb) {
	if (this.length != sb.length ()) {
		return false;
	}
	var v = sb.getValue ();
	var i = 0;
	var j = 0;
	var n = count;
	while (n-- != 0) {
		if (this.charCodeAt (i++) != v[j++]) {
			return false;
		}
	}
	return true;
};

String.prototype.getChars = function (srcBegin, srcEnd, dst, dstBegin) {
	if (srcBegin < 0) {
		throw new StringIndexOutOfBoundsException(srcBegin);
	}
	if (srcEnd > this.length) {
		throw new StringIndexOutOfBoundsException(srcEnd);
	}
	if (srcBegin > srcEnd) {
		throw new StringIndexOutOfBoundsException(srcEnd - srcBegin);
	}
	for (var i = 0; i < srcEnd - srcBegin; i++) {
		dst[dstBegin + i] = this.charAt (srcBegin + i);
	}
};

String.indexOf = function (source, sourceOffset, sourceCount,
		target, targetOffset, targetCount, fromIndex) {
	if (fromIndex >= sourceCount) {
		return (targetCount == 0 ? sourceCount : -1);
	}
	if (fromIndex < 0) {
		fromIndex = 0;
	}
	if (targetCount == 0) {
		return fromIndex;
	}

	var first  = target[targetOffset];
	var i = sourceOffset + fromIndex;
	var max = sourceOffset + (sourceCount - targetCount);

	startSearchForFirstChar:
	while (true) {
		/* Look for first character. */
		while (i <= max && source[i] != first) {
			i++;
		}
		if (i > max) {
			return -1;
		}

		/* Found first character, now look at the rest of v2 */
		var j = i + 1;
		var end = j + targetCount - 1;
		var k = targetOffset + 1;
		while (j < end) {
			if (source[j++] != target[k++]) {
				i++;
				/* Look for str's first char again. */
				continue startSearchForFirstChar;
			}
		}
		return i - sourceOffset;	/* Found whole string. */
	}
};

String.instantialize = function () {
	if (arguments.length == 0) {
		return new String ();
	} else if (arguments.length == 1) {
		var x = arguments[0];
		if (typeof x == "string" || x instanceof String) {
			return new String (x);
		} else if (x instanceof Array) {
			if (x.length > 0 && typeof x[0] == "number") {
				var arr = new Array (x.length);
				for (var i = 0; i < x.length; i++) {
					arr[i] = String.fromCharCode (x[i] & 0xff);
				}
				return Encoding.readUTF8 (arr.join (''));
			}
			return x.join ('');
		} else if (x.__CLASS_NAME__ == "StringBuffer") {
            x.setShared();
            var value = x.getValue();
			var length = x.length ();
			var valueCopy = new Array (length);
			for (var i = 0; i < length; i++) {
				valueCopy[i] = value[i];
			}
			return valueCopy.join ('')
			//return x.value.join ('');
		} else {
			return "" + x;
		}
	} else if (arguments.length == 2) {
		var x = arguments[0];
		var y = arguments[1];
		return String.instantialize (x, 0, x.length, y);
	} else if (arguments.length == 3) {
		var bytes = arguments[0];
		var offset = arguments[1];
		var length = arguments[2];
		var arr = new Array (length);
		for (var i = 0; i < length; i++) {
			arr[i] = bytes[offset + i];
		}
		return arr.join ('');
	} else if (arguments.length == 4) {
		var bytes = arguments[0];
		var y = arguments[3];
		if (typeof y == "string" || y instanceof String) {
			var offset = arguments[1];
			var length = arguments[2];
			var arr = new Array (length);
			for (var i = 0; i < length; i++) {
				arr[i] = bytes[offset + i];
				if (typeof arr[i] == "number") {
					arr[i] = String.fromCharCode (arr[i] & 0xff);
				}
			}
			if (y.toLowerCase () == "utf-8") {
				return Encoding.readUTF8 (arr.join (''));
			} else {
				return arr.join ('');
			}
		} else {
			var value = new Array (count);

			if (hibyte == 0) {
				for (var i = count ; i-- > 0 ;) {
					value[i] = String.fromCharCode (ascii[i + offset] & 0xff);
				}
			} else {
				hibyte <<= 8;
				for (var i = count ; i-- > 0 ;) {
					value[i] = String.fromCharCode (hibyte | (ascii[i + offset] & 0xff));
				}
			}
			return value.join ('');
		}
	} else {
		var s = "";
		for (var i = 0; i < arguments.length; i++) {
		s += arguments[i];
		}
		return s;
	}
};
Clazz.declarePackage ("java.lang");
java.lang.Runnable = Runnable = function () {
};
Runnable.__CLASS_NAME__ = Runnable.prototype.__CLASS_NAME__ = "Runnable";
Clazz.declarePackage ("java.lang");
java.lang.Cloneable = Cloneable = function () {
};
Cloneable.__CLASS_NAME__ = Cloneable.prototype.__CLASS_NAME__ = "Cloneable";
Clazz.declarePackage ("java.io");
java.io.Serializable = function () {
};
java.io.Serializable.__CLASS_NAME__ = java.io.Serializable.prototype.__CLASS_NAME__ = "java.io.Serializable";
Clazz.declarePackage ("java.lang");
java.lang.Comparable = Comparable = function () {
};
Comparable.__CLASS_NAME__ = Comparable.prototype.__CLASS_NAME__ = "Comparable";
Clazz.declarePackage ("java.lang");
java.lang.CharSequence = CharSequence = function () {
};
CharSequence.__CLASS_NAME__ = CharSequence.prototype.__CLASS_NAME__ = "CharSequence";
Clazz.declarePackage ("java.lang");
java.lang.StringBuffer = StringBuffer = function () {
this.value = null;
this.count = 0;
this.shared = false;
this.serialVersionUID = StringBuffer.serialVersionUID;
Clazz.makeConstructor (StringBuffer, 
function () {
this.construct (16);
});
Clazz.makeConstructor (StringBuffer, 
function (length) {
this.value =  Clazz.newArray (length, '\0');
this.shared = false;
}, "Number");
Clazz.makeConstructor (StringBuffer, 
function (str) {
this.construct (str.length + 16);
this.append (str);
}, "String");
Clazz.defineMethod (StringBuffer, "length", 
function () {
return this.count;
});
Clazz.defineMethod (StringBuffer, "capacity", 
function () {
return this.value.length;
});
Clazz.defineMethod (StringBuffer, "copy", 
function () {
var newValue =  Clazz.newArray (this.value.length, '\0');
System.arraycopy (this.value, 0, newValue, 0, this.count);
this.value = newValue;
this.shared = false;
});
Clazz.defineMethod (StringBuffer, "ensureCapacity", 
function (minimumCapacity) {
if (minimumCapacity > this.value.length) {
this.expandCapacity (minimumCapacity);
}}, "Number");
Clazz.defineMethod (StringBuffer, "expandCapacity", 
function (minimumCapacity) {
var newCapacity = (this.value.length + 1) * 2;
if (newCapacity < 0) {
newCapacity = Integer.MAX_VALUE;
} else if (minimumCapacity > newCapacity) {
newCapacity = minimumCapacity;
}var newValue =  Clazz.newArray (newCapacity, '\0');
System.arraycopy (this.value, 0, newValue, 0, this.count);
this.value = newValue;
this.shared = false;
}, "Number");
Clazz.defineMethod (StringBuffer, "setLength", 
function (newLength) {
if (newLength < 0) {
throw  new StringIndexOutOfBoundsException (newLength);
}if (newLength > this.value.length) {
this.expandCapacity (newLength);
}if (this.count < newLength) {
if (this.shared) this.copy ();
for (; this.count < newLength; this.count++) {
this.value[this.count] = '\0';
}
} else {
this.count = newLength;
if (this.shared) {
if (newLength > 0) {
this.copy ();
} else {
this.value =  Clazz.newArray (16, '\0');
this.shared = false;
}}}}, "Number");
Clazz.defineMethod (StringBuffer, "charAt", 
function (index) {
if ((index < 0) || (index >= this.count)) {
throw  new StringIndexOutOfBoundsException (index);
}return this.value[index];
}, "Number");
Clazz.defineMethod (StringBuffer, "getChars", 
function (srcBegin, srcEnd, dst, dstBegin) {
if (srcBegin < 0) {
throw  new StringIndexOutOfBoundsException (srcBegin);
}if ((srcEnd < 0) || (srcEnd > this.count)) {
throw  new StringIndexOutOfBoundsException (srcEnd);
}if (srcBegin > srcEnd) {
throw  new StringIndexOutOfBoundsException ("srcBegin > srcEnd");
}System.arraycopy (this.value, srcBegin, dst, dstBegin, srcEnd - srcBegin);
}, "Number, Number, Array, Number");
Clazz.defineMethod (StringBuffer, "setCharAt", 
function (index, ch) {
if ((index < 0) || (index >= this.count)) {
throw  new StringIndexOutOfBoundsException (index);
}if (this.shared) this.copy ();
this.value[index] = ch;
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "append", 
function (obj) {
return this.append (String.valueOf (obj));
}, "Object");
Clazz.defineMethod (StringBuffer, "append", 
function (str) {
if (str == null) {
str = String.valueOf (str);
}var len = str.length;
var newcount = this.count + len;
if (newcount > this.value.length) this.expandCapacity (newcount);
str.getChars (0, len, this.value, this.count);
this.count = newcount;
return this;
}, "String");
Clazz.defineMethod (StringBuffer, "append", 
function (sb) {
if (sb == null) {
sb = StringBuffer.NULL;
}var len = sb.length ();
var newcount = this.count + len;
if (newcount > this.value.length) this.expandCapacity (newcount);
sb.getChars (0, len, this.value, this.count);
this.count = newcount;
return this;
}, "StringBuffer");
this.NULL = StringBuffer.NULL;
Clazz.defineMethod (StringBuffer, "append", 
function (str) {
var len = str.length;
var newcount = this.count + len;
if (newcount > this.value.length) this.expandCapacity (newcount);
System.arraycopy (str, 0, this.value, this.count, len);
this.count = newcount;
return this;
}, "Array");
Clazz.defineMethod (StringBuffer, "append", 
function (str, offset, len) {
var newcount = this.count + len;
if (newcount > this.value.length) this.expandCapacity (newcount);
System.arraycopy (str, offset, this.value, this.count, len);
this.count = newcount;
return this;
}, "Array, Number, Number");
Clazz.defineMethod (StringBuffer, "append", 
function (b) {
if (b) {
var newcount = this.count + 4;
if (newcount > this.value.length) this.expandCapacity (newcount);
this.value[this.count++] = 't';
this.value[this.count++] = 'r';
this.value[this.count++] = 'u';
this.value[this.count++] = 'e';
} else {
var newcount = this.count + 5;
if (newcount > this.value.length) this.expandCapacity (newcount);
this.value[this.count++] = 'f';
this.value[this.count++] = 'a';
this.value[this.count++] = 'l';
this.value[this.count++] = 's';
this.value[this.count++] = 'e';
}return this;
}, "Boolean");
Clazz.defineMethod (StringBuffer, "append", 
function (c) {
var newcount = this.count + 1;
if (newcount > this.value.length) this.expandCapacity (newcount);
this.value[this.count++] = c;
return this;
}, "Number");
Clazz.defineMethod (StringBuffer, "append", 
function (i) {
this.append ("" + i);
return this;
}, "Number");
Clazz.defineMethod (StringBuffer, "append", 
function (l) {
this.append ("" + l);
return this;
}, "Number");
Clazz.defineMethod (StringBuffer, "append", 
function (f) {
this.append ("" + f);
return this;
}, "Number");
Clazz.defineMethod (StringBuffer, "append", 
function (d) {
this.append ("" + d);
return this;
}, "Number");
Clazz.defineMethod (StringBuffer, "$delete", 
function (start, end) {
if (start < 0) throw  new StringIndexOutOfBoundsException (start);
if (end > this.count) end = this.count;
if (start > end) throw  new StringIndexOutOfBoundsException ();
var len = end - start;
if (len > 0) {
if (this.shared) this.copy ();
System.arraycopy (this.value, start + len, this.value, start, this.count - end);
this.count -= len;
}return this;
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "deleteCharAt", 
function (index) {
if ((index < 0) || (index >= this.count)) throw  new StringIndexOutOfBoundsException ();
if (this.shared) this.copy ();
System.arraycopy (this.value, index + 1, this.value, index, this.count - index - 1);
this.count--;
return this;
}, "Number");
Clazz.defineMethod (StringBuffer, "replace", 
function (start, end, str) {
if (start < 0) throw  new StringIndexOutOfBoundsException (start);
if (end > this.count) end = this.count;
if (start > end) throw  new StringIndexOutOfBoundsException ();
var len = str.length;
var newCount = this.count + len - (end - start);
if (newCount > this.value.length) this.expandCapacity (newCount);
 else if (this.shared) this.copy ();
System.arraycopy (this.value, end, this.value, start + len, this.count - end);
str.getChars (0, len, this.value, start);
this.count = newCount;
return this;
}, "Number, Number, String");
Clazz.defineMethod (StringBuffer, "substring", 
function (start) {
return this.substring (start, this.count);
}, "Number");
Clazz.defineMethod (StringBuffer, "subSequence", 
function (start, end) {
return this.substring (start, end);
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "substring", 
function (start, end) {
if (start < 0) throw  new StringIndexOutOfBoundsException (start);
if (end > this.count) throw  new StringIndexOutOfBoundsException (end);
if (start > end) throw  new StringIndexOutOfBoundsException (end - start);
return  String.instantialize (this.value, start, end - start);
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "insert", 
function (index, str, offset, len) {
if ((index < 0) || (index > this.count)) throw  new StringIndexOutOfBoundsException ();
if ((offset < 0) || (offset + len < 0) || (offset + len > str.length)) throw  new StringIndexOutOfBoundsException (offset);
if (len < 0) throw  new StringIndexOutOfBoundsException (len);
var newCount = this.count + len;
if (newCount > this.value.length) this.expandCapacity (newCount);
 else if (this.shared) this.copy ();
System.arraycopy (this.value, index, this.value, index + len, this.count - index);
System.arraycopy (str, offset, this.value, index, len);
this.count = newCount;
return this;
}, "Number, Array, Number, Number");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, obj) {
return this.insert (offset, String.valueOf (obj));
}, "Number, Object");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, str) {
if ((offset < 0) || (offset > this.count)) {
throw  new StringIndexOutOfBoundsException ();
}if (str == null) {
str = String.valueOf (str);
}var len = str.length;
var newcount = this.count + len;
if (newcount > this.value.length) this.expandCapacity (newcount);
 else if (this.shared) this.copy ();
System.arraycopy (this.value, offset, this.value, offset + len, this.count - offset);
str.getChars (0, len, this.value, offset);
this.count = newcount;
return this;
}, "Number, String");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, str) {
if ((offset < 0) || (offset > this.count)) {
throw  new StringIndexOutOfBoundsException ();
}var len = str.length;
var newcount = this.count + len;
if (newcount > this.value.length) this.expandCapacity (newcount);
 else if (this.shared) this.copy ();
System.arraycopy (this.value, offset, this.value, offset + len, this.count - offset);
System.arraycopy (str, 0, this.value, offset, len);
this.count = newcount;
return this;
}, "Number, Array");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, b) {
return this.insert (offset, String.valueOf (b));
}, "Number, Boolean");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, c) {
var newcount = this.count + 1;
if (newcount > this.value.length) this.expandCapacity (newcount);
 else if (this.shared) this.copy ();
System.arraycopy (this.value, offset, this.value, offset + 1, this.count - offset);
this.value[offset] = c;
this.count = newcount;
return this;
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, i) {
return this.insert (offset, String.valueOf (i));
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, l) {
return this.insert (offset, String.valueOf (l));
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, f) {
return this.insert (offset, String.valueOf (f));
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "insert", 
function (offset, d) {
return this.insert (offset, String.valueOf (d));
}, "Number, Number");
Clazz.defineMethod (StringBuffer, "indexOf", 
function (str) {
return this.indexOf (str, 0);
}, "String");
Clazz.defineMethod (StringBuffer, "indexOf", 
function (str, fromIndex) {
return String.indexOf (this.value, 0, this.count, str.toCharArray (), 0, str.length, fromIndex);
}, "String, Number");
Clazz.defineMethod (StringBuffer, "lastIndexOf", 
function (str) {
return this.lastIndexOf (str, this.count);
}, "String");
Clazz.defineMethod (StringBuffer, "lastIndexOf", 
function (str, fromIndex) {
return String.lastIndexOf (this.value, 0, this.count, str.toCharArray (), 0, str.length, fromIndex);
}, "String, Number");
Clazz.defineMethod (StringBuffer, "reverse", 
function () {
if (this.shared) this.copy ();
var n = this.count - 1;
for (var j = (n - 1) >> 1; j >= 0; --j) {
var temp = this.value[j];
this.value[j] = this.value[n - j];
this.value[n - j] = temp;
}
return this;
});
Clazz.defineMethod (StringBuffer, "toString", 
function () {
return  String.instantialize (this);
});
Clazz.defineMethod (StringBuffer, "setShared", 
function () {
this.shared = true;
});
Clazz.defineMethod (StringBuffer, "getValue", 
function () {
return this.value;
});
Clazz.defineMethod (StringBuffer, "readObject", 
function (s) {
s.defaultReadObject ();
this.value = this.value.clone ();
this.shared = false;
}, "java.io.ObjectInputStream");
Clazz.instantialize (this, arguments);
};
StringBuffer.__CLASS_NAME__ = StringBuffer.prototype.__CLASS_NAME__ = "StringBuffer";
Clazz.implementOf (StringBuffer, java.io.Serializable);
Clazz.implementOf (StringBuffer, CharSequence);
StringBuffer.serialVersionUID = 3388685877147921107;
StringBuffer.NULL =  new StringBuffer ("null");
Clazz.declarePackage ("java.util");
java.util.Comparator = function () {
};
java.util.Comparator.__CLASS_NAME__ = java.util.Comparator.prototype.__CLASS_NAME__ = "java.util.Comparator";
Clazz.declarePackage ("java.util");
java.util.Iterator = function () {
};
java.util.Iterator.__CLASS_NAME__ = java.util.Iterator.prototype.__CLASS_NAME__ = "java.util.Iterator";
Clazz.declarePackage ("java.util");
java.util.ListIterator = function () {
};
java.util.ListIterator.__CLASS_NAME__ = java.util.ListIterator.prototype.__CLASS_NAME__ = "java.util.ListIterator";
Clazz.implementOf (java.util.ListIterator, java.util.Iterator);
Clazz.declarePackage ("java.util");
java.util.Enumeration = function () {
};
java.util.Enumeration.__CLASS_NAME__ = java.util.Enumeration.prototype.__CLASS_NAME__ = "java.util.Enumeration";
Clazz.declarePackage ("java.util");
java.util.Collection = function () {
};
java.util.Collection.__CLASS_NAME__ = java.util.Collection.prototype.__CLASS_NAME__ = "java.util.Collection";
Clazz.declarePackage ("java.util");
java.util.Set = function () {
};
java.util.Set.__CLASS_NAME__ = java.util.Set.prototype.__CLASS_NAME__ = "java.util.Set";
Clazz.implementOf (java.util.Set, java.util.Collection);
Clazz.declarePackage ("java.util");
java.util.Map = function () {
};
java.util.Map.__CLASS_NAME__ = java.util.Map.prototype.__CLASS_NAME__ = "java.util.Map";
java.util.Map.Entry = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
};
java.util.Map.Entry.__CLASS_NAME__ = java.util.Map.Entry.prototype.__CLASS_NAME__ = "java.util.Map.Entry";
Clazz.declarePackage ("java.util");
java.util.List = function () {
};
java.util.List.__CLASS_NAME__ = java.util.List.prototype.__CLASS_NAME__ = "java.util.List";
Clazz.implementOf (java.util.List, java.util.Collection);
Clazz.declarePackage ("java.util");
java.util.RandomAccess = function () {
};
java.util.RandomAccess.__CLASS_NAME__ = java.util.RandomAccess.prototype.__CLASS_NAME__ = "java.util.RandomAccess";
Clazz.declarePackage ("java.util");
java.util.AbstractCollection = function () {
Clazz.makeConstructor (java.util.AbstractCollection, 
function () {
});
Clazz.defineMethod (java.util.AbstractCollection, "isEmpty", 
function () {
return this.size () == 0;
});
Clazz.defineMethod (java.util.AbstractCollection, "contains", 
function (o) {
var e = this.iterator ();
if (o == null) {
while (e.hasNext ()) if (e.next () == null) return true;

} else {
while (e.hasNext ()) if (o.equals (e.next ())) return true;

}return false;
}, "Object");
Clazz.defineMethod (java.util.AbstractCollection, "toArray", 
function () {
var result =  new Array (this.size ());
var e = this.iterator ();
for (var i = 0; e.hasNext (); i++) result[i] = e.next ();

return result;
});
Clazz.defineMethod (java.util.AbstractCollection, "toArray", 
function (a) {
var size = this.size ();
if (a.length < size) a = java.lang.reflect.Array.newInstance (a.getClass ().getComponentType (), size);
var it = this.iterator ();
for (var i = 0; i < size; i++) a[i] = it.next ();

if (a.length > size) a[size] = null;
return a;
}, "Array");
Clazz.defineMethod (java.util.AbstractCollection, "add", 
function (o) {
throw  new UnsupportedOperationException ();
}, "Object");
Clazz.defineMethod (java.util.AbstractCollection, "remove", 
function (o) {
var e = this.iterator ();
if (o == null) {
while (e.hasNext ()) {
if (e.next () == null) {
e.remove ();
return true;
}}
} else {
while (e.hasNext ()) {
if (o.equals (e.next ())) {
e.remove ();
return true;
}}
}return false;
}, "Object");
Clazz.defineMethod (java.util.AbstractCollection, "containsAll", 
function (c) {
var e = c.iterator ();
while (e.hasNext ()) if (!this.contains (e.next ())) return false;

return true;
}, "java.util.Collection");
Clazz.defineMethod (java.util.AbstractCollection, "addAll", 
function (c) {
var modified = false;
var e = c.iterator ();
while (e.hasNext ()) {
if (this.add (e.next ())) modified = true;
}
return modified;
}, "java.util.Collection");
Clazz.defineMethod (java.util.AbstractCollection, "removeAll", 
function (c) {
var modified = false;
var e = this.iterator ();
while (e.hasNext ()) {
if (c.contains (e.next ())) {
e.remove ();
modified = true;
}}
return modified;
}, "java.util.Collection");
Clazz.defineMethod (java.util.AbstractCollection, "retainAll", 
function (c) {
var modified = false;
var e = this.iterator ();
while (e.hasNext ()) {
if (!c.contains (e.next ())) {
e.remove ();
modified = true;
}}
return modified;
}, "java.util.Collection");
Clazz.defineMethod (java.util.AbstractCollection, "clear", 
function () {
var e = this.iterator ();
while (e.hasNext ()) {
e.next ();
e.remove ();
}
});
Clazz.defineMethod (java.util.AbstractCollection, "toString", 
function () {
var buf =  new StringBuffer ();
buf.append ("[");
var i = this.iterator ();
var hasNext = i.hasNext ();
while (hasNext) {
var o = i.next ();
buf.append (o == this ? "(this Collection)" : String.valueOf (o));
hasNext = i.hasNext ();
if (hasNext) buf.append (", ");
}
buf.append ("]");
return buf.toString ();
});
Clazz.instantialize (this, arguments);
};
java.util.AbstractCollection.__CLASS_NAME__ = java.util.AbstractCollection.prototype.__CLASS_NAME__ = "java.util.AbstractCollection";
Clazz.implementOf (java.util.AbstractCollection, java.util.Collection);
Clazz.declarePackage ("java.util");
java.util.AbstractSet = function () {
Clazz.makeConstructor (java.util.AbstractSet, 
function () {
Clazz.superConstructor (this, java.util.AbstractSet, []);
});
Clazz.defineMethod (java.util.AbstractSet, "equals", 
function (o) {
if (o == this) return true;
if (!(Clazz.instanceOf (o, java.util.Set))) return false;
var c = o;
if (c.size () != this.size ()) return false;
try {
return this.containsAll (c);
} catch (e) {
if (Clazz.instanceOf (e, ClassCastException)) {
return false;
} else if (Clazz.instanceOf (e, NullPointerException)) {
return false;
}
}

}, "Object");
Clazz.defineMethod (java.util.AbstractSet, "hashCode", 
function () {
var h = 0;
var i = this.iterator ();
while (i.hasNext ()) {
var obj = i.next ();
if (obj != null) h += obj.hashCode ();
}
return h;
});
Clazz.defineMethod (java.util.AbstractSet, "removeAll", 
function (c) {
var modified = false;
if (this.size () > c.size ()) {
for (var i = c.iterator (); i.hasNext (); ) modified = new Boolean (modified | this.remove (i.next ()));

} else {
for (var i = this.iterator (); i.hasNext (); ) {
if (c.contains (i.next ())) {
i.remove ();
modified = true;
}}
}return modified;
}, "java.util.Collection");
Clazz.instantialize (this, arguments);
};
java.util.AbstractSet.__CLASS_NAME__ = java.util.AbstractSet.prototype.__CLASS_NAME__ = "java.util.AbstractSet";
Clazz.inheritClass (java.util.AbstractSet, java.util.AbstractCollection);
Clazz.implementOf (java.util.AbstractSet, java.util.Set);
Clazz.declarePackage ("java.util");
java.util.AbstractMap = function () {
Clazz.makeConstructor (java.util.AbstractMap, 
function () {
});
Clazz.defineMethod (java.util.AbstractMap, "size", 
function () {
return this.entrySet ().size ();
});
Clazz.defineMethod (java.util.AbstractMap, "isEmpty", 
function () {
return this.size () == 0;
});
Clazz.defineMethod (java.util.AbstractMap, "containsValue", 
function (value) {
var i = this.entrySet ().iterator ();
if (value == null) {
while (i.hasNext ()) {
var e = i.next ();
if (e.getValue () == null) return true;
}
} else {
while (i.hasNext ()) {
var e = i.next ();
if (value.equals (e.getValue ())) return true;
}
}return false;
}, "Object");
Clazz.defineMethod (java.util.AbstractMap, "containsKey", 
function (key) {
var i = this.entrySet ().iterator ();
if (key == null) {
while (i.hasNext ()) {
var e = i.next ();
if (e.getKey () == null) return true;
}
} else {
while (i.hasNext ()) {
var e = i.next ();
if (key.equals (e.getKey ())) return true;
}
}return false;
}, "Object");
Clazz.defineMethod (java.util.AbstractMap, "get", 
function (key) {
var i = this.entrySet ().iterator ();
if (key == null) {
while (i.hasNext ()) {
var e = i.next ();
if (e.getKey () == null) return e.getValue ();
}
} else {
while (i.hasNext ()) {
var e = i.next ();
if (key.equals (e.getKey ())) return e.getValue ();
}
}return null;
}, "Object");
Clazz.defineMethod (java.util.AbstractMap, "put", 
function (key, value) {
throw  new UnsupportedOperationException ();
}, "Object, Object");
Clazz.defineMethod (java.util.AbstractMap, "remove", 
function (key) {
var i = this.entrySet ().iterator ();
var correctEntry = null;
if (key == null) {
while (correctEntry == null && i.hasNext ()) {
var e = i.next ();
if (e.getKey () == null) correctEntry = e;
}
} else {
while (correctEntry == null && i.hasNext ()) {
var e = i.next ();
if (key.equals (e.getKey ())) correctEntry = e;
}
}var oldValue = null;
if (correctEntry != null) {
oldValue = correctEntry.getValue ();
i.remove ();
}return oldValue;
}, "Object");
Clazz.defineMethod (java.util.AbstractMap, "putAll", 
function (t) {
var i = t.entrySet ().iterator ();
while (i.hasNext ()) {
var e = i.next ();
this.put (e.getKey (), e.getValue ());
}
}, "java.util.Map");
Clazz.defineMethod (java.util.AbstractMap, "clear", 
function () {
this.entrySet ().clear ();
});
this.$keySet = null;
this.$values = null;
Clazz.defineMethod (java.util.AbstractMap, "keySet", 
function () {
if (this.$keySet == null) {
this.$keySet = (function (innerThis) {
if (!Clazz.isClassDefined ("java.util.AbstractMap$1")) {
java.util.AbstractMap$1 = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.AbstractMap$1, "iterator", 
function () {
return (function (innerThis) {
if (!Clazz.isClassDefined ("java.util.AbstractMap$1$2")) {
java.util.AbstractMap$1$2 = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.i = this.callbacks["java.util.AbstractMap"].entrySet ().iterator ();
Clazz.defineMethod (java.util.AbstractMap$1$2, "hasNext", 
function () {
return this.i.hasNext ();
});
Clazz.defineMethod (java.util.AbstractMap$1$2, "next", 
function () {
return (this.i.next ()).getKey ();
});
Clazz.defineMethod (java.util.AbstractMap$1$2, "remove", 
function () {
this.i.remove ();
});
Clazz.instantialize (this, arguments);
};
Clazz.implementOf (java.util.AbstractMap$1$2, java.util.Iterator);
java.util.AbstractMap$1$2.__CLASS_NAME__ = java.util.AbstractMap$1$2.prototype.__CLASS_NAME__ = "java.util.AbstractMap$1$2";
}
return Clazz.innerTypeInstance (java.util.AbstractMap$1$2, innerThis);
}) (this);
});
Clazz.defineMethod (java.util.AbstractMap$1, "size", 
function () {
return this.callbacks["java.util.AbstractMap"].size ();
});
Clazz.defineMethod (java.util.AbstractMap$1, "contains", 
function (k) {
return this.callbacks["java.util.AbstractMap"].containsKey (k);
}, "Object");
Clazz.instantialize (this, arguments);
};
Clazz.inheritClass (java.util.AbstractMap$1, java.util.AbstractSet);
java.util.AbstractMap$1.__CLASS_NAME__ = java.util.AbstractMap$1.prototype.__CLASS_NAME__ = "java.util.AbstractMap$1";
}
return Clazz.innerTypeInstance (java.util.AbstractMap$1, innerThis);
}) (this);
}return this.$keySet;
});
Clazz.defineMethod (java.util.AbstractMap, "values", 
function () {
if (this.$values == null) {
this.$values = (function (innerThis) {
if (!Clazz.isClassDefined ("java.util.AbstractMap$3")) {
java.util.AbstractMap$3 = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.AbstractMap$3, "iterator", 
function () {
return (function (innerThis) {
if (!Clazz.isClassDefined ("java.util.AbstractMap$3$4")) {
java.util.AbstractMap$3$4 = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.i = this.callbacks["java.util.AbstractMap"].entrySet ().iterator ();
Clazz.defineMethod (java.util.AbstractMap$3$4, "hasNext", 
function () {
return this.i.hasNext ();
});
Clazz.defineMethod (java.util.AbstractMap$3$4, "next", 
function () {
return (this.i.next ()).getValue ();
});
Clazz.defineMethod (java.util.AbstractMap$3$4, "remove", 
function () {
this.i.remove ();
});
Clazz.instantialize (this, arguments);
};
Clazz.implementOf (java.util.AbstractMap$3$4, java.util.Iterator);
java.util.AbstractMap$3$4.__CLASS_NAME__ = java.util.AbstractMap$3$4.prototype.__CLASS_NAME__ = "java.util.AbstractMap$3$4";
}
return Clazz.innerTypeInstance (java.util.AbstractMap$3$4, innerThis);
}) (this);
});
Clazz.defineMethod (java.util.AbstractMap$3, "size", 
function () {
return this.callbacks["java.util.AbstractMap"].size ();
});
Clazz.defineMethod (java.util.AbstractMap$3, "contains", 
function (v) {
return this.callbacks["java.util.AbstractMap"].containsValue (v);
}, "Object");
Clazz.instantialize (this, arguments);
};
Clazz.inheritClass (java.util.AbstractMap$3, java.util.AbstractCollection);
java.util.AbstractMap$3.__CLASS_NAME__ = java.util.AbstractMap$3.prototype.__CLASS_NAME__ = "java.util.AbstractMap$3";
}
return Clazz.innerTypeInstance (java.util.AbstractMap$3, innerThis);
}) (this);
}return this.$values;
});
Clazz.defineMethod (java.util.AbstractMap, "equals", 
function (o) {
if (o == this) return true;
if (!(Clazz.instanceOf (o, java.util.Map))) return false;
var t = o;
if (t.size () != this.size ()) return false;
try {
var i = this.entrySet ().iterator ();
while (i.hasNext ()) {
var e = i.next ();
var key = e.getKey ();
var value = e.getValue ();
if (value == null) {
if (!(t.get (key) == null && t.containsKey (key))) return false;
} else {
if (!value.equals (t.get (key))) return false;
}}
} catch (e) {
if (Clazz.instanceOf (e, ClassCastException)) {
return false;
} else if (Clazz.instanceOf (e, NullPointerException)) {
return false;
}
}

return true;
}, "Object");
Clazz.defineMethod (java.util.AbstractMap, "hashCode", 
function () {
var h = 0;
var i = this.entrySet ().iterator ();
while (i.hasNext ()) h += i.next ().hashCode ();

return h;
});
Clazz.defineMethod (java.util.AbstractMap, "toString", 
function () {
var buf =  new StringBuffer ();
buf.append ("{");
var i = this.entrySet ().iterator ();
var hasNext = i.hasNext ();
while (hasNext) {
var e = (i.next ());
var key = e.getKey ();
var value = e.getValue ();
buf.append ((key == this ? "(this Map)" : key) + "=" + (value == this ? "(this Map)" : value));
hasNext = i.hasNext ();
if (hasNext) buf.append (", ");
}
buf.append ("}");
return buf.toString ();
});
Clazz.defineMethod (java.util.AbstractMap, "clone", 
function () {
var result = Clazz.superCall (this, java.util.AbstractMap, "clone", []);
result.keySet = null;
result.values = null;
return result;
});
Clazz.instantialize (this, arguments);
};
java.util.AbstractMap.__CLASS_NAME__ = java.util.AbstractMap.prototype.__CLASS_NAME__ = "java.util.AbstractMap";
Clazz.implementOf (java.util.AbstractMap, java.util.Map);
java.util.AbstractMap.SimpleEntry = function () {
this.key = null;
this.value = null;
Clazz.makeConstructor (java.util.AbstractMap.SimpleEntry, 
function (key, value) {
this.key = key;
this.value = value;
}, "Object, Object");
Clazz.makeConstructor (java.util.AbstractMap.SimpleEntry, 
function (e) {
this.key = e.getKey ();
this.value = e.getValue ();
}, "java.util.Map.Entry");
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "getKey", 
function () {
return this.key;
});
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "getValue", 
function () {
return this.value;
});
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "setValue", 
function (value) {
var oldValue = this.value;
this.value = value;
return oldValue;
}, "Object");
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "equals", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return false;
var e = o;
return java.util.AbstractMap.SimpleEntry.eq (this.key, e.getKey ()) && java.util.AbstractMap.SimpleEntry.eq (this.value, e.getValue ());
}, "Object");
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "hashCode", 
function () {
var v;
return ((this.key == null) ? 0 : this.key.hashCode ()) ^ ((this.value == null) ? 0 : this.value.hashCode ());
});
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "toString", 
function () {
return this.key + "=" + this.value;
});
Clazz.instantialize (this, arguments);
};
java.util.AbstractMap.SimpleEntry.__CLASS_NAME__ = java.util.AbstractMap.SimpleEntry.prototype.__CLASS_NAME__ = "java.util.AbstractMap.SimpleEntry";
Clazz.implementOf (java.util.AbstractMap.SimpleEntry, java.util.Map.Entry);
Clazz.defineMethod (java.util.AbstractMap.SimpleEntry, "eq", 
function (o1, o2) {
return (o1 == null ? o2 == null : o1.equals (o2));
}, "Object, Object");
java.util.AbstractMap.SimpleEntry.eq = java.util.AbstractMap.SimpleEntry.prototype.eq;
Clazz.declarePackage ("java.util");
java.util.AbstractList = function () {
Clazz.makeConstructor (java.util.AbstractList, 
function () {
Clazz.superConstructor (this, java.util.AbstractList, []);
});
Clazz.defineMethod (java.util.AbstractList, "add", 
function (o) {
this.add (this.size (), o);
return true;
}, "Object");
Clazz.defineMethod (java.util.AbstractList, "set", 
function (index, element) {
throw  new UnsupportedOperationException ();
}, "Number, Object");
Clazz.defineMethod (java.util.AbstractList, "add", 
function (index, element) {
throw  new UnsupportedOperationException ();
}, "Number, Object");
Clazz.defineMethod (java.util.AbstractList, "remove", 
function (index) {
throw  new UnsupportedOperationException ();
}, "Number");
Clazz.defineMethod (java.util.AbstractList, "indexOf", 
function (o) {
var e = this.listIterator ();
if (o == null) {
while (e.hasNext ()) if (e.next () == null) return e.previousIndex ();

} else {
while (e.hasNext ()) if (o.equals (e.next ())) return e.previousIndex ();

}return -1;
}, "Object");
Clazz.defineMethod (java.util.AbstractList, "lastIndexOf", 
function (o) {
var e = this.listIterator (this.size ());
if (o == null) {
while (e.hasPrevious ()) if (e.previous () == null) return e.nextIndex ();

} else {
while (e.hasPrevious ()) if (o.equals (e.previous ())) return e.nextIndex ();

}return -1;
}, "Object");
Clazz.defineMethod (java.util.AbstractList, "clear", 
function () {
this.removeRange (0, this.size ());
});
Clazz.defineMethod (java.util.AbstractList, "addAll", 
function (index, c) {
var modified = false;
var e = c.iterator ();
while (e.hasNext ()) {
this.add (index++, e.next ());
modified = true;
}
return modified;
}, "Number, java.util.Collection");
Clazz.defineMethod (java.util.AbstractList, "iterator", 
function () {
return Clazz.innerTypeInstance (java.util.AbstractList.Itr, this);
});
Clazz.defineMethod (java.util.AbstractList, "listIterator", 
function () {
return this.listIterator (0);
});
Clazz.defineMethod (java.util.AbstractList, "listIterator", 
function (index) {
if (index < 0 || index > this.size ()) throw  new IndexOutOfBoundsException ("Index: " + index);
return Clazz.innerTypeInstance (java.util.AbstractList.ListItr, this, index);
}, "Number");
if (!Clazz.isClassDefined ("java.util.AbstractList.Itr")) {
java.util.AbstractList.Itr = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.cursor = 0;
this.lastRet = -1;
this.expectedModCount = this.callbacks["java.util.AbstractList"].modCount;
Clazz.defineMethod (java.util.AbstractList.Itr, "hasNext", 
function () {
return this.cursor != this.callbacks["java.util.AbstractList"].size ();
});
Clazz.defineMethod (java.util.AbstractList.Itr, "next", 
function () {
this.checkForComodification ();
try {
var next = this.callbacks["java.util.AbstractList"].get (this.cursor);
this.lastRet = this.cursor++;
return next;
} catch (e) {
if (Clazz.instanceOf (e, IndexOutOfBoundsException)) {
this.checkForComodification ();
throw  new java.util.NoSuchElementException ();
}
}

});
Clazz.defineMethod (java.util.AbstractList.Itr, "remove", 
function () {
if (this.lastRet == -1) throw  new IllegalStateException ();
this.checkForComodification ();
try {
this.callbacks["java.util.AbstractList"].remove (this.lastRet);
if (this.lastRet < this.cursor) this.cursor--;
this.lastRet = -1;
this.expectedModCount = this.callbacks["java.util.AbstractList"].modCount;
} catch (e) {
if (Clazz.instanceOf (e, IndexOutOfBoundsException)) {
throw  new java.util.ConcurrentModificationException ();
}
}

});
Clazz.defineMethod (java.util.AbstractList.Itr, "checkForComodification", 
function () {
if (this.callbacks["java.util.AbstractList"].modCount != this.expectedModCount) throw  new java.util.ConcurrentModificationException ();
});
Clazz.instantialize (this, arguments);
};
java.util.AbstractList.Itr.__CLASS_NAME__ = java.util.AbstractList.Itr.prototype.__CLASS_NAME__ = "java.util.AbstractList.Itr";
Clazz.implementOf (java.util.AbstractList.Itr, java.util.Iterator);
}
if (!Clazz.isClassDefined ("java.util.AbstractList.ListItr")) {
java.util.AbstractList.ListItr = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.makeConstructor (java.util.AbstractList.ListItr, 
function (index) {
Clazz.superConstructor (this, java.util.AbstractList.ListItr, []);
this.cursor = index;
}, "Number");
Clazz.defineMethod (java.util.AbstractList.ListItr, "hasPrevious", 
function () {
return this.cursor != 0;
});
Clazz.defineMethod (java.util.AbstractList.ListItr, "previous", 
function () {
this.checkForComodification ();
try {
var i = this.cursor - 1;
var previous = this.callbacks["java.util.AbstractList"].get (i);
this.lastRet = this.cursor = i;
return previous;
} catch (e) {
if (Clazz.instanceOf (e, IndexOutOfBoundsException)) {
this.checkForComodification ();
throw  new java.util.NoSuchElementException ();
}
}

});
Clazz.defineMethod (java.util.AbstractList.ListItr, "nextIndex", 
function () {
return this.cursor;
});
Clazz.defineMethod (java.util.AbstractList.ListItr, "previousIndex", 
function () {
return this.cursor - 1;
});
Clazz.defineMethod (java.util.AbstractList.ListItr, "set", 
function (o) {
if (this.lastRet == -1) throw  new IllegalStateException ();
this.checkForComodification ();
try {
this.callbacks["java.util.AbstractList"].set (this.lastRet, o);
this.expectedModCount = this.callbacks["java.util.AbstractList"].modCount;
} catch (e) {
if (Clazz.instanceOf (e, IndexOutOfBoundsException)) {
throw  new java.util.ConcurrentModificationException ();
}
}

}, "Object");
Clazz.defineMethod (java.util.AbstractList.ListItr, "add", 
function (o) {
this.checkForComodification ();
try {
this.callbacks["java.util.AbstractList"].add (this.cursor++, o);
this.lastRet = -1;
this.expectedModCount = this.callbacks["java.util.AbstractList"].modCount;
} catch (e) {
if (Clazz.instanceOf (e, IndexOutOfBoundsException)) {
throw  new java.util.ConcurrentModificationException ();
}
}

}, "Object");
Clazz.instantialize (this, arguments);
};
java.util.AbstractList.ListItr.__CLASS_NAME__ = java.util.AbstractList.ListItr.prototype.__CLASS_NAME__ = "java.util.AbstractList.ListItr";
Clazz.inheritClass (java.util.AbstractList.ListItr, java.util.AbstractList.Itr, Clazz.innerTypeInstance (java.util.AbstractList.Itr, this, Clazz.inheritArgs));
Clazz.implementOf (java.util.AbstractList.ListItr, java.util.ListIterator);
}
Clazz.defineMethod (java.util.AbstractList, "subList", 
function (fromIndex, toIndex) {
return (Clazz.instanceOf (this, java.util.RandomAccess) ?  new java.util.RandomAccessSubList (this, fromIndex, toIndex) :  new java.util.SubList (this, fromIndex, toIndex));
}, "Number, Number");
Clazz.defineMethod (java.util.AbstractList, "equals", 
function (o) {
if (o == this) return true;
if (!(Clazz.instanceOf (o, java.util.List))) return false;
var e1 = this.listIterator ();
var e2 = (o).listIterator ();
while (e1.hasNext () && e2.hasNext ()) {
var o1 = e1.next ();
var o2 = e2.next ();
if (!(o1 == null ? o2 == null : o1.equals (o2))) return false;
}
return !(e1.hasNext () || e2.hasNext ());
}, "Object");
Clazz.defineMethod (java.util.AbstractList, "hashCode", 
function () {
var hashCode = 1;
var i = this.iterator ();
while (i.hasNext ()) {
var obj = i.next ();
hashCode = 31 * hashCode + (obj == null ? 0 : obj.hashCode ());
}
return hashCode;
});
Clazz.defineMethod (java.util.AbstractList, "removeRange", 
function (fromIndex, toIndex) {
var it = this.listIterator (fromIndex);
for (var i = 0, n = toIndex - fromIndex; i < n; i++) {
it.next ();
it.remove ();
}
}, "Number, Number");
this.modCount = 0;
Clazz.instantialize (this, arguments);
};
java.util.AbstractList.__CLASS_NAME__ = java.util.AbstractList.prototype.__CLASS_NAME__ = "java.util.AbstractList";
Clazz.inheritClass (java.util.AbstractList, java.util.AbstractCollection);
Clazz.implementOf (java.util.AbstractList, java.util.List);
java.util.SubList = function () {
this.l = null;
this.offset = 0;
this.$size = 0;
this.expectedModCount = 0;
Clazz.makeConstructor (java.util.SubList, 
function (list, fromIndex, toIndex) {
Clazz.superConstructor (this, java.util.SubList, []);
if (fromIndex < 0) throw  new IndexOutOfBoundsException ("fromIndex = " + fromIndex);
if (toIndex > list.size ()) throw  new IndexOutOfBoundsException ("toIndex = " + toIndex);
if (fromIndex > toIndex) throw  new IllegalArgumentException ("fromIndex(" + fromIndex + ") > toIndex(" + toIndex + ")");
this.l = list;
this.offset = fromIndex;
this.$size = toIndex - fromIndex;
this.expectedModCount = this.l.modCount;
}, "java.util.AbstractList, Number, Number");
Clazz.defineMethod (java.util.SubList, "set", 
function (index, element) {
this.rangeCheck (index);
this.checkForComodification ();
return this.l.set (index + this.offset, element);
}, "Number, Object");
Clazz.defineMethod (java.util.SubList, "get", 
function (index) {
this.rangeCheck (index);
this.checkForComodification ();
return this.l.get (index + this.offset);
}, "Number");
Clazz.defineMethod (java.util.SubList, "size", 
function () {
this.checkForComodification ();
return this.$size;
});
Clazz.defineMethod (java.util.SubList, "add", 
function (index, element) {
if (index < 0 || index > this.$size) throw  new IndexOutOfBoundsException ();
this.checkForComodification ();
this.l.add (index + this.offset, element);
this.expectedModCount = this.l.modCount;
this.$size++;
this.modCount++;
}, "Number, Object");
Clazz.defineMethod (java.util.SubList, "remove", 
function (index) {
this.rangeCheck (index);
this.checkForComodification ();
var result = this.l.remove (index + this.offset);
this.expectedModCount = this.l.modCount;
this.$size--;
this.modCount++;
return result;
}, "Number");
Clazz.defineMethod (java.util.SubList, "removeRange", 
function (fromIndex, toIndex) {
this.checkForComodification ();
this.l.removeRange (fromIndex + this.offset, toIndex + this.offset);
this.expectedModCount = this.l.modCount;
this.$size -= (toIndex - fromIndex);
this.modCount++;
}, "Number, Number");
Clazz.defineMethod (java.util.SubList, "addAll", 
function (c) {
return this.addAll (this.$size, c);
}, "java.util.Collection");
Clazz.defineMethod (java.util.SubList, "addAll", 
function (index, c) {
if (index < 0 || index > this.$size) throw  new IndexOutOfBoundsException ("Index: " + index + ", Size: " + this.$size);
var cSize = c.size ();
if (cSize == 0) return false;
this.checkForComodification ();
this.l.addAll (this.offset + index, c);
this.expectedModCount = this.l.modCount;
this.$size += cSize;
this.modCount++;
return true;
}, "Number, java.util.Collection");
Clazz.defineMethod (java.util.SubList, "iterator", 
function () {
return this.listIterator ();
});
Clazz.defineMethod (java.util.SubList, "listIterator", 
function (index) {
this.checkForComodification ();
if (index < 0 || index > this.$size) throw  new IndexOutOfBoundsException ("Index: " + index + ", Size: " + this.$size);
return (function (innerThis) {
if (!Clazz.isClassDefined ("java.util.SubList$1")) {
java.util.SubList$1 = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.i = this.callbacks["java.util.SubList"].l.listIterator (index + this.callbacks["java.util.SubList"].offset);
Clazz.defineMethod (java.util.SubList$1, "hasNext", 
function () {
return this.nextIndex () < this.callbacks["java.util.SubList"].$size;
});
Clazz.defineMethod (java.util.SubList$1, "next", 
function () {
if (this.hasNext ()) return this.i.next ();
 else throw  new java.util.NoSuchElementException ();
});
Clazz.defineMethod (java.util.SubList$1, "hasPrevious", 
function () {
return this.previousIndex () >= 0;
});
Clazz.defineMethod (java.util.SubList$1, "previous", 
function () {
if (this.hasPrevious ()) return this.i.previous ();
 else throw  new java.util.NoSuchElementException ();
});
Clazz.defineMethod (java.util.SubList$1, "nextIndex", 
function () {
return this.i.nextIndex () - this.callbacks["java.util.SubList"].offset;
});
Clazz.defineMethod (java.util.SubList$1, "previousIndex", 
function () {
return this.i.previousIndex () - this.callbacks["java.util.SubList"].offset;
});
Clazz.defineMethod (java.util.SubList$1, "remove", 
function () {
this.i.remove ();
this.callbacks["java.util.SubList"].expectedModCount = this.callbacks["java.util.SubList"].l.modCount;
this.callbacks["java.util.SubList"].$size--;
this.callbacks["java.util.SubList"].modCount++;
});
Clazz.defineMethod (java.util.SubList$1, "set", 
function (o) {
this.i.set (o);
}, "Object");
Clazz.defineMethod (java.util.SubList$1, "add", 
function (o) {
this.i.add (o);
this.callbacks["java.util.SubList"].expectedModCount = this.callbacks["java.util.SubList"].l.modCount;
this.callbacks["java.util.SubList"].$size++;
this.callbacks["java.util.SubList"].modCount++;
}, "Object");
Clazz.instantialize (this, arguments);
};
Clazz.implementOf (java.util.SubList$1, java.util.ListIterator);
java.util.SubList$1.__CLASS_NAME__ = java.util.SubList$1.prototype.__CLASS_NAME__ = "java.util.SubList$1";
}
return Clazz.innerTypeInstance (java.util.SubList$1, innerThis);
}) (this);
}, "Number");
Clazz.defineMethod (java.util.SubList, "subList", 
function (fromIndex, toIndex) {
return  new java.util.SubList (this, fromIndex, toIndex);
}, "Number, Number");
Clazz.defineMethod (java.util.SubList, "rangeCheck", 
function (index) {
if (index < 0 || index >= this.$size) throw  new IndexOutOfBoundsException ("Index: " + index + ",Size: " + this.$size);
}, "Number");
Clazz.defineMethod (java.util.SubList, "checkForComodification", 
function () {
if (this.l.modCount != this.expectedModCount) throw  new java.util.ConcurrentModificationException ();
});
Clazz.instantialize (this, arguments);
};
java.util.SubList.__CLASS_NAME__ = java.util.SubList.prototype.__CLASS_NAME__ = "java.util.SubList";
Clazz.inheritClass (java.util.SubList, java.util.AbstractList);
java.util.RandomAccessSubList = function () {
Clazz.makeConstructor (java.util.RandomAccessSubList, 
function (list, fromIndex, toIndex) {
Clazz.superConstructor (this, java.util.RandomAccessSubList, [list, fromIndex, toIndex]);
}, "java.util.AbstractList, Number, Number");
Clazz.defineMethod (java.util.RandomAccessSubList, "subList", 
function (fromIndex, toIndex) {
return  new java.util.RandomAccessSubList (this, fromIndex, toIndex);
}, "Number, Number");
Clazz.instantialize (this, arguments);
};
java.util.RandomAccessSubList.__CLASS_NAME__ = java.util.RandomAccessSubList.prototype.__CLASS_NAME__ = "java.util.RandomAccessSubList";
Clazz.inheritClass (java.util.RandomAccessSubList, java.util.SubList);
Clazz.implementOf (java.util.RandomAccessSubList, java.util.RandomAccess);
Clazz.declarePackage ("java.util");
java.util.ArrayList = function () {
this.serialVersionUID = java.util.ArrayList.serialVersionUID;
this.elementData = null;
this.$size = 0;
Clazz.makeConstructor (java.util.ArrayList, 
function (initialCapacity) {
Clazz.superConstructor (this, java.util.ArrayList);
if (initialCapacity < 0) throw  new IllegalArgumentException ("Illegal Capacity: " + initialCapacity);
this.elementData =  new Array (initialCapacity);
}, "Number");
Clazz.makeConstructor (java.util.ArrayList, 
function () {
Clazz.superConstructor (this, java.util.ArrayList, []);
this.construct (10);
});
Clazz.makeConstructor (java.util.ArrayList, 
function (c) {
Clazz.superConstructor (this, java.util.ArrayList, []);
this.$size = c.size ();
this.elementData =  new Array (parseInt (Math.min (Math.floor ((this.$size * 110) / 100), Integer.MAX_VALUE)));
c.toArray (this.elementData);
}, "java.util.Collection");
Clazz.defineMethod (java.util.ArrayList, "trimToSize", 
function () {
this.modCount++;
var oldCapacity = this.elementData.length;
if (this.$size < oldCapacity) {
var oldData = this.elementData;
this.elementData =  new Array (this.$size);
System.arraycopy (oldData, 0, this.elementData, 0, this.$size);
}});
Clazz.defineMethod (java.util.ArrayList, "ensureCapacity", 
function (minCapacity) {
this.modCount++;
var oldCapacity = this.elementData.length;
if (minCapacity > oldCapacity) {
var oldData = this.elementData;
var newCapacity = Math.floor ((oldCapacity * 3) / 2) + 1;
if (newCapacity < minCapacity) newCapacity = minCapacity;
this.elementData =  new Array (newCapacity);
System.arraycopy (oldData, 0, this.elementData, 0, this.$size);
}}, "Number");
Clazz.defineMethod (java.util.ArrayList, "size", 
function () {
return this.$size;
});
Clazz.defineMethod (java.util.ArrayList, "isEmpty", 
function () {
return this.$size == 0;
});
Clazz.defineMethod (java.util.ArrayList, "contains", 
function (elem) {
return this.indexOf (elem) >= 0;
}, "Object");
Clazz.defineMethod (java.util.ArrayList, "indexOf", 
function (elem) {
if (elem == null) {
for (var i = 0; i < this.$size; i++) if (this.elementData[i] == null) return i;

} else {
for (var i = 0; i < this.$size; i++) if (elem.equals (this.elementData[i])) return i;

}return -1;
}, "Object");
Clazz.defineMethod (java.util.ArrayList, "lastIndexOf", 
function (elem) {
if (elem == null) {
for (var i = this.$size - 1; i >= 0; i--) if (this.elementData[i] == null) return i;

} else {
for (var i = this.$size - 1; i >= 0; i--) if (elem.equals (this.elementData[i])) return i;

}return -1;
}, "Object");
Clazz.defineMethod (java.util.ArrayList, "clone", 
function () {
try {
var v = Clazz.superCall (this, java.util.ArrayList, "clone", []);
v.elementData =  new Array (this.$size);
System.arraycopy (this.elementData, 0, v.elementData, 0, this.$size);
v.modCount = 0;
return v;
} catch (e) {
if (Clazz.instanceOf (e, CloneNotSupportedException)) {
throw  new InternalError ();
}
}

});
Clazz.defineMethod (java.util.ArrayList, "toArray", 
function () {
var result =  new Array (this.$size);
System.arraycopy (this.elementData, 0, result, 0, this.$size);
return result;
});
Clazz.defineMethod (java.util.ArrayList, "toArray", 
function (a) {
if (a.length < this.$size) a = java.lang.reflect.Array.newInstance (a.getClass ().getComponentType (), this.$size);
System.arraycopy (this.elementData, 0, a, 0, this.$size);
if (a.length > this.$size) a[this.$size] = null;
return a;
}, "Array");
Clazz.defineMethod (java.util.ArrayList, "get", 
function (index) {
this.RangeCheck (index);
return this.elementData[index];
}, "Number");
Clazz.defineMethod (java.util.ArrayList, "set", 
function (index, element) {
this.RangeCheck (index);
var oldValue = this.elementData[index];
this.elementData[index] = element;
return oldValue;
}, "Number, Object");
Clazz.defineMethod (java.util.ArrayList, "add", 
function (o) {
this.ensureCapacity (this.$size + 1);
this.elementData[this.$size++] = o;
return true;
}, "Object");
Clazz.defineMethod (java.util.ArrayList, "add", 
function (index, element) {
if (index > this.$size || index < 0) throw  new IndexOutOfBoundsException ("Index: " + index + ", Size: " + this.$size);
this.ensureCapacity (this.$size + 1);
System.arraycopy (this.elementData, index, this.elementData, index + 1, this.$size - index);
this.elementData[index] = element;
this.$size++;
}, "Number, Object");
Clazz.defineMethod (java.util.ArrayList, "remove", 
function (index) {
this.RangeCheck (index);
this.modCount++;
var oldValue = this.elementData[index];
var numMoved = this.$size - index - 1;
if (numMoved > 0) System.arraycopy (this.elementData, index + 1, this.elementData, index, numMoved);
this.elementData[--this.$size] = null;
return oldValue;
}, "Number");
Clazz.defineMethod (java.util.ArrayList, "clear", 
function () {
this.modCount++;
for (var i = 0; i < this.$size; i++) this.elementData[i] = null;

this.$size = 0;
});
Clazz.defineMethod (java.util.ArrayList, "addAll", 
function (c) {
var a = c.toArray ();
var numNew = a.length;
this.ensureCapacity (this.$size + numNew);
System.arraycopy (a, 0, this.elementData, this.$size, numNew);
this.$size += numNew;
return numNew != 0;
}, "java.util.Collection");
Clazz.defineMethod (java.util.ArrayList, "addAll", 
function (index, c) {
if (index > this.$size || index < 0) throw  new IndexOutOfBoundsException ("Index: " + index + ", Size: " + this.$size);
var a = c.toArray ();
var numNew = a.length;
this.ensureCapacity (this.$size + numNew);
var numMoved = this.$size - index;
if (numMoved > 0) System.arraycopy (this.elementData, index, this.elementData, index + numNew, numMoved);
System.arraycopy (a, 0, this.elementData, index, numNew);
this.$size += numNew;
return numNew != 0;
}, "Number, java.util.Collection");
Clazz.defineMethod (java.util.ArrayList, "removeRange", 
function (fromIndex, toIndex) {
this.modCount++;
var numMoved = this.$size - toIndex;
System.arraycopy (this.elementData, toIndex, this.elementData, fromIndex, numMoved);
var newSize = this.$size - (toIndex - fromIndex);
while (this.$size != newSize) this.elementData[--this.$size] = null;

}, "Number, Number");
Clazz.defineMethod (java.util.ArrayList, "RangeCheck", 
function (index) {
if (index >= this.$size) throw  new IndexOutOfBoundsException ("Index: " + index + ", Size: " + this.$size);
}, "Number");
Clazz.defineMethod (java.util.ArrayList, "writeObject", 
function (s) {
s.defaultWriteObject ();
s.writeInt (this.elementData.length);
for (var i = 0; i < this.$size; i++) s.writeObject (this.elementData[i]);

}, "java.io.ObjectOutputStream");
Clazz.defineMethod (java.util.ArrayList, "readObject", 
function (s) {
s.defaultReadObject ();
var arrayLength = s.readInt ();
this.elementData =  new Array (arrayLength);
for (var i = 0; i < this.$size; i++) this.elementData[i] = s.readObject ();

}, "java.io.ObjectInputStream");
Clazz.instantialize (this, arguments);
};
java.util.ArrayList.__CLASS_NAME__ = java.util.ArrayList.prototype.__CLASS_NAME__ = "java.util.ArrayList";
Clazz.inheritClass (java.util.ArrayList, java.util.AbstractList);
Clazz.implementOf (java.util.ArrayList, java.util.List);
Clazz.implementOf (java.util.ArrayList, java.util.RandomAccess);
Clazz.implementOf (java.util.ArrayList, Cloneable);
Clazz.implementOf (java.util.ArrayList, java.io.Serializable);
java.util.ArrayList.serialVersionUID = 8683452581122892189;
Clazz.declarePackage ("java.util");
java.util.HashMap = function () {
this.DEFAULT_INITIAL_CAPACITY = java.util.HashMap.DEFAULT_INITIAL_CAPACITY;
this.MAXIMUM_CAPACITY = java.util.HashMap.MAXIMUM_CAPACITY;
this.DEFAULT_LOAD_FACTOR = java.util.HashMap.DEFAULT_LOAD_FACTOR;
this.table = null;
this.$size = 0;
this.threshold = 0;
this.$loadFactor = 0;
this.modCount = 0;
Clazz.makeConstructor (java.util.HashMap, 
function (initialCapacity, loadFactor) {
Clazz.superConstructor (this, java.util.HashMap, []);
if (initialCapacity < 0) throw  new IllegalArgumentException ("Illegal initial capacity: " + initialCapacity);
if (initialCapacity > java.util.HashMap.MAXIMUM_CAPACITY) initialCapacity = java.util.HashMap.MAXIMUM_CAPACITY;
if (loadFactor <= 0 || Float.isNaN (loadFactor)) throw  new IllegalArgumentException ("Illegal load factor: " + loadFactor);
var capacity = 1;
while (capacity < initialCapacity) capacity <<= 1;

this.$loadFactor = loadFactor;
this.threshold = parseInt ((capacity * loadFactor));
this.table =  new Array (capacity);
this.init ();
}, "Number, Number");
Clazz.makeConstructor (java.util.HashMap, 
function (initialCapacity) {
Clazz.superConstructor (this, java.util.HashMap, []);
this.construct (initialCapacity, java.util.HashMap.DEFAULT_LOAD_FACTOR);
}, "Number");
Clazz.makeConstructor (java.util.HashMap, 
function () {
Clazz.superConstructor (this, java.util.HashMap, []);
this.$loadFactor = java.util.HashMap.DEFAULT_LOAD_FACTOR;
this.threshold = parseInt ((java.util.HashMap.DEFAULT_INITIAL_CAPACITY * java.util.HashMap.DEFAULT_LOAD_FACTOR));
this.table =  new Array (java.util.HashMap.DEFAULT_INITIAL_CAPACITY);
this.init ();
});
Clazz.makeConstructor (java.util.HashMap, 
function (m) {
Clazz.superConstructor (this, java.util.HashMap, []);
this.construct (Math.max (parseInt ((m.size () / java.util.HashMap.DEFAULT_LOAD_FACTOR)) + 1, java.util.HashMap.DEFAULT_INITIAL_CAPACITY), java.util.HashMap.DEFAULT_LOAD_FACTOR);
this.putAllForCreate (m);
}, "java.util.Map");
Clazz.defineMethod (java.util.HashMap, "init", 
function () {
});
this.NULL_KEY = java.util.HashMap.NULL_KEY;
Clazz.defineMethod (java.util.HashMap, "size", 
function () {
return this.$size;
});
Clazz.defineMethod (java.util.HashMap, "isEmpty", 
function () {
return this.$size == 0;
});
Clazz.defineMethod (java.util.HashMap, "get", 
function (key) {
var k = java.util.HashMap.maskNull (key);
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
var e = this.table[i];
while (true) {
if (e == null) return e;
if (e.hash == hash && java.util.HashMap.eq (k, e.key)) return e.value;
e = e.next;
}
}, "Object");
Clazz.defineMethod (java.util.HashMap, "containsKey", 
function (key) {
var k = java.util.HashMap.maskNull (key);
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
var e = this.table[i];
while (e != null) {
if (e.hash == hash && java.util.HashMap.eq (k, e.key)) return true;
e = e.next;
}
return false;
}, "Object");
Clazz.defineMethod (java.util.HashMap, "getEntry", 
function (key) {
var k = java.util.HashMap.maskNull (key);
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
var e = this.table[i];
while (e != null && !(e.hash == hash && java.util.HashMap.eq (k, e.key))) e = e.next;

return e;
}, "Object");
Clazz.defineMethod (java.util.HashMap, "put", 
function (key, value) {
var k = java.util.HashMap.maskNull (key);
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
for (var e = this.table[i]; e != null; e = e.next) {
if (e.hash == hash && java.util.HashMap.eq (k, e.key)) {
var oldValue = e.value;
e.value = value;
e.recordAccess (this);
return oldValue;
}}
this.modCount++;
this.addEntry (hash, k, value, i);
return null;
}, "Object, Object");
Clazz.defineMethod (java.util.HashMap, "putForCreate", 
function (key, value) {
var k = java.util.HashMap.maskNull (key);
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
for (var e = this.table[i]; e != null; e = e.next) {
if (e.hash == hash && java.util.HashMap.eq (k, e.key)) {
e.value = value;
return ;
}}
this.createEntry (hash, k, value, i);
}, "Object, Object");
Clazz.defineMethod (java.util.HashMap, "putAllForCreate", 
function (m) {
for (var i = m.entrySet ().iterator (); i.hasNext (); ) {
var e = i.next ();
this.putForCreate (e.getKey (), e.getValue ());
}
}, "java.util.Map");
Clazz.defineMethod (java.util.HashMap, "resize", 
function (newCapacity) {
var oldTable = this.table;
var oldCapacity = oldTable.length;
if (oldCapacity == java.util.HashMap.MAXIMUM_CAPACITY) {
this.threshold = Integer.MAX_VALUE;
return ;
}var newTable =  new Array (newCapacity);
this.transfer (newTable);
this.table = newTable;
this.threshold = parseInt ((newCapacity * this.$loadFactor));
}, "Number");
Clazz.defineMethod (java.util.HashMap, "transfer", 
function (newTable) {
var src = this.table;
var newCapacity = newTable.length;
for (var j = 0; j < src.length; j++) {
var e = src[j];
if (e != null) {
src[j] = null;
do {
var next = e.next;
var i = java.util.HashMap.indexFor (e.hash, newCapacity);
e.next = newTable[i];
newTable[i] = e;
e = next;
} while (e != null);
}}
}, "Array");
Clazz.defineMethod (java.util.HashMap, "putAll", 
function (m) {
var numKeysToBeAdded = m.size ();
if (numKeysToBeAdded == 0) return ;
if (numKeysToBeAdded > this.threshold) {
var targetCapacity = parseInt ((numKeysToBeAdded / this.$loadFactor + 1));
if (targetCapacity > java.util.HashMap.MAXIMUM_CAPACITY) targetCapacity = java.util.HashMap.MAXIMUM_CAPACITY;
var newCapacity = this.table.length;
while (newCapacity < targetCapacity) newCapacity <<= 1;

if (newCapacity > this.table.length) this.resize (newCapacity);
}for (var i = m.entrySet ().iterator (); i.hasNext (); ) {
var e = i.next ();
this.put (e.getKey (), e.getValue ());
}
}, "java.util.Map");
Clazz.defineMethod (java.util.HashMap, "remove", 
function (key) {
var e = this.removeEntryForKey (key);
return (e == null ? e : e.value);
}, "Object");
Clazz.defineMethod (java.util.HashMap, "removeEntryForKey", 
function (key) {
var k = java.util.HashMap.maskNull (key);
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
var prev = this.table[i];
var e = prev;
while (e != null) {
var next = e.next;
if (e.hash == hash && java.util.HashMap.eq (k, e.key)) {
this.modCount++;
this.$size--;
if (prev == e) this.table[i] = next;
 else prev.next = next;
e.recordRemoval (this);
return e;
}prev = e;
e = next;
}
return e;
}, "Object");
Clazz.defineMethod (java.util.HashMap, "removeMapping", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return null;
var entry = o;
var k = java.util.HashMap.maskNull (entry.getKey ());
var hash = java.util.HashMap.hash (k);
var i = java.util.HashMap.indexFor (hash, this.table.length);
var prev = this.table[i];
var e = prev;
while (e != null) {
var next = e.next;
if (e.hash == hash && e.equals (entry)) {
this.modCount++;
this.$size--;
if (prev == e) this.table[i] = next;
 else prev.next = next;
e.recordRemoval (this);
return e;
}prev = e;
e = next;
}
return e;
}, "Object");
Clazz.defineMethod (java.util.HashMap, "clear", 
function () {
this.modCount++;
var tab = this.table;
for (var i = 0; i < tab.length; i++) tab[i] = null;

this.$size = 0;
});
Clazz.defineMethod (java.util.HashMap, "containsValue", 
function (value) {
if (value == null) return this.containsNullValue ();
var tab = this.table;
for (var i = 0; i < tab.length; i++) for (var e = tab[i]; e != null; e = e.next) if (value.equals (e.value)) return true;


return false;
}, "Object");
Clazz.defineMethod (java.util.HashMap, "containsNullValue", 
function () {
var tab = this.table;
for (var i = 0; i < tab.length; i++) for (var e = tab[i]; e != null; e = e.next) if (e.value == null) return true;


return false;
});
Clazz.defineMethod (java.util.HashMap, "clone", 
function () {
var result = null;
try {
result = Clazz.superCall (this, java.util.HashMap, "clone", []);
} catch (e) {
if (Clazz.instanceOf (e, CloneNotSupportedException)) {
}
}

result.table =  new Array (this.table.length);
result.entrySet = null;
result.modCount = 0;
result.size = 0;
result.init ();
result.putAllForCreate (this);
return result;
});
Clazz.defineMethod (java.util.HashMap, "addEntry", 
function (hash, key, value, bucketIndex) {
this.table[bucketIndex] =  new java.util.HashMap.Entry (hash, key, value, this.table[bucketIndex]);
if (this.$size++ >= this.threshold) this.resize (2 * this.table.length);
}, "Number, Object, Object, Number");
Clazz.defineMethod (java.util.HashMap, "createEntry", 
function (hash, key, value, bucketIndex) {
this.table[bucketIndex] =  new java.util.HashMap.Entry (hash, key, value, this.table[bucketIndex]);
this.$size++;
}, "Number, Object, Object, Number");
if (!Clazz.isClassDefined ("java.util.HashMap.HashIterator")) {
java.util.HashMap.HashIterator = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.$next = null;
this.expectedModCount = 0;
this.index = 0;
this.current = null;
Clazz.makeConstructor (java.util.HashMap.HashIterator, 
function () {
this.expectedModCount = this.callbacks["java.util.HashMap"].modCount;
var t = this.callbacks["java.util.HashMap"].table;
var i = t.length;
var n = null;
if (this.callbacks["java.util.HashMap"].$size != 0) {
while (i > 0 && (n = t[--i]) == null) ;
}this.$next = n;
this.index = i;
});
Clazz.defineMethod (java.util.HashMap.HashIterator, "hasNext", 
function () {
return this.$next != null;
});
Clazz.defineMethod (java.util.HashMap.HashIterator, "nextEntry", 
function () {
if (this.callbacks["java.util.HashMap"].modCount != this.expectedModCount) throw  new java.util.ConcurrentModificationException ();
var e = this.$next;
if (e == null) throw  new java.util.NoSuchElementException ();
var n = e.next;
var t = this.callbacks["java.util.HashMap"].table;
var i = this.index;
while (n == null && i > 0) n = t[--i];

this.index = i;
this.$next = n;
return this.current = e;
});
Clazz.defineMethod (java.util.HashMap.HashIterator, "remove", 
function () {
if (this.current == null) throw  new IllegalStateException ();
if (this.callbacks["java.util.HashMap"].modCount != this.expectedModCount) throw  new java.util.ConcurrentModificationException ();
var k = this.current.key;
this.current = null;
this.callbacks["java.util.HashMap"].removeEntryForKey (k);
this.expectedModCount = this.callbacks["java.util.HashMap"].modCount;
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.HashIterator.__CLASS_NAME__ = java.util.HashMap.HashIterator.prototype.__CLASS_NAME__ = "java.util.HashMap.HashIterator";
Clazz.implementOf (java.util.HashMap.HashIterator, java.util.Iterator);
}
if (!Clazz.isClassDefined ("java.util.HashMap.ValueIterator")) {
java.util.HashMap.ValueIterator = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.HashMap.ValueIterator, "next", 
function () {
return this.nextEntry ().value;
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.ValueIterator.__CLASS_NAME__ = java.util.HashMap.ValueIterator.prototype.__CLASS_NAME__ = "java.util.HashMap.ValueIterator";
Clazz.inheritClass (java.util.HashMap.ValueIterator, java.util.HashMap.HashIterator, Clazz.innerTypeInstance (java.util.HashMap.HashIterator, this, Clazz.inheritArgs));
}
if (!Clazz.isClassDefined ("java.util.HashMap.KeyIterator")) {
java.util.HashMap.KeyIterator = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.HashMap.KeyIterator, "next", 
function () {
return this.nextEntry ().getKey ();
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.KeyIterator.__CLASS_NAME__ = java.util.HashMap.KeyIterator.prototype.__CLASS_NAME__ = "java.util.HashMap.KeyIterator";
Clazz.inheritClass (java.util.HashMap.KeyIterator, java.util.HashMap.HashIterator, Clazz.innerTypeInstance (java.util.HashMap.HashIterator, this, Clazz.inheritArgs));
}
if (!Clazz.isClassDefined ("java.util.HashMap.EntryIterator")) {
java.util.HashMap.EntryIterator = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.HashMap.EntryIterator, "next", 
function () {
return this.nextEntry ();
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.EntryIterator.__CLASS_NAME__ = java.util.HashMap.EntryIterator.prototype.__CLASS_NAME__ = "java.util.HashMap.EntryIterator";
Clazz.inheritClass (java.util.HashMap.EntryIterator, java.util.HashMap.HashIterator, Clazz.innerTypeInstance (java.util.HashMap.HashIterator, this, Clazz.inheritArgs));
}
Clazz.defineMethod (java.util.HashMap, "newKeyIterator", 
function () {
return Clazz.innerTypeInstance (java.util.HashMap.KeyIterator, this);
});
Clazz.defineMethod (java.util.HashMap, "newValueIterator", 
function () {
return Clazz.innerTypeInstance (java.util.HashMap.ValueIterator, this);
});
Clazz.defineMethod (java.util.HashMap, "newEntryIterator", 
function () {
return Clazz.innerTypeInstance (java.util.HashMap.EntryIterator, this);
});
this.$entrySet = null;
Clazz.defineMethod (java.util.HashMap, "keySet", 
function () {
var ks = this.$keySet;
return (ks != null ? ks : (this.$keySet = Clazz.innerTypeInstance (java.util.HashMap.KeySet, this)));
});
if (!Clazz.isClassDefined ("java.util.HashMap.KeySet")) {
java.util.HashMap.KeySet = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.HashMap.KeySet, "iterator", 
function () {
return this.callbacks["java.util.HashMap"].newKeyIterator ();
});
Clazz.defineMethod (java.util.HashMap.KeySet, "size", 
function () {
return this.callbacks["java.util.HashMap"].$size;
});
Clazz.defineMethod (java.util.HashMap.KeySet, "contains", 
function (o) {
return this.callbacks["java.util.HashMap"].containsKey (o);
}, "Object");
Clazz.defineMethod (java.util.HashMap.KeySet, "remove", 
function (o) {
return this.callbacks["java.util.HashMap"].removeEntryForKey (o) != null;
}, "Object");
Clazz.defineMethod (java.util.HashMap.KeySet, "clear", 
function () {
this.callbacks["java.util.HashMap"].clear ();
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.KeySet.__CLASS_NAME__ = java.util.HashMap.KeySet.prototype.__CLASS_NAME__ = "java.util.HashMap.KeySet";
Clazz.inheritClass (java.util.HashMap.KeySet, java.util.AbstractSet);
}
Clazz.defineMethod (java.util.HashMap, "values", 
function () {
var vs = this.$values;
return (vs != null ? vs : (this.$values = Clazz.innerTypeInstance (java.util.HashMap.Values, this)));
});
if (!Clazz.isClassDefined ("java.util.HashMap.Values")) {
java.util.HashMap.Values = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.HashMap.Values, "iterator", 
function () {
return this.callbacks["java.util.HashMap"].newValueIterator ();
});
Clazz.defineMethod (java.util.HashMap.Values, "size", 
function () {
return this.callbacks["java.util.HashMap"].$size;
});
Clazz.defineMethod (java.util.HashMap.Values, "contains", 
function (o) {
return this.callbacks["java.util.HashMap"].containsValue (o);
}, "Object");
Clazz.defineMethod (java.util.HashMap.Values, "clear", 
function () {
this.callbacks["java.util.HashMap"].clear ();
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.Values.__CLASS_NAME__ = java.util.HashMap.Values.prototype.__CLASS_NAME__ = "java.util.HashMap.Values";
Clazz.inheritClass (java.util.HashMap.Values, java.util.AbstractCollection);
}
Clazz.defineMethod (java.util.HashMap, "entrySet", 
function () {
var es = this.$entrySet;
return (es != null ? es : (this.$entrySet = Clazz.innerTypeInstance (java.util.HashMap.EntrySet, this)));
});
if (!Clazz.isClassDefined ("java.util.HashMap.EntrySet")) {
java.util.HashMap.EntrySet = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.HashMap.EntrySet, "iterator", 
function () {
return this.callbacks["java.util.HashMap"].newEntryIterator ();
});
Clazz.defineMethod (java.util.HashMap.EntrySet, "contains", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return false;
var e = o;
var candidate = this.callbacks["java.util.HashMap"].getEntry (e.getKey ());
return candidate != null && candidate.equals (e);
}, "Object");
Clazz.defineMethod (java.util.HashMap.EntrySet, "remove", 
function (o) {
return this.callbacks["java.util.HashMap"].removeMapping (o) != null;
}, "Object");
Clazz.defineMethod (java.util.HashMap.EntrySet, "size", 
function () {
return this.callbacks["java.util.HashMap"].$size;
});
Clazz.defineMethod (java.util.HashMap.EntrySet, "clear", 
function () {
this.callbacks["java.util.HashMap"].clear ();
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.EntrySet.__CLASS_NAME__ = java.util.HashMap.EntrySet.prototype.__CLASS_NAME__ = "java.util.HashMap.EntrySet";
Clazz.inheritClass (java.util.HashMap.EntrySet, java.util.AbstractSet);
}
Clazz.defineMethod (java.util.HashMap, "writeObject", 
function (s) {
s.defaultWriteObject ();
s.writeInt (this.table.length);
s.writeInt (this.$size);
for (var i = this.entrySet ().iterator (); i.hasNext (); ) {
var e = i.next ();
s.writeObject (e.getKey ());
s.writeObject (e.getValue ());
}
}, "java.io.ObjectOutputStream");
this.serialVersionUID = java.util.HashMap.serialVersionUID;
Clazz.defineMethod (java.util.HashMap, "readObject", 
function (s) {
s.defaultReadObject ();
var numBuckets = s.readInt ();
this.table =  new Array (numBuckets);
this.init ();
var size = s.readInt ();
for (var i = 0; i < size; i++) {
var key = s.readObject ();
var value = s.readObject ();
this.putForCreate (key, value);
}
}, "java.io.ObjectInputStream");
Clazz.defineMethod (java.util.HashMap, "capacity", 
function () {
return this.table.length;
});
Clazz.defineMethod (java.util.HashMap, "loadFactor", 
function () {
return this.$loadFactor;
});
Clazz.instantialize (this, arguments);
};
java.util.HashMap.__CLASS_NAME__ = java.util.HashMap.prototype.__CLASS_NAME__ = "java.util.HashMap";
Clazz.inheritClass (java.util.HashMap, java.util.AbstractMap);
Clazz.implementOf (java.util.HashMap, java.util.Map);
Clazz.implementOf (java.util.HashMap, Cloneable);
Clazz.implementOf (java.util.HashMap, java.io.Serializable);
java.util.HashMap.Entry = function () {
this.key = null;
this.value = null;
this.hash = 0;
this.next = null;
Clazz.makeConstructor (java.util.HashMap.Entry, 
function (h, k, v, n) {
this.value = v;
this.next = n;
this.key = k;
this.hash = h;
}, "Number, Object, Object, java.util.HashMap.Entry");
Clazz.defineMethod (java.util.HashMap.Entry, "getKey", 
function () {
return java.util.HashMap.unmaskNull (this.key);
});
Clazz.defineMethod (java.util.HashMap.Entry, "getValue", 
function () {
return this.value;
});
Clazz.defineMethod (java.util.HashMap.Entry, "setValue", 
function (newValue) {
var oldValue = this.value;
this.value = newValue;
return oldValue;
}, "Object");
Clazz.defineMethod (java.util.HashMap.Entry, "equals", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return false;
var e = o;
var k1 = this.getKey ();
var k2 = e.getKey ();
if (k1 == k2 || (k1 != null && k1.equals (k2))) {
var v1 = this.getValue ();
var v2 = e.getValue ();
if (v1 == v2 || (v1 != null && v1.equals (v2))) return true;
}return false;
}, "Object");
Clazz.defineMethod (java.util.HashMap.Entry, "hashCode", 
function () {
return (this.key == java.util.HashMap.NULL_KEY ? 0 : this.key.hashCode ()) ^ (this.value == null ? 0 : this.value.hashCode ());
});
Clazz.defineMethod (java.util.HashMap.Entry, "toString", 
function () {
return this.getKey () + "=" + this.getValue ();
});
Clazz.defineMethod (java.util.HashMap.Entry, "recordAccess", 
function (m) {
}, "java.util.HashMap");
Clazz.defineMethod (java.util.HashMap.Entry, "recordRemoval", 
function (m) {
}, "java.util.HashMap");
Clazz.instantialize (this, arguments);
};
java.util.HashMap.Entry.__CLASS_NAME__ = java.util.HashMap.Entry.prototype.__CLASS_NAME__ = "java.util.HashMap.Entry";
Clazz.implementOf (java.util.HashMap.Entry, java.util.Map.Entry);
java.util.HashMap.DEFAULT_INITIAL_CAPACITY = 16;
java.util.HashMap.MAXIMUM_CAPACITY = 1 << 30;
java.util.HashMap.DEFAULT_LOAD_FACTOR = 0.75;
java.util.HashMap.NULL_KEY =  new Object ();
java.util.HashMap.serialVersionUID = 362498820763181265;
Clazz.defineMethod (java.util.HashMap, "maskNull", 
function (key) {
return (key == null ? java.util.HashMap.NULL_KEY : key);
}, "Object");
java.util.HashMap.maskNull = java.util.HashMap.prototype.maskNull;
Clazz.defineMethod (java.util.HashMap, "unmaskNull", 
function (key) {
return (key == java.util.HashMap.NULL_KEY ? null : key);
}, "Object");
java.util.HashMap.unmaskNull = java.util.HashMap.prototype.unmaskNull;
Clazz.defineMethod (java.util.HashMap, "hash", 
function (x) {
var h = x.hashCode ();
h += ~(h << 9);
h ^= (h >>> 14);
h += (h << 4);
h ^= (h >>> 10);
return h;
}, "Object");
java.util.HashMap.hash = java.util.HashMap.prototype.hash;
Clazz.defineMethod (java.util.HashMap, "eq", 
function (x, y) {
return x == y || x.equals (y);
}, "Object, Object");
java.util.HashMap.eq = java.util.HashMap.prototype.eq;
Clazz.defineMethod (java.util.HashMap, "indexFor", 
function (h, length) {
return h & (length - 1);
}, "Number, Number");
java.util.HashMap.indexFor = java.util.HashMap.prototype.indexFor;
Clazz.declarePackage ("java.util");
java.util.HashSet = function () {
this.serialVersionUID = java.util.HashSet.serialVersionUID;
this.map = null;
this.PRESENT = java.util.HashSet.PRESENT;
Clazz.makeConstructor (java.util.HashSet, 
function () {
Clazz.superConstructor (this, java.util.HashSet, []);
this.map =  new java.util.HashMap ();
});
Clazz.makeConstructor (java.util.HashSet, 
function (c) {
Clazz.superConstructor (this, java.util.HashSet, []);
this.map =  new java.util.HashMap (Math.max (parseInt ((c.size () / .75)) + 1, 16));
this.addAll (c);
}, "java.util.Collection");
Clazz.makeConstructor (java.util.HashSet, 
function (initialCapacity, loadFactor) {
Clazz.superConstructor (this, java.util.HashSet, []);
this.map =  new java.util.HashMap (initialCapacity, loadFactor);
}, "Number, Number");
Clazz.makeConstructor (java.util.HashSet, 
function (initialCapacity) {
Clazz.superConstructor (this, java.util.HashSet, []);
this.map =  new java.util.HashMap (initialCapacity);
}, "Number");
Clazz.makeConstructor (java.util.HashSet, 
function (initialCapacity, loadFactor, dummy) {
Clazz.superConstructor (this, java.util.HashSet, []);
this.map =  new java.util.LinkedHashMap (initialCapacity, loadFactor);
}, "Number, Number, Boolean");
Clazz.defineMethod (java.util.HashSet, "iterator", 
function () {
return this.map.keySet ().iterator ();
});
Clazz.defineMethod (java.util.HashSet, "size", 
function () {
return this.map.size ();
});
Clazz.defineMethod (java.util.HashSet, "isEmpty", 
function () {
return this.map.isEmpty ();
});
Clazz.defineMethod (java.util.HashSet, "contains", 
function (o) {
return this.map.containsKey (o);
}, "Object");
Clazz.defineMethod (java.util.HashSet, "add", 
function (o) {
return this.map.put (o, java.util.HashSet.PRESENT) == null;
}, "Object");
Clazz.defineMethod (java.util.HashSet, "remove", 
function (o) {
return this.map.remove (o) == java.util.HashSet.PRESENT;
}, "Object");
Clazz.defineMethod (java.util.HashSet, "clear", 
function () {
this.map.clear ();
});
Clazz.defineMethod (java.util.HashSet, "clone", 
function () {
try {
var newSet = Clazz.superCall (this, java.util.HashSet, "clone", []);
newSet.map = this.map.clone ();
return newSet;
} catch (e) {
if (Clazz.instanceOf (e, CloneNotSupportedException)) {
throw  new InternalError ();
}
}

});
Clazz.defineMethod (java.util.HashSet, "writeObject", 
function (s) {
s.defaultWriteObject ();
s.writeInt (this.map.capacity ());
s.writeFloat (this.map.loadFactor ());
s.writeInt (this.map.size ());
for (var i = this.map.keySet ().iterator (); i.hasNext (); ) s.writeObject (i.next ());

}, "java.io.ObjectOutputStream");
Clazz.defineMethod (java.util.HashSet, "readObject", 
function (s) {
s.defaultReadObject ();
var capacity = s.readInt ();
var loadFactor = s.readFloat ();
this.map = (Clazz.instanceOf (this, java.util.LinkedHashSet) ?  new java.util.LinkedHashMap (capacity, loadFactor) :  new java.util.HashMap (capacity, loadFactor));
var size = s.readInt ();
for (var i = 0; i < size; i++) {
var e = s.readObject ();
this.map.put (e, java.util.HashSet.PRESENT);
}
}, "java.io.ObjectInputStream");
Clazz.instantialize (this, arguments);
};
java.util.HashSet.__CLASS_NAME__ = java.util.HashSet.prototype.__CLASS_NAME__ = "java.util.HashSet";
Clazz.inheritClass (java.util.HashSet, java.util.AbstractSet);
Clazz.implementOf (java.util.HashSet, java.util.Set);
Clazz.implementOf (java.util.HashSet, Cloneable);
Clazz.implementOf (java.util.HashSet, java.io.Serializable);
java.util.HashSet.serialVersionUID = -5024744406713321676;
java.util.HashSet.PRESENT =  new Object ();
Clazz.declarePackage ("java.util");
java.util.Dictionary = function () {
Clazz.makeConstructor (java.util.Dictionary, 
function () {
});
Clazz.instantialize (this, arguments);
};
java.util.Dictionary.__CLASS_NAME__ = java.util.Dictionary.prototype.__CLASS_NAME__ = "java.util.Dictionary";
Clazz.declarePackage ("java.util");
java.util.Hashtable = function () {
this.table = null;
this.count = 0;
this.threshold = 0;
this.loadFactor = 0;
this.modCount = 0;
this.serialVersionUID = java.util.Hashtable.serialVersionUID;
Clazz.makeConstructor (java.util.Hashtable, 
function (initialCapacity, loadFactor) {
Clazz.superConstructor (this, java.util.Hashtable, []);
if (initialCapacity < 0) throw  new IllegalArgumentException ("Illegal Capacity: " + initialCapacity);
if (loadFactor <= 0 || Float.isNaN (loadFactor)) throw  new IllegalArgumentException ("Illegal Load: " + loadFactor);
if (initialCapacity == 0) initialCapacity = 1;
this.loadFactor = loadFactor;
this.table =  new Array (initialCapacity);
this.threshold = parseInt ((initialCapacity * loadFactor));
}, "Number, Number");
Clazz.makeConstructor (java.util.Hashtable, 
function (initialCapacity) {
Clazz.superConstructor (this, java.util.Hashtable, []);
this.construct (initialCapacity, 0.75);
}, "Number");
Clazz.makeConstructor (java.util.Hashtable, 
function () {
Clazz.superConstructor (this, java.util.Hashtable, []);
this.construct (11, 0.75);
});
Clazz.makeConstructor (java.util.Hashtable, 
function (t) {
Clazz.superConstructor (this, java.util.Hashtable, []);
this.construct (Math.max (2 * t.size (), 11), 0.75);
this.putAll (t);
}, "java.util.Map");
Clazz.defineMethod (java.util.Hashtable, "size", 
function () {
return this.count;
});
Clazz.defineMethod (java.util.Hashtable, "isEmpty", 
function () {
return this.count == 0;
});
Clazz.defineMethod (java.util.Hashtable, "keys", 
function () {
return this.getEnumeration (java.util.Hashtable.KEYS);
});
Clazz.defineMethod (java.util.Hashtable, "elements", 
function () {
return this.getEnumeration (java.util.Hashtable.VALUES);
});
Clazz.defineMethod (java.util.Hashtable, "contains", 
function (value) {
if (value == null) {
throw  new NullPointerException ();
}var tab = this.table;
for (var i = tab.length; i-- > 0; ) {
for (var e = tab[i]; e != null; e = e.next) {
if (e.value.equals (value)) {
return true;
}}
}
return false;
}, "Object");
Clazz.defineMethod (java.util.Hashtable, "containsValue", 
function (value) {
return this.contains (value);
}, "Object");
Clazz.defineMethod (java.util.Hashtable, "containsKey", 
function (key) {
var tab = this.table;
var hash = key.hashCode ();
var index = (hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index]; e != null; e = e.next) {
if ((e.hash == hash) && e.key.equals (key)) {
return true;
}}
return false;
}, "Object");
Clazz.defineMethod (java.util.Hashtable, "get", 
function (key) {
var tab = this.table;
var hash = key.hashCode ();
var index = (hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index]; e != null; e = e.next) {
if ((e.hash == hash) && e.key.equals (key)) {
return e.value;
}}
return null;
}, "Object");
Clazz.defineMethod (java.util.Hashtable, "rehash", 
function () {
var oldCapacity = this.table.length;
var oldMap = this.table;
var newCapacity = oldCapacity * 2 + 1;
var newMap =  new Array (newCapacity);
this.modCount++;
this.threshold = parseInt ((newCapacity * this.loadFactor));
this.table = newMap;
for (var i = oldCapacity; i-- > 0; ) {
for (var old = oldMap[i]; old != null; ) {
var e = old;
old = old.next;
var index = (e.hash & 0x7FFFFFFF) % newCapacity;
e.next = newMap[index];
newMap[index] = e;
}
}
});
Clazz.defineMethod (java.util.Hashtable, "put", 
function (key, value) {
if (value == null) {
throw  new NullPointerException ();
}var tab = this.table;
var hash = key.hashCode ();
var index = (hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index]; e != null; e = e.next) {
if ((e.hash == hash) && e.key.equals (key)) {
var old = e.value;
e.value = value;
return old;
}}
this.modCount++;
if (this.count >= this.threshold) {
this.rehash ();
tab = this.table;
index = (hash & 0x7FFFFFFF) % tab.length;
}var e =  new java.util.Hashtable.Entry (hash, key, value, tab[index]);
tab[index] = e;
this.count++;
return null;
}, "Object, Object");
Clazz.defineMethod (java.util.Hashtable, "remove", 
function (key) {
var tab = this.table;
var hash = key.hashCode ();
var index = (hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index], prev = null; e != null; prev = e, e = e.next) {
if ((e.hash == hash) && e.key.equals (key)) {
this.modCount++;
if (prev != null) {
prev.next = e.next;
} else {
tab[index] = e.next;
}this.count--;
var oldValue = e.value;
e.value = null;
return oldValue;
}}
return null;
}, "Object");
Clazz.defineMethod (java.util.Hashtable, "putAll", 
function (t) {
var i = t.entrySet ().iterator ();
while (i.hasNext ()) {
var e = i.next ();
this.put (e.getKey (), e.getValue ());
}
}, "java.util.Map");
Clazz.defineMethod (java.util.Hashtable, "clear", 
function () {
var tab = this.table;
this.modCount++;
for (var index = tab.length; --index >= 0; ) tab[index] = null;

this.count = 0;
});
Clazz.defineMethod (java.util.Hashtable, "clone", 
function () {
try {
var t = Clazz.superCall (this, java.util.Hashtable, "clone", []);
t.table =  new Array (this.table.length);
for (var i = this.table.length; i-- > 0; ) {
t.table[i] = (this.table[i] != null) ? this.table[i].clone () : null;
}
t.keySet = null;
t.entrySet = null;
t.values = null;
t.modCount = 0;
return t;
} catch (e) {
if (Clazz.instanceOf (e, CloneNotSupportedException)) {
throw  new InternalError ();
}
}

});
Clazz.defineMethod (java.util.Hashtable, "toString", 
function () {
var max = this.size () - 1;
var buf =  new StringBuffer ();
var it = this.entrySet ().iterator ();
buf.append ("{");
for (var i = 0; i <= max; i++) {
var e = (it.next ());
var key = e.getKey ();
var value = e.getValue ();
buf.append ((key == this ? "(this Map)" : key) + "=" + (value == this ? "(this Map)" : value));
if (i < max) buf.append (", ");
}
buf.append ("}");
return buf.toString ();
});
Clazz.defineMethod (java.util.Hashtable, "getEnumeration", 
function (type) {
if (this.count == 0) {
return java.util.Hashtable.emptyEnumerator;
} else {
return Clazz.innerTypeInstance (java.util.Hashtable.Enumerator, this, type, false);
}}, "Number");
Clazz.defineMethod (java.util.Hashtable, "getIterator", 
function (type) {
if (this.count == 0) {
return java.util.Hashtable.emptyIterator;
} else {
return Clazz.innerTypeInstance (java.util.Hashtable.Enumerator, this, type, true);
}}, "Number");
this.$keySet = null;
this.$entrySet = null;
this.$values = null;
Clazz.defineMethod (java.util.Hashtable, "keySet", 
function () {
if (this.$keySet == null) this.$keySet = java.util.Collections.synchronizedSet (Clazz.innerTypeInstance (java.util.Hashtable.KeySet, this), this);
return this.$keySet;
});
if (!Clazz.isClassDefined ("java.util.Hashtable.KeySet")) {
java.util.Hashtable.KeySet = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.Hashtable.KeySet, "iterator", 
function () {
return this.callbacks["java.util.Hashtable"].getIterator (java.util.Hashtable.KEYS);
});
Clazz.defineMethod (java.util.Hashtable.KeySet, "size", 
function () {
return this.callbacks["java.util.Hashtable"].count;
});
Clazz.defineMethod (java.util.Hashtable.KeySet, "contains", 
function (o) {
return this.callbacks["java.util.Hashtable"].containsKey (o);
}, "Object");
Clazz.defineMethod (java.util.Hashtable.KeySet, "remove", 
function (o) {
return this.callbacks["java.util.Hashtable"].remove (o) != null;
}, "Object");
Clazz.defineMethod (java.util.Hashtable.KeySet, "clear", 
function () {
this.callbacks["java.util.Hashtable"].clear ();
});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.KeySet.__CLASS_NAME__ = java.util.Hashtable.KeySet.prototype.__CLASS_NAME__ = "java.util.Hashtable.KeySet";
Clazz.inheritClass (java.util.Hashtable.KeySet, java.util.AbstractSet);
}
Clazz.defineMethod (java.util.Hashtable, "entrySet", 
function () {
if (this.$entrySet == null) this.$entrySet = java.util.Collections.synchronizedSet (Clazz.innerTypeInstance (java.util.Hashtable.EntrySet, this), this);
return this.$entrySet;
});
if (!Clazz.isClassDefined ("java.util.Hashtable.EntrySet")) {
java.util.Hashtable.EntrySet = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.Hashtable.EntrySet, "iterator", 
function () {
return this.callbacks["java.util.Hashtable"].getIterator (java.util.Hashtable.ENTRIES);
});
Clazz.defineMethod (java.util.Hashtable.EntrySet, "contains", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return false;
var entry = o;
var key = entry.getKey ();
var tab = this.callbacks["java.util.Hashtable"].table;
var hash = key.hashCode ();
var index = (hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index]; e != null; e = e.next) if (e.hash == hash && e.equals (entry)) return true;

return false;
}, "Object");
Clazz.defineMethod (java.util.Hashtable.EntrySet, "remove", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return false;
var entry = o;
var key = entry.getKey ();
var tab = this.callbacks["java.util.Hashtable"].table;
var hash = key.hashCode ();
var index = (hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index], prev = null; e != null; prev = e, e = e.next) {
if (e.hash == hash && e.equals (entry)) {
this.callbacks["java.util.Hashtable"].modCount++;
if (prev != null) prev.next = e.next;
 else tab[index] = e.next;
this.callbacks["java.util.Hashtable"].count--;
e.value = null;
return true;
}}
return false;
}, "Object");
Clazz.defineMethod (java.util.Hashtable.EntrySet, "size", 
function () {
return this.callbacks["java.util.Hashtable"].count;
});
Clazz.defineMethod (java.util.Hashtable.EntrySet, "clear", 
function () {
this.callbacks["java.util.Hashtable"].clear ();
});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.EntrySet.__CLASS_NAME__ = java.util.Hashtable.EntrySet.prototype.__CLASS_NAME__ = "java.util.Hashtable.EntrySet";
Clazz.inheritClass (java.util.Hashtable.EntrySet, java.util.AbstractSet);
}
Clazz.defineMethod (java.util.Hashtable, "values", 
function () {
if (this.$values == null) this.$values = java.util.Collections.synchronizedCollection (Clazz.innerTypeInstance (java.util.Hashtable.ValueCollection, this), this);
return this.$values;
});
if (!Clazz.isClassDefined ("java.util.Hashtable.ValueCollection")) {
java.util.Hashtable.ValueCollection = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
Clazz.defineMethod (java.util.Hashtable.ValueCollection, "iterator", 
function () {
return this.callbacks["java.util.Hashtable"].getIterator (java.util.Hashtable.VALUES);
});
Clazz.defineMethod (java.util.Hashtable.ValueCollection, "size", 
function () {
return this.callbacks["java.util.Hashtable"].count;
});
Clazz.defineMethod (java.util.Hashtable.ValueCollection, "contains", 
function (o) {
return this.callbacks["java.util.Hashtable"].containsValue (o);
}, "Object");
Clazz.defineMethod (java.util.Hashtable.ValueCollection, "clear", 
function () {
this.callbacks["java.util.Hashtable"].clear ();
});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.ValueCollection.__CLASS_NAME__ = java.util.Hashtable.ValueCollection.prototype.__CLASS_NAME__ = "java.util.Hashtable.ValueCollection";
Clazz.inheritClass (java.util.Hashtable.ValueCollection, java.util.AbstractCollection);
}
Clazz.defineMethod (java.util.Hashtable, "equals", 
function (o) {
if (o == this) return true;
if (!(Clazz.instanceOf (o, java.util.Map))) return false;
var t = o;
if (t.size () != this.size ()) return false;
try {
var i = this.entrySet ().iterator ();
while (i.hasNext ()) {
var e = i.next ();
var key = e.getKey ();
var value = e.getValue ();
if (value == null) {
if (!(t.get (key) == null && t.containsKey (key))) return false;
} else {
if (!value.equals (t.get (key))) return false;
}}
} catch (e) {
if (Clazz.instanceOf (e, ClassCastException)) {
return false;
} else if (Clazz.instanceOf (e, NullPointerException)) {
return false;
}
}

return true;
}, "Object");
Clazz.defineMethod (java.util.Hashtable, "hashCode", 
function () {
var h = 0;
if (this.count == 0 || this.loadFactor < 0) return h;
this.loadFactor = -this.loadFactor;
var tab = this.table;
for (var i = 0; i < tab.length; i++) for (var e = tab[i]; e != null; e = e.next) h += e.key.hashCode () ^ e.value.hashCode ();


this.loadFactor = -this.loadFactor;
return h;
});
Clazz.defineMethod (java.util.Hashtable, "writeObject", 
function (s) {
s.defaultWriteObject ();
s.writeInt (this.table.length);
s.writeInt (this.count);
for (var index = this.table.length - 1; index >= 0; index--) {
var entry = this.table[index];
while (entry != null) {
s.writeObject (entry.key);
s.writeObject (entry.value);
entry = entry.next;
}
}
}, "java.io.ObjectOutputStream");
Clazz.defineMethod (java.util.Hashtable, "readObject", 
function (s) {
s.defaultReadObject ();
var origlength = s.readInt ();
var elements = s.readInt ();
var length = parseInt ((elements * this.loadFactor)) + (Math.floor (elements / 20)) + 3;
if (length > elements && (length & 1) == 0) length--;
if (origlength > 0 && length > origlength) length = origlength;
this.table =  new Array (length);
this.count = 0;
for (; elements > 0; elements--) {
var key = s.readObject ();
var value = s.readObject ();
this.put (key, value);
}
}, "java.io.ObjectInputStream");
this.KEYS = java.util.Hashtable.KEYS;
this.VALUES = java.util.Hashtable.VALUES;
this.ENTRIES = java.util.Hashtable.ENTRIES;
if (!Clazz.isClassDefined ("java.util.Hashtable.Enumerator")) {
java.util.Hashtable.Enumerator = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.table = this.callbacks["java.util.Hashtable"].table;
this.index = this.table.length;
this.entry = null;
this.lastReturned = null;
this.type = 0;
this.iterator = false;
this.expectedModCount = this.callbacks["java.util.Hashtable"].modCount;
Clazz.makeConstructor (java.util.Hashtable.Enumerator, 
function (type, iterator) {
this.type = type;
this.iterator = iterator;
}, "Number, Boolean");
Clazz.defineMethod (java.util.Hashtable.Enumerator, "hasMoreElements", 
function () {
var e = this.entry;
var i = this.index;
var t = this.table;
while (e == null && i > 0) {
e = t[--i];
}
this.entry = e;
this.index = i;
return e != null;
});
Clazz.defineMethod (java.util.Hashtable.Enumerator, "nextElement", 
function () {
var et = this.entry;
var i = this.index;
var t = this.table;
while (et == null && i > 0) {
et = t[--i];
}
this.entry = et;
this.index = i;
if (et != null) {
var e = this.lastReturned = this.entry;
this.entry = e.next;
return this.type == java.util.Hashtable.KEYS ? e.key : (this.type == java.util.Hashtable.VALUES ? e.value : e);
}throw  new java.util.NoSuchElementException ("Hashtable Enumerator");
});
Clazz.defineMethod (java.util.Hashtable.Enumerator, "hasNext", 
function () {
return this.hasMoreElements ();
});
Clazz.defineMethod (java.util.Hashtable.Enumerator, "next", 
function () {
if (this.callbacks["java.util.Hashtable"].modCount != this.expectedModCount) throw  new java.util.ConcurrentModificationException ();
return this.nextElement ();
});
Clazz.defineMethod (java.util.Hashtable.Enumerator, "remove", 
function () {
if (!this.iterator) throw  new UnsupportedOperationException ();
if (this.lastReturned == null) throw  new IllegalStateException ("Hashtable Enumerator");
if (this.callbacks["java.util.Hashtable"].modCount != this.expectedModCount) throw  new java.util.ConcurrentModificationException ();
{
var tab = this.callbacks["java.util.Hashtable"].table;
var index = (this.lastReturned.hash & 0x7FFFFFFF) % tab.length;
for (var e = tab[index], prev = null; e != null; prev = e, e = e.next) {
if (e == this.lastReturned) {
this.callbacks["java.util.Hashtable"].modCount++;
this.expectedModCount++;
if (prev == null) tab[index] = e.next;
 else prev.next = e.next;
this.callbacks["java.util.Hashtable"].count--;
this.lastReturned = null;
return ;
}}
throw  new java.util.ConcurrentModificationException ();
}});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.Enumerator.__CLASS_NAME__ = java.util.Hashtable.Enumerator.prototype.__CLASS_NAME__ = "java.util.Hashtable.Enumerator";
Clazz.implementOf (java.util.Hashtable.Enumerator, java.util.Enumeration);
Clazz.implementOf (java.util.Hashtable.Enumerator, java.util.Iterator);
}
this.emptyEnumerator = java.util.Hashtable.emptyEnumerator;
this.emptyIterator = java.util.Hashtable.emptyIterator;
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.__CLASS_NAME__ = java.util.Hashtable.prototype.__CLASS_NAME__ = "java.util.Hashtable";
Clazz.inheritClass (java.util.Hashtable, java.util.Dictionary);
Clazz.implementOf (java.util.Hashtable, java.util.Map);
Clazz.implementOf (java.util.Hashtable, Cloneable);
Clazz.implementOf (java.util.Hashtable, java.io.Serializable);
java.util.Hashtable.Entry = function () {
this.hash = 0;
this.key = null;
this.value = null;
this.next = null;
Clazz.makeConstructor (java.util.Hashtable.Entry, 
function (hash, key, value, next) {
this.hash = hash;
this.key = key;
this.value = value;
this.next = next;
}, "Number, Object, Object, java.util.Hashtable.Entry");
Clazz.defineMethod (java.util.Hashtable.Entry, "clone", 
function () {
return  new java.util.Hashtable.Entry (this.hash, this.key, this.value, (this.next == null ? null : this.next.clone ()));
});
Clazz.defineMethod (java.util.Hashtable.Entry, "getKey", 
function () {
return this.key;
});
Clazz.defineMethod (java.util.Hashtable.Entry, "getValue", 
function () {
return this.value;
});
Clazz.defineMethod (java.util.Hashtable.Entry, "setValue", 
function (value) {
if (value == null) throw  new NullPointerException ();
var oldValue = this.value;
this.value = value;
return oldValue;
}, "Object");
Clazz.defineMethod (java.util.Hashtable.Entry, "equals", 
function (o) {
if (!(Clazz.instanceOf (o, java.util.Map.Entry))) return false;
var e = o;
return (this.key == null ? e.getKey () == null : this.key.equals (e.getKey ())) && (this.value == null ? e.getValue () == null : this.value.equals (e.getValue ()));
}, "Object");
Clazz.defineMethod (java.util.Hashtable.Entry, "hashCode", 
function () {
return this.hash ^ (this.value == null ? 0 : this.value.hashCode ());
});
Clazz.defineMethod (java.util.Hashtable.Entry, "toString", 
function () {
return this.key.toString () + "=" + this.value.toString ();
});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.Entry.__CLASS_NAME__ = java.util.Hashtable.Entry.prototype.__CLASS_NAME__ = "java.util.Hashtable.Entry";
Clazz.implementOf (java.util.Hashtable.Entry, java.util.Map.Entry);
java.util.Hashtable.EmptyEnumerator = function () {
Clazz.makeConstructor (java.util.Hashtable.EmptyEnumerator, 
function () {
});
Clazz.defineMethod (java.util.Hashtable.EmptyEnumerator, "hasMoreElements", 
function () {
return false;
});
Clazz.defineMethod (java.util.Hashtable.EmptyEnumerator, "nextElement", 
function () {
throw  new java.util.NoSuchElementException ("Hashtable Enumerator");
});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.EmptyEnumerator.__CLASS_NAME__ = java.util.Hashtable.EmptyEnumerator.prototype.__CLASS_NAME__ = "java.util.Hashtable.EmptyEnumerator";
Clazz.implementOf (java.util.Hashtable.EmptyEnumerator, java.util.Enumeration);
java.util.Hashtable.EmptyIterator = function () {
Clazz.makeConstructor (java.util.Hashtable.EmptyIterator, 
function () {
});
Clazz.defineMethod (java.util.Hashtable.EmptyIterator, "hasNext", 
function () {
return false;
});
Clazz.defineMethod (java.util.Hashtable.EmptyIterator, "next", 
function () {
throw  new java.util.NoSuchElementException ("Hashtable Iterator");
});
Clazz.defineMethod (java.util.Hashtable.EmptyIterator, "remove", 
function () {
throw  new IllegalStateException ("Hashtable Iterator");
});
Clazz.instantialize (this, arguments);
};
java.util.Hashtable.EmptyIterator.__CLASS_NAME__ = java.util.Hashtable.EmptyIterator.prototype.__CLASS_NAME__ = "java.util.Hashtable.EmptyIterator";
Clazz.implementOf (java.util.Hashtable.EmptyIterator, java.util.Iterator);
java.util.Hashtable.serialVersionUID = 1421746759512286392;
java.util.Hashtable.KEYS = 0;
java.util.Hashtable.VALUES = 1;
java.util.Hashtable.ENTRIES = 2;
java.util.Hashtable.emptyEnumerator =  new java.util.Hashtable.EmptyEnumerator ();
java.util.Hashtable.emptyIterator =  new java.util.Hashtable.EmptyIterator ();
Clazz.declarePackage ("java.util");
java.util.Properties = function () {
Clazz.stackSuperField (this, java.util.Properties, "serialVersionUID");
this.serialVersionUID = java.util.Properties.serialVersionUID;
this.defaults = null;
Clazz.makeConstructor (java.util.Properties, 
function () {
Clazz.superConstructor (this, java.util.Properties, []);
this.construct (null);
});
Clazz.makeConstructor (java.util.Properties, 
function (defaults) {
Clazz.superConstructor (this, java.util.Properties, []);
this.defaults = defaults;
}, "java.util.Properties");
Clazz.defineMethod (java.util.Properties, "setProperty", 
function (key, value) {
return this.put (key, value);
}, "String, String");
this.keyValueSeparators = java.util.Properties.keyValueSeparators;
this.strictKeyValueSeparators = java.util.Properties.strictKeyValueSeparators;
this.specialSaveChars = java.util.Properties.specialSaveChars;
this.whiteSpaceChars = java.util.Properties.whiteSpaceChars;
Clazz.defineMethod (java.util.Properties, "load", 
function (inStream) {
var $in =  new java.io.BufferedReader ( new java.io.InputStreamReader (inStream, "8859_1"));
while (true) {
var line = $in.readLine ();
if (line == null) return ;
if (line.length > 0) {
var len = line.length;
var keyStart;
for (keyStart = 0; keyStart < len; keyStart++) if (java.util.Properties.whiteSpaceChars.indexOf (line.charAt (keyStart)) == -1) break;

if (keyStart == len) continue ;var firstChar = line.charAt (keyStart);
if (((firstChar).charCodeAt (0) != ('#').charCodeAt (0)) && ((firstChar).charCodeAt (0) != ('!').charCodeAt (0))) {
while (this.continueLine (line)) {
var nextLine = $in.readLine ();
if (nextLine == null) nextLine = "";
var loppedLine = line.substring (0, len - 1);
var startIndex;
for (startIndex = 0; startIndex < nextLine.length; startIndex++) if (java.util.Properties.whiteSpaceChars.indexOf (nextLine.charAt (startIndex)) == -1) break;

nextLine = nextLine.substring (startIndex, nextLine.length);
line =  String.instantialize (loppedLine + nextLine);
len = line.length;
}
var separatorIndex;
for (separatorIndex = keyStart; separatorIndex < len; separatorIndex++) {
var currentChar = line.charAt (separatorIndex);
if ((currentChar).charCodeAt (0) == ('\\').charCodeAt (0)) separatorIndex++;
 else if (java.util.Properties.keyValueSeparators.indexOf (currentChar) != -1) break;
}
var valueIndex;
for (valueIndex = separatorIndex; valueIndex < len; valueIndex++) if (java.util.Properties.whiteSpaceChars.indexOf (line.charAt (valueIndex)) == -1) break;

if (valueIndex < len) if (java.util.Properties.strictKeyValueSeparators.indexOf (line.charAt (valueIndex)) != -1) valueIndex++;
while (valueIndex < len) {
if (java.util.Properties.whiteSpaceChars.indexOf (line.charAt (valueIndex)) == -1) break;
valueIndex++;
}
var key = line.substring (keyStart, separatorIndex);
var value = (separatorIndex < len) ? line.substring (valueIndex, len) : "";
key = this.loadConvert (key);
value = this.loadConvert (value);
this.put (key, value);
}}}
}, "java.io.InputStream");
Clazz.defineMethod (java.util.Properties, "continueLine", 
function (line) {
var slashCount = 0;
var index = line.length - 1;
while ((index >= 0) && ((line.charAt (index--)).charCodeAt (0) == ('\\').charCodeAt (0))) slashCount++;

return (slashCount % 2 == 1);
}, "String");
Clazz.defineMethod (java.util.Properties, "loadConvert", 
function (theString) {
var aChar;
var len = theString.length;
var outBuffer =  new StringBuffer (len);
for (var x = 0; x < len; ) {
aChar = theString.charAt (x++);
if ((aChar).charCodeAt (0) == ('\\').charCodeAt (0)) {
aChar = theString.charAt (x++);
if ((aChar).charCodeAt (0) == ('u').charCodeAt (0)) {
var value = 0;
for (var i = 0; i < 4; i++) {
aChar = theString.charAt (x++);
switch (aChar) {
case '0':
case '1':
case '2':
case '3':
case '4':
case '5':
case '6':
case '7':
case '8':
case '9':
value = (value << 4) + (aChar).charCodeAt (0) - ('0').charCodeAt (0);
break;
case 'a':
case 'b':
case 'c':
case 'd':
case 'e':
case 'f':
value = (value << 4) + 10 + (aChar).charCodeAt (0) - ('a').charCodeAt (0);
break;
case 'A':
case 'B':
case 'C':
case 'D':
case 'E':
case 'F':
value = (value << 4) + 10 + (aChar).charCodeAt (0) - ('A').charCodeAt (0);
break;
default:
throw  new IllegalArgumentException ("Malformed \\uxxxx encoding.");
}
}
outBuffer.append (String.fromCharCode (value));
} else {
if ((aChar).charCodeAt (0) == ('t').charCodeAt (0)) aChar = '\t';
 else if ((aChar).charCodeAt (0) == ('r').charCodeAt (0)) aChar = '\r';
 else if ((aChar).charCodeAt (0) == ('n').charCodeAt (0)) aChar = '\n';
 else if ((aChar).charCodeAt (0) == ('f').charCodeAt (0)) aChar = '\f';
outBuffer.append (aChar);
}} else outBuffer.append (aChar);
}
return outBuffer.toString ();
}, "String");
Clazz.defineMethod (java.util.Properties, "saveConvert", 
function (theString, escapeSpace) {
var len = theString.length;
var outBuffer =  new StringBuffer (len * 2);
for (var x = 0; x < len; x++) {
var aChar = theString.charAt (x);
switch (aChar) {
case ' ':
if (x == 0 || escapeSpace) outBuffer.append ('\\');
outBuffer.append (' ');
break;
case '\\':
outBuffer.append ('\\');
outBuffer.append ('\\');
break;
case '\t':
outBuffer.append ('\\');
outBuffer.append ('t');
break;
case '\n':
outBuffer.append ('\\');
outBuffer.append ('n');
break;
case '\r':
outBuffer.append ('\\');
outBuffer.append ('r');
break;
case '\f':
outBuffer.append ('\\');
outBuffer.append ('f');
break;
default:
if (((aChar).charCodeAt (0) < 0x0020) || ((aChar).charCodeAt (0) > 0x007e)) {
outBuffer.append ('\\');
outBuffer.append ('u');
outBuffer.append (java.util.Properties.toHex (((aChar).charCodeAt (0) >> 12) & 0xF));
outBuffer.append (java.util.Properties.toHex (((aChar).charCodeAt (0) >> 8) & 0xF));
outBuffer.append (java.util.Properties.toHex (((aChar).charCodeAt (0) >> 4) & 0xF));
outBuffer.append (java.util.Properties.toHex ((aChar).charCodeAt (0) & 0xF));
} else {
if (java.util.Properties.specialSaveChars.indexOf (aChar) != -1) outBuffer.append ('\\');
outBuffer.append (aChar);
}}
}
return outBuffer.toString ();
}, "String, Boolean");
Clazz.defineMethod (java.util.Properties, "save", 
function (out, header) {
try {
this.store (out, header);
} catch (e) {
if (Clazz.instanceOf (e, java.io.IOException)) {
}
}

}, "java.io.OutputStream, String");
Clazz.defineMethod (java.util.Properties, "store", 
function (out, header) {
var awriter;
awriter =  new java.io.BufferedWriter ( new java.io.OutputStreamWriter (out, "8859_1"));
if (header != null) java.util.Properties.writeln (awriter, "#" + header);
java.util.Properties.writeln (awriter, "#" +  new java.util.Date ().toString ());
for (var e = this.keys (); e.hasMoreElements (); ) {
var key = e.nextElement ();
var val = this.get (key);
key = this.saveConvert (key, true);
val = this.saveConvert (val, false);
java.util.Properties.writeln (awriter, key + "=" + val);
}
awriter.flush ();
}, "java.io.OutputStream, String");
Clazz.defineMethod (java.util.Properties, "getProperty", 
function (key) {
var oval = Clazz.superCall (this, java.util.Properties, "get", [key]);
var sval = (Clazz.instanceOf (oval, String)) ? oval : null;
return ((sval == null) && (this.defaults != null)) ? this.defaults.getProperty (key) : sval;
}, "String");
Clazz.defineMethod (java.util.Properties, "getProperty", 
function (key, defaultValue) {
var val = this.getProperty (key);
return (val == null) ? defaultValue : val;
}, "String, String");
Clazz.defineMethod (java.util.Properties, "propertyNames", 
function () {
var h =  new java.util.Hashtable ();
this.enumerate (h);
return h.keys ();
});
Clazz.defineMethod (java.util.Properties, "list", 
function (out) {
out.println ("-- listing properties --");
var h =  new java.util.Hashtable ();
this.enumerate (h);
for (var e = h.keys (); e.hasMoreElements (); ) {
var key = e.nextElement ();
var val = h.get (key);
if (val.length > 40) {
val = val.substring (0, 37) + "...";
}out.println (key + "=" + val);
}
}, "java.io.PrintStream");
Clazz.defineMethod (java.util.Properties, "list", 
function (out) {
out.println ("-- listing properties --");
var h =  new java.util.Hashtable ();
this.enumerate (h);
for (var e = h.keys (); e.hasMoreElements (); ) {
var key = e.nextElement ();
var val = h.get (key);
if (val.length > 40) {
val = val.substring (0, 37) + "...";
}out.println (key + "=" + val);
}
}, "java.io.PrintWriter");
Clazz.defineMethod (java.util.Properties, "enumerate", 
function (h) {
if (this.defaults != null) {
this.defaults.enumerate (h);
}for (var e = this.keys (); e.hasMoreElements (); ) {
var key = e.nextElement ();
h.put (key, this.get (key));
}
}, "java.util.Hashtable");
this.hexDigit = java.util.Properties.hexDigit;
Clazz.instantialize (this, arguments);
};
java.util.Properties.__CLASS_NAME__ = java.util.Properties.prototype.__CLASS_NAME__ = "java.util.Properties";
Clazz.inheritClass (java.util.Properties, java.util.Hashtable);
java.util.Properties.serialVersionUID = 4112578634029874840;
java.util.Properties.keyValueSeparators = "=: \t\r\n\f";
java.util.Properties.strictKeyValueSeparators = "=:";
java.util.Properties.specialSaveChars = "=: \t\r\n\f#!";
java.util.Properties.whiteSpaceChars = " \t\r\n\f";
java.util.Properties.hexDigit = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'];
Clazz.defineMethod (java.util.Properties, "writeln", 
function (bw, s) {
bw.write (s);
bw.newLine ();
}, "java.io.BufferedWriter, String");
java.util.Properties.writeln = java.util.Properties.prototype.writeln;
Clazz.defineMethod (java.util.Properties, "toHex", 
function (nibble) {
return java.util.Properties.hexDigit[(nibble & 0xF)];
}, "Number");
java.util.Properties.toHex = java.util.Properties.prototype.toHex;
Clazz.declarePackage ("java.util");
java.util.Vector = function () {
this.elementData = null;
this.elementCount = 0;
this.capacityIncrement = 0;
this.serialVersionUID = java.util.Vector.serialVersionUID;
Clazz.makeConstructor (java.util.Vector, 
function (initialCapacity, capacityIncrement) {
Clazz.superConstructor (this, java.util.Vector);
if (initialCapacity < 0) throw  new IllegalArgumentException ("Illegal Capacity: " + initialCapacity);
this.elementData =  new Array (initialCapacity);
this.capacityIncrement = capacityIncrement;
}, "Number, Number");
Clazz.makeConstructor (java.util.Vector, 
function (initialCapacity) {
Clazz.superConstructor (this, java.util.Vector, []);
this.construct (initialCapacity, 0);
}, "Number");
Clazz.makeConstructor (java.util.Vector, 
function () {
Clazz.superConstructor (this, java.util.Vector, []);
this.construct (10);
});
Clazz.makeConstructor (java.util.Vector, 
function (c) {
Clazz.superConstructor (this, java.util.Vector, []);
this.elementCount = c.size ();
this.elementData =  new Array (parseInt (Math.min (Math.floor ((this.elementCount * 110) / 100), Integer.MAX_VALUE)));
c.toArray (this.elementData);
}, "java.util.Collection");
Clazz.defineMethod (java.util.Vector, "copyInto", 
function (anArray) {
System.arraycopy (this.elementData, 0, anArray, 0, this.elementCount);
}, "Array");
Clazz.defineMethod (java.util.Vector, "trimToSize", 
function () {
this.modCount++;
var oldCapacity = this.elementData.length;
if (this.elementCount < oldCapacity) {
var oldData = this.elementData;
this.elementData =  new Array (this.elementCount);
System.arraycopy (oldData, 0, this.elementData, 0, this.elementCount);
}});
Clazz.defineMethod (java.util.Vector, "ensureCapacity", 
function (minCapacity) {
this.modCount++;
this.ensureCapacityHelper (minCapacity);
}, "Number");
Clazz.defineMethod (java.util.Vector, "ensureCapacityHelper", 
function (minCapacity) {
var oldCapacity = this.elementData.length;
if (minCapacity > oldCapacity) {
var oldData = this.elementData;
var newCapacity = (this.capacityIncrement > 0) ? (oldCapacity + this.capacityIncrement) : (oldCapacity * 2);
if (newCapacity < minCapacity) {
newCapacity = minCapacity;
}this.elementData =  new Array (newCapacity);
System.arraycopy (oldData, 0, this.elementData, 0, this.elementCount);
}}, "Number");
Clazz.defineMethod (java.util.Vector, "setSize", 
function (newSize) {
this.modCount++;
if (newSize > this.elementCount) {
this.ensureCapacityHelper (newSize);
} else {
for (var i = newSize; i < this.elementCount; i++) {
this.elementData[i] = null;
}
}this.elementCount = newSize;
}, "Number");
Clazz.defineMethod (java.util.Vector, "capacity", 
function () {
return this.elementData.length;
});
Clazz.defineMethod (java.util.Vector, "size", 
function () {
return this.elementCount;
});
Clazz.defineMethod (java.util.Vector, "isEmpty", 
function () {
return this.elementCount == 0;
});
Clazz.defineMethod (java.util.Vector, "elements", 
function () {
return (function (innerThis) {
if (!Clazz.isClassDefined ("java.util.Vector$1")) {
java.util.Vector$1 = function () {
Clazz.prepareCallback (this, arguments[0]);
for (var i = 0; i < arguments.length - 1; i++) {
arguments[i] = arguments[i + 1];
}
arguments.length--;
this.count = 0;
Clazz.defineMethod (java.util.Vector$1, "hasMoreElements", 
function () {
return this.count < this.callbacks["java.util.Vector"].elementCount;
});
Clazz.defineMethod (java.util.Vector$1, "nextElement", 
function () {
{
if (this.count < this.callbacks["java.util.Vector"].elementCount) {
return this.callbacks["java.util.Vector"].elementData[this.count++];
}}throw  new java.util.NoSuchElementException ("Vector Enumeration");
});
Clazz.instantialize (this, arguments);
};
Clazz.implementOf (java.util.Vector$1, java.util.Enumeration);
java.util.Vector$1.__CLASS_NAME__ = java.util.Vector$1.prototype.__CLASS_NAME__ = "java.util.Vector$1";
}
return Clazz.innerTypeInstance (java.util.Vector$1, innerThis);
}) (this);
});
Clazz.defineMethod (java.util.Vector, "contains", 
function (elem) {
return this.indexOf (elem, 0) >= 0;
}, "Object");
Clazz.defineMethod (java.util.Vector, "indexOf", 
function (elem) {
return this.indexOf (elem, 0);
}, "Object");
Clazz.defineMethod (java.util.Vector, "indexOf", 
function (elem, index) {
if (elem == null) {
for (var i = index; i < this.elementCount; i++) if (this.elementData[i] == null) return i;

} else {
for (var i = index; i < this.elementCount; i++) if (elem.equals (this.elementData[i])) return i;

}return -1;
}, "Object, Number");
Clazz.defineMethod (java.util.Vector, "lastIndexOf", 
function (elem) {
return this.lastIndexOf (elem, this.elementCount - 1);
}, "Object");
Clazz.defineMethod (java.util.Vector, "lastIndexOf", 
function (elem, index) {
if (index >= this.elementCount) throw  new IndexOutOfBoundsException (index + " >= " + this.elementCount);
if (elem == null) {
for (var i = index; i >= 0; i--) if (this.elementData[i] == null) return i;

} else {
for (var i = index; i >= 0; i--) if (elem.equals (this.elementData[i])) return i;

}return -1;
}, "Object, Number");
Clazz.defineMethod (java.util.Vector, "elementAt", 
function (index) {
if (index >= this.elementCount) {
throw  new ArrayIndexOutOfBoundsException (index + " >= " + this.elementCount);
}return this.elementData[index];
}, "Number");
Clazz.defineMethod (java.util.Vector, "firstElement", 
function () {
if (this.elementCount == 0) {
throw  new java.util.NoSuchElementException ();
}return this.elementData[0];
});
Clazz.defineMethod (java.util.Vector, "lastElement", 
function () {
if (this.elementCount == 0) {
throw  new java.util.NoSuchElementException ();
}return this.elementData[this.elementCount - 1];
});
Clazz.defineMethod (java.util.Vector, "setElementAt", 
function (obj, index) {
if (index >= this.elementCount) {
throw  new ArrayIndexOutOfBoundsException (index + " >= " + this.elementCount);
}this.elementData[index] = obj;
}, "Object, Number");
Clazz.defineMethod (java.util.Vector, "removeElementAt", 
function (index) {
this.modCount++;
if (index >= this.elementCount) {
throw  new ArrayIndexOutOfBoundsException (index + " >= " + this.elementCount);
} else if (index < 0) {
throw  new ArrayIndexOutOfBoundsException (index);
}var j = this.elementCount - index - 1;
if (j > 0) {
System.arraycopy (this.elementData, index + 1, this.elementData, index, j);
}this.elementCount--;
this.elementData[this.elementCount] = null;
}, "Number");
Clazz.defineMethod (java.util.Vector, "insertElementAt", 
function (obj, index) {
this.modCount++;
if (index > this.elementCount) {
throw  new ArrayIndexOutOfBoundsException (index + " > " + this.elementCount);
}this.ensureCapacityHelper (this.elementCount + 1);
System.arraycopy (this.elementData, index, this.elementData, index + 1, this.elementCount - index);
this.elementData[index] = obj;
this.elementCount++;
}, "Object, Number");
Clazz.defineMethod (java.util.Vector, "addElement", 
function (obj) {
this.modCount++;
this.ensureCapacityHelper (this.elementCount + 1);
this.elementData[this.elementCount++] = obj;
}, "Object");
Clazz.defineMethod (java.util.Vector, "removeElement", 
function (obj) {
this.modCount++;
var i = this.indexOf (obj);
if (i >= 0) {
this.removeElementAt (i);
return true;
}return false;
}, "Object");
Clazz.defineMethod (java.util.Vector, "removeAllElements", 
function () {
this.modCount++;
for (var i = 0; i < this.elementCount; i++) this.elementData[i] = null;

this.elementCount = 0;
});
Clazz.defineMethod (java.util.Vector, "clone", 
function () {
try {
var v = Clazz.superCall (this, java.util.Vector, "clone", []);
v.elementData =  new Array (this.elementCount);
System.arraycopy (this.elementData, 0, v.elementData, 0, this.elementCount);
v.modCount = 0;
return v;
} catch (e) {
if (Clazz.instanceOf (e, CloneNotSupportedException)) {
throw  new InternalError ();
}
}

});
Clazz.defineMethod (java.util.Vector, "toArray", 
function () {
var result =  new Array (this.elementCount);
System.arraycopy (this.elementData, 0, result, 0, this.elementCount);
return result;
});
Clazz.defineMethod (java.util.Vector, "toArray", 
function (a) {
if (a.length < this.elementCount) a = java.lang.reflect.Array.newInstance (a.getClass ().getComponentType (), this.elementCount);
System.arraycopy (this.elementData, 0, a, 0, this.elementCount);
if (a.length > this.elementCount) a[this.elementCount] = null;
return a;
}, "Array");
Clazz.defineMethod (java.util.Vector, "get", 
function (index) {
if (index >= this.elementCount) throw  new ArrayIndexOutOfBoundsException (index);
return this.elementData[index];
}, "Number");
Clazz.defineMethod (java.util.Vector, "set", 
function (index, element) {
if (index >= this.elementCount) throw  new ArrayIndexOutOfBoundsException (index);
var oldValue = this.elementData[index];
this.elementData[index] = element;
return oldValue;
}, "Number, Object");
Clazz.defineMethod (java.util.Vector, "add", 
function (o) {
this.modCount++;
this.ensureCapacityHelper (this.elementCount + 1);
this.elementData[this.elementCount++] = o;
return true;
}, "Object");
Clazz.defineMethod (java.util.Vector, "remove", 
function (o) {
return this.removeElement (o);
}, "Object");
Clazz.defineMethod (java.util.Vector, "add", 
function (index, element) {
this.insertElementAt (element, index);
}, "Number, Object");
Clazz.defineMethod (java.util.Vector, "remove", 
function (index) {
this.modCount++;
if (index >= this.elementCount) throw  new ArrayIndexOutOfBoundsException (index);
var oldValue = this.elementData[index];
var numMoved = this.elementCount - index - 1;
if (numMoved > 0) System.arraycopy (this.elementData, index + 1, this.elementData, index, numMoved);
this.elementData[--this.elementCount] = null;
return oldValue;
}, "Number");
Clazz.defineMethod (java.util.Vector, "clear", 
function () {
this.removeAllElements ();
});
Clazz.defineMethod (java.util.Vector, "containsAll", 
function (c) {
return Clazz.superCall (this, java.util.Vector, "containsAll", [c]);
}, "java.util.Collection");
Clazz.defineMethod (java.util.Vector, "addAll", 
function (c) {
this.modCount++;
var a = c.toArray ();
var numNew = a.length;
this.ensureCapacityHelper (this.elementCount + numNew);
System.arraycopy (a, 0, this.elementData, this.elementCount, numNew);
this.elementCount += numNew;
return numNew != 0;
}, "java.util.Collection");
Clazz.defineMethod (java.util.Vector, "removeAll", 
function (c) {
return Clazz.superCall (this, java.util.Vector, "removeAll", [c]);
}, "java.util.Collection");
Clazz.defineMethod (java.util.Vector, "retainAll", 
function (c) {
return Clazz.superCall (this, java.util.Vector, "retainAll", [c]);
}, "java.util.Collection");
Clazz.defineMethod (java.util.Vector, "addAll", 
function (index, c) {
this.modCount++;
if (index < 0 || index > this.elementCount) throw  new ArrayIndexOutOfBoundsException (index);
var a = c.toArray ();
var numNew = a.length;
this.ensureCapacityHelper (this.elementCount + numNew);
var numMoved = this.elementCount - index;
if (numMoved > 0) System.arraycopy (this.elementData, index, this.elementData, index + numNew, numMoved);
System.arraycopy (a, 0, this.elementData, index, numNew);
this.elementCount += numNew;
return numNew != 0;
}, "Number, java.util.Collection");
Clazz.defineMethod (java.util.Vector, "equals", 
function (o) {
return Clazz.superCall (this, java.util.Vector, "equals", [o]);
}, "Object");
Clazz.defineMethod (java.util.Vector, "hashCode", 
function () {
return Clazz.superCall (this, java.util.Vector, "hashCode", []);
});
Clazz.defineMethod (java.util.Vector, "toString", 
function () {
return Clazz.superCall (this, java.util.Vector, "toString", []);
});
Clazz.defineMethod (java.util.Vector, "subList", 
function (fromIndex, toIndex) {
return java.util.Collections.synchronizedList (Clazz.superCall (this, java.util.Vector, "subList", [fromIndex, toIndex]), this);
}, "Number, Number");
Clazz.defineMethod (java.util.Vector, "removeRange", 
function (fromIndex, toIndex) {
this.modCount++;
var numMoved = this.elementCount - toIndex;
System.arraycopy (this.elementData, toIndex, this.elementData, fromIndex, numMoved);
var newElementCount = this.elementCount - (toIndex - fromIndex);
while (this.elementCount != newElementCount) this.elementData[--this.elementCount] = null;

}, "Number, Number");
Clazz.defineMethod (java.util.Vector, "writeObject", 
function (s) {
s.defaultWriteObject ();
}, "java.io.ObjectOutputStream");
Clazz.instantialize (this, arguments);
};
java.util.Vector.__CLASS_NAME__ = java.util.Vector.prototype.__CLASS_NAME__ = "java.util.Vector";
Clazz.inheritClass (java.util.Vector, java.util.AbstractList);
Clazz.implementOf (java.util.Vector, java.util.List);
Clazz.implementOf (java.util.Vector, java.util.RandomAccess);
Clazz.implementOf (java.util.Vector, Cloneable);
Clazz.implementOf (java.util.Vector, java.io.Serializable);
java.util.Vector.serialVersionUID = -2767605614048989439;
Clazz.declarePackage ("java.util");
java.util.Stack = function () {
Clazz.makeConstructor (java.util.Stack, 
function () {
Clazz.superConstructor (this, java.util.Stack, []);
});
Clazz.defineMethod (java.util.Stack, "push", 
function (item) {
this.addElement (item);
return item;
}, "Object");
Clazz.defineMethod (java.util.Stack, "pop", 
function () {
var obj;
var len = this.size ();
obj = this.peek ();
this.removeElementAt (len - 1);
return obj;
});
Clazz.defineMethod (java.util.Stack, "peek", 
function () {
var len = this.size ();
if (len == 0) throw  new java.util.EmptyStackException ();
return this.elementAt (len - 1);
});
Clazz.defineMethod (java.util.Stack, "empty", 
function () {
return this.size () == 0;
});
Clazz.defineMethod (java.util.Stack, "search", 
function (o) {
var i = this.lastIndexOf (o);
if (i >= 0) {
return this.size () - i;
}return -1;
}, "Object");
Clazz.stackSuperField (this, java.util.Stack, "serialVersionUID");
this.serialVersionUID = java.util.Stack.serialVersionUID;
Clazz.instantialize (this, arguments);
};
java.util.Stack.__CLASS_NAME__ = java.util.Stack.prototype.__CLASS_NAME__ = "java.util.Stack";
Clazz.inheritClass (java.util.Stack, java.util.Vector);
java.util.Stack.serialVersionUID = 1224463164541339165;
